
!----------------------------------------------------------
!              SPHERE part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: boundary_mod.f90,v 1.1 2001/08/11 15:38:36 ingves Exp ingves $
!----------------------------------------------------------



Module Boundary_mod


Use Global_definitions


Contains

!------------------------------------------------------------------
  Subroutine boundary_conditions(MPC,ienergy,int,int_tr1,zeta,param)
    implicit none
    Type(parameters) :: param 
    Type(integrals)  :: Int(0:,0:,0:), int_tr1(0:,0:,0:) 
    integer          :: ienergy
    Real(wp)         :: zeta(0:,0:,0:)
    complex(wp)      :: MPC(2*param%Mpole_order,0:1)

    if (param%above) then
       call boundary_conditions_above(MPC,ienergy,int,int_tr1,zeta,param)
    else
       call boundary_conditions_below(MPC,ienergy,int,int_tr1,zeta,param)
    end if
  end Subroutine boundary_conditions
    

!-------------------------------------------------------------------------------------------


  Subroutine boundary_conditions_above(MPC,ienergy,int,int_tr1,zeta,param)
    !-------------------------------------
    ! Multipoles above the surface
    !-------------------------------------
    Use Legendre, Only : asslegpoly, Gausslegendre
    
    implicit none
    Type(parameters) :: param 
    Type(integrals)  :: Int(0:,0:,0:), int_tr1(0:,0:,0:) 
    integer          :: ienergy
    Real(wp)         :: zeta(0:,0:,0:)
    complex(wpc)      :: MPC(2*param%Mpole_order,0:1)

    ! Parameters for the number of points on the sphere
    integer, parameter                :: Ntheta=180,Nphi=300
    ! Output files
    character(len=2),parameter        :: filename1='BC'
    character(len=4),parameter        :: filename2='NDBC'
    logical                           :: exportdata=.false.
    
    ! Local
    integer                  :: mpo,l,m,itheta,iphi
    real(wp)                 :: dtheta,dphi
    real(sp)                 :: v0max 
    real(wp),allocatable     :: theta(:),phi(:)
    real(sp),allocatable     :: BC(:,:,:),NDBC(:,:,:)
    complex(wpc),allocatable :: BCcom(:,:),NDBCcom(:,:)
    real(wp),allocatable     :: x(:),x_mp(:),x_imp(:)
    real(wp)                 :: mu_mp,mu_imp,tmp,t,t0,p,p0,d
    real(wp),allocatable     :: prefactor(:,:)
    real(wp),allocatable     :: P_mp(:,:,:),P_imp(:,:,:),dPdx_mp(:,:,:),dPdx_imp(:,:,:)
    real(wp),allocatable     :: derivA_mp(:,:,:), derivA_imp(:,:,:)
    real(wp),allocatable     :: derivB_mp(:,:,:), derivB_imp(:,:,:)                            
    real(wp),allocatable     :: r_mp(:),r_imp(:),gamma(:)
    complex(wpc)             :: e1,e2,e3,e4, v0,v1,v2,v3,v4
    complex(wpc)             :: Y_mp,Y_imp,phase,b0,ctmp
    complex(wpc),allocatable :: A(:,:),B(:,:),Ar(:,:),Br(:,:),At(:,:),Bt(:,:)
    real(wp)                 :: tmp1(3,3)
    integer                  :: dim
    
    ! Allocation of output arrays
    Allocate(theta(Ntheta),phi(Nphi))
    Allocate(BC(Ntheta,Nphi,3),NDBC(Ntheta,Nphi,3))
    Allocate(BCcom(Ntheta,Nphi),NDBCcom(Ntheta,Nphi))
    Allocate(x(Ntheta),x_mp(Ntheta),x_imp(Ntheta))
    Allocate(r_mp(Ntheta),r_imp(Ntheta),gamma(Ntheta))
    
    ! Output file
    open(70,file=Trim(Adjustl(filename1))//'.dat')
    open(71,file=Trim(Adjustl(filename2))//'.dat')
    
    ! Dielectric constants
    e1 = 1._wp
    e2 = param%eps_substrate(ienergy)
    e3 = param%eps_island(ienergy)
    e4 = e2
    
    !-------------------------------------------------
    ! Sets up the spherical coordinates
    !-------------------------------------------------
    dtheta    =  (3.14_wp-0.01_wp)/(Ntheta-1)
    dphi      =  2*pi/(Nphi-1)
    theta     =  arth(0.01_wp,dtheta,Ntheta)
    phi       =  arth(0._wp,dphi,Nphi)

    !-------------------------------------------------
    ! Some parameters 
    !-------------------------------------------------
    x          =  cos(theta)
    mu_mp      =  mu_func(param%tr,1,param)    ! Z-position
    mu_imp     =  mu_func(param%tr,2,param) 
    r_mp       =  sqrt( gamma_func(x,mu_mp) )  ! Radius
    r_imp      =  sqrt( gamma_func(x,mu_imp) )          
    x_mp       =  chi_func(x,mu_mp)            ! cos(theta)
    x_imp      =  chi_func(x,mu_imp)   
    v0max      =  0._sp


    !-------------------------------------------------
    ! Sets up the multipole expansion coefficients
    !-------------------------------------------------
    mpo = param%Mpole_order    
    allocate( A(mpo,-1:1),  B(mpo,-1:1)  ) 
    allocate( Ar(mpo,-1:1), Br(mpo,-1:1) )
    allocate( At(mpo,-1:1), Bt(mpo,-1:1) )
    ! m=0,1
    A(:,0:1)  =   MPC(1:mpo,0:1)
    B(:,0:1)  =  MPC(mpo+1:2*mpo,0:1)
    ! m=-1
    A(:,-1)   = - A(:,1) * exp(2*imu*param%phi0_calc)
    B(:,-1)   = - B(:,1) * exp(2*imu*param%phi0_calc)   

    ! Reflected multipole foefficients
    do l=1,mpo
       do m=-1,1
          Ar(l,m) = (-1._wp)**(l+m) * (e1 - e2)/(e1 + e2) * A(l,m)
          Br(l,m) = (-1._wp)**(l+m) * (e3 - e4)/(e3 + e4) * B(l,m)
       enddo
    enddo
    ! transmitted multipole foefficients
    At  =  2*e1/(e1+e2) * A
    Bt  =  2*e3/(e3+e4) * B


    !-------------------------------------------------
    ! Calculates the associated Legendre polymomials 
    ! and prefactors for the spherical harmonics
    !-------------------------------------------------
    allocate( P_mp(mpo,-1:1,Ntheta), P_imp(mpo,-1:1,Ntheta) )
    allocate( dPdx_mp(mpo,-1:1,Ntheta), dPdx_imp(mpo,-1:1,Ntheta) )
    allocate( prefactor(mpo,-1:1) )

    Do l=1,mpo
       Do m=1,-1,-1
          ! the prefactor
          select case(m)
          case(1)
             tmp      =  1._wp/(l*(l+1))
          case(0)
             tmp      =  1._wp                
          case(-1)
             tmp      =  1._wp*l*(l+1)                
          case default
             write(*,*) 'ERROR'
             Pause
			 stop
                      end select
          prefactor(l,m)   =  sqrt( (2*l+1)/(4*pi) * tmp )
          ! The associated legendre polynomials and derivatived of those
          P_mp(l,m,:)  =  Ass_Legendre(l,m,x_mp)     ! Multipoles 
          P_imp(l,m,:) =  Ass_Legendre(l,m,x_imp)    ! Image multipoles
          dPdx_mp(l,m,:)  =  Deriv_Ass_Legendre(l,m,x_mp)  ! Multipoles     
          dPdx_imp(l,m,:) =  Deriv_Ass_Legendre(l,m,x_imp) ! Image multipoles   
       enddo
    enddo

          
    ! Calculates the derivatives of r*P()
    allocate( derivA_mp(mpo,-1:1,Ntheta), derivA_imp(mpo,-1:1,Ntheta) )
    allocate( derivB_mp(mpo,-1:1,Ntheta), derivB_imp(mpo,-1:1,Ntheta) )

    Do l=1,mpo
       Do m=1,-1,-1
          gamma = r_mp**2
          derivA_mp(l,m,:) =                                   &
                     -(l+1)*gamma**(-0.5_wp*l-1.5_wp)          &
                      *(1._wp-mu_mp*x)*P_mp(l,m,:)             &
                     + gamma**(-0.5_wp*l-2._wp)                &
                        *(x*gamma-(x-mu_mp)*(1._wp-mu_mp*x))   &
                           * dPdx_mp(l,m,:)

          gamma = r_imp**2
          derivA_imp(l,m,:) =                                  &
                     -(l+1)*gamma**(-0.5_wp*l-1.5_wp)          &
                      *(1._wp-mu_imp*x)*P_imp(l,m,:)           &
                      + gamma**(-0.5_wp*l-2._wp)               &
                        *(x*gamma-(x-mu_imp)*(1._wp-mu_imp*x)) &
                           * dPdx_imp(l,m,:)
          
          gamma =  r_mp**2         
          derivB_mp(l,m,:) =                                   &
                  l*gamma**(0.5_wp*l-1)                        &
                    *(1._wp-mu_mp*x)*P_mp(l,m,:)               &
                      +gamma**(0.5_wp*l-1.5_wp)                &
                        *(x*gamma-(x-mu_mp)*(1._wp-mu_mp*x))   &
                           * dPdx_mp(l,m,:)
          
          gamma =  r_imp**2         
          derivB_imp(l,m,:) =                                  & 
                  l*gamma**(0.5_wp*l-1)                        &
                    *(1._wp-mu_imp*x)*P_imp(l,m,:)             &
                      +gamma**(0.5_wp*l-1.5_wp)                &
                        *(x*gamma-(x-mu_imp)*(1._wp-mu_imp*x)) &
                           * dPdx_imp(l,m,:)
          
       Enddo
    Enddo





    !------------------------------------------------------
    !---  Calculates the constant b0
    !------------------------------------------------------
    t0 = param%theta0_calc  
    p0 = param%phi0_calc    
    d  = param%div_surface     
    ctmp  = (e1/e2-1._wp)*cos(t0) 
    b0  =    ctmp/sqrt(3._wp)*zeta(0,1,0)*int(0,1,0)%Q          &
         +  param%tr*ctmp * (1._wp-zeta(0,0,0)*int(0,0,0)%Q)
    do l=1,mpo
       ctmp  = A(l,0)*zeta(0,l,0)/sqrt(4*pi)
       ctmp  = ctmp                                                       &
            *( Int(0,l,0)%K%mu(1)                                         &
            +(-1._wp)**l*((e1-e2)/(e1+e2))*Int(0,l,0)%K%mu(2)             &
            +2*e1/(e1+e2)*( Int_tr1(0,l,0)%K%mu(1) - Int(0,l,0)%K%mu(1) ) )
       b0 = b0 + ctmp
       
       ctmp  = B(l,0)*zeta(0,l,0)/sqrt(4*pi)
       ctmp  = ctmp                                                       &
            *( Int(0,l,0)%M%mu(1)                                         &
            +(-1._wp)**l*((e3-e4)/(e3+e4))*Int(0,l,0)%M%mu(2)             &
            +2*e3/(e3+e4)*( Int_tr1(0,l,0)%M%mu(1) - Int(0,l,0)%M%mu(1) ) )
       b0 = b0 - ctmp
    end do
   



    BC    = 0._sp
    !************************************************************
    !*** Continuity of the potetial 
    !*** Note : We actually calculate pot./R
    !************************************************************

    !------------------------------------------------------
    !---  Loops over angles
    !------------------------------------------------------
    theta_loop : do itheta=1,size(theta,1)
       
       !------------------------------------------------------
       ! Upper sphere
       !------------------------------------------------------
       if (theta(itheta) >= acos(param%tr)) then

          ! Loop phi
          phi_loop_above : do iphi=1,size(phi,1)

             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             v1 = 0._wp
             v3 = 0._wp
             
             do l=1,mpo
                do m=-1,1
                   ! spherical harmonics
                   phase = (-1._wp)**m * exp(imu*m*phi(iphi)) 
                   Y_mp  = prefactor(l,m) * P_mp(l,m,itheta)*phase  
                   Y_imp = prefactor(l,m) * P_imp(l,m,itheta)*phase  
                   ! Potentials                
                   v1 = v1 + A(l,m)*r_mp(itheta)**(-l-1)*Y_mp     &
                        + Ar(l,m)*r_imp(itheta)**(-l-1)*Y_imp 
                   v3 = v3 + B(l,m)*r_mp(itheta)**(l)*Y_mp        &
                        + Br(l,m)*r_imp(itheta)**(l)*Y_imp 

                enddo
             enddo
             
             ! Calculates the constant potetial v0
             t  = theta(itheta) 
             p  = phi(iphi)      
             v0 =    -   (   cos(t)*cos(t0)                 &     ! -
                           - sin(t)*cos(p)*sin(t0)*cos(p0)  &     ! +
                           - sin(t)*sin(p)*sin(t0)*sin(p0)  &     ! +
                         )
             v0max = max(v0max,real(abs(v0),sp))

             ! Add the constant potetials
             v1 = v1 +  v0
             v3 = v3 +  b0

             ! Sore the results
             BC(itheta,iphi,1) =  real(v1,sp) 
             BC(itheta,iphi,2) =  real(v3,sp) 
             BC(itheta,iphi,3) =  ( real(v1-v3,sp)   )   !/real(v1+v3,sp) )
             BCcom(itheta,iphi) =  v1 - v3

          end do phi_loop_above

          else
          !------------------------------------------------------
          ! lower sphere
          !------------------------------------------------------

          ! Loop phi
          phi_loop_below : do iphi=1,size(phi,1)

             v2 = 0._wp
             v4 = 0._wp
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             do l=1,mpo
                do m=-1,1
                   ! spherical harmonics
                   phase = (-1._wp)**m * exp(imu*m*phi(iphi)) 
                   Y_mp  = prefactor(l,m) * P_mp(l,m,itheta)*phase   
                   ! Potentials                
                   v2 = v2 + At(l,m)*r_mp(itheta)**(-l-1)*Y_mp     
                   v4 = v4 + Bt(l,m)*r_mp(itheta)**(l)*Y_mp    
                enddo
             enddo

             ! Calculates the constant potetials
             t  = theta(itheta) 
             p  = phi(iphi)      
             v0 =    -   (   e1/e2*cos(t)*cos(t0)           &     ! -
                           - sin(t)*cos(p)*sin(t0)*cos(p0)  &     ! +
                           - sin(t)*sin(p)*sin(t0)*sin(p0)  &     ! +
                         )
             v0max = max(v0max,real(abs(v0),sp))

             ! Add constant potetials          
             v2 = v2 +  v0 + param%tr*(e1/e2-1._wp) * cos(t0)
             v4 = v4 +  b0

             ! Sore the results
             BC(itheta,iphi,1) =  real(v2,sp) 
             BC(itheta,iphi,2) =  real(v4,sp) 
             BC(itheta,iphi,3) =  ( real(v2-v4,sp)      )     !/real(v2+v4,sp) )
			 BCcom(itheta,iphi) =  v2 - v4 

          end do phi_loop_below

          endif 

    enddo theta_loop
    !------------------------------------------------------
    !---  END : Loops over angles
    !------------------------------------------------------

    ! Scaling
    Select Case(param%scaling)
    Case('none')
       continue
    Case('incident_field') 
       BCcom(:,:) =	 BCcom(:,:) / v0max 
    Case('mean_value')
       BCcom(:,:)	=	 2*BCcom(:,:)/(Abs(BC(:,:,1))+Abs(BC(:,:,2)))
    Case Default
       Write(6,*) 'Error in Boundary_conditions. Scaling : ', &
            param%scaling,' non existing yet !'
    End Select
    
    ! Output = mean value of the error
    tmp1 = 0 
    dim = size(BCcom,1)*size(BCcom,2)
    ! mean values
    tmp1(1,1) =  sum(abs(real(BCcom(:,:)))) 
    tmp1(2,1) =  sum(abs(aimag(BCcom(:,:))))
    tmp1(3,1) =  sum(abs(BCcom(:,:)))
    ! standard deviation of tmpmean
    tmp1(1,2) =  sum(abs(real(BCcom(:,:))**2))
    tmp1(2,2) =  sum(abs(aimag(BCcom(:,:))**2))
    tmp1(3,2) =  sum(abs(BCcom(:,:))**2)
    ! normalization
    tmp1 = tmp1 / dim
    ! deviation
    tmp1(1,2) = sqrt( tmp1(1,2) - tmp1(1,1)**2)
    tmp1(2,2) = sqrt( tmp1(2,2) - tmp1(2,1)**2)
    tmp1(3,2) = sqrt( tmp1(3,2) - tmp1(3,1)**2)
    ! maximum value
    tmp1(1,3) = maxval(abs(real(BCcom(:,:))))
    tmp1(2,3) = maxval(abs(aimag(BCcom(:,:))))
    tmp1(3,3) = maxval(abs(BCcom(:,:)))
    
    write(70,200) param%tr,param%energy(ienergy),&
         tmp1(1,1),tmp1(1,2),tmp1(2,1), &
         tmp1(2,2),tmp1(3,1),tmp1(3,2), &
         tmp1(1,3),tmp1(2,3),tmp1(3,3)
    
    
    v0max = 0._sp
    NDBC  = 0._sp
    !************************************************************
    !*** Continuity of the normal  derivative (times 1/eps)
    !************************************************************
    theta_loop_nd : do itheta=1,size(theta,1)
       
       !------------------------------------------------------
       ! Upper sphere
       !------------------------------------------------------
       if (theta(itheta) >= acos(param%tr)) then
          
          ! Loop phi
          phi_loop_above_nd : do iphi=1,size(phi,1)
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             v1 = 0._wp
             v3 = 0._wp
             
             do l=1,mpo
                do m=-1,1
                   ! The Phase
                   phase = (-1._wp)**m * exp(imu*m*phi(iphi)) 
                   ! Potentials 
                   v1 = v1 + A(l,m) * derivA_mp(l,m,itheta)*prefactor(l,m)*phase   &
                        + Ar(l,m) * derivA_imp(l,m,itheta)*prefactor(l,m)*phase 
                   v3 = v3 + B(l,m)* derivB_mp(l,m,itheta)*prefactor(l,m)*phase    &
                        + Br(l,m) * derivB_imp(l,m,itheta)*prefactor(l,m)*phase 
                enddo
             enddo

             ! Calculates the constant potetial v0
             t  = theta(itheta) 
             p  = phi(iphi)      
             v0 =    -   (   cos(t)*cos(t0)                 &     ! -
                           - sin(t)*cos(p)*sin(t0)*cos(p0)  &     ! +
                           - sin(t)*sin(p)*sin(t0)*sin(p0) )      ! +
             v0max = max(v0max, real( max( abs(real(e3*v0)),abs(real(e1*v0)) ),sp) )

             ! Add constant potetials          
             v1 = v1 +  v0 
      

             ! Multiply by eps
             v1 = v1*e1
             v3 = v3*e3  
			  
			 			              
             ! Store the results
             NDBC(itheta,iphi,1) =  real(v1,sp) 
             NDBC(itheta,iphi,2) =  real(v3,sp) 
             NDBC(itheta,iphi,3) =  ( real(v1-v3,sp)  )      !/real(v1+v3,sp) )
             NDBCcom(itheta,iphi) =  v1 - v3

          end do phi_loop_above_nd

       else
          !------------------------------------------------------
          ! lower sphere
          !------------------------------------------------------
          
          ! Loop phi
          phi_loop_below_nd : do iphi=1,size(phi,1)
             
             v2 = 0._wp
             v4 = 0._wp
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             do l=1,mpo
                do m=-1,1

                   ! The Phase
                   phase = (-1._wp)**m * exp(imu*m*phi(iphi)) 
                   ! Potentials                
                   v2 = v2 + At(l,m) * derivA_mp(l,m,itheta)*prefactor(l,m)*phase  
                   v4 = v4 + Bt(l,m)* derivB_mp(l,m,itheta)*prefactor(l,m)*phase   
                enddo
             enddo

             ! Calculates the constant potetials
             t  = theta(itheta) 
             p  = phi(iphi)      
             v0 =    -   (   e1/e2*cos(t)*cos(t0)           &     ! -
                           - sin(t)*cos(p)*sin(t0)*cos(p0)  &     ! +
                           - sin(t)*sin(p)*sin(t0)*sin(p0)  &     ! +
                         )
             v0max = max( v0max,real( max(abs(real(e2*v0)),abs(real(e4*v0))),sp ) )
             ! Add constant potetials          
             v2 = v2 +  v0 
  
             ! Multiply by eps
             v2 = v2*e2
             v4 = v4*e4

             ! Store the results
             NDBC(itheta,iphi,1) =  real(v2,sp) 
             NDBC(itheta,iphi,2) =  real(v4,sp) 
             NDBC(itheta,iphi,3) =  ( real(v2-v4,sp))    !/real(v2+v4,sp) )
             NDBCcom(itheta,iphi) =  v2 - v4
          end do phi_loop_below_nd

       endif
       
    enddo theta_loop_nd


    ! Scaling
    Select Case(param%scaling)
    Case('none')
       continue
    Case('incident_field') 
       NDBCcom(:,:) =	 NDBCcom(:,:) / v0max 
    Case('mean_value')
       NDBCcom(:,:)	=	 2*NDBCcom(:,:)/(Abs(NDBC(:,:,1))+Abs(NDBC(:,:,2)))
    Case Default
       Write(6,*) 'Error in Boundary_conditions. Scaling : ', &
            param%scaling,' non existing yet !'
    End Select
    
    
    ! Output = mean value of the error 
    dim = size(NDBCcom,1)*size(NDBCcom,2)
    ! mean values
    tmp1(1,1) =  sum(abs(real(NDBCcom(:,:)))) 
    tmp1(2,1) =  sum(abs(aimag(NDBCcom(:,:))))
    tmp1(3,1) =  sum(abs(NDBCcom(:,:)))
    ! standard deviation of tmpmean
    tmp1(1,2) =  sum(abs(real(NDBCcom(:,:))**2)) 
    tmp1(2,2) =  sum(abs(aimag(NDBCcom(:,:))**2))
    tmp1(3,2) =  sum(abs(NDBCcom(:,:))**2)
    ! normalization
    tmp1 = tmp1 / dim
    ! deviation
    tmp1(1,2) = sqrt( tmp1(1,2) - tmp1(1,1)**2)
    tmp1(2,2) = sqrt( tmp1(2,2) - tmp1(2,1)**2)
    tmp1(3,2) = sqrt( tmp1(3,2) - tmp1(3,1)**2)
    ! maximum value
    tmp1(1,3) = maxval(abs(real(NDBCcom(:,:))))
    tmp1(2,3) = maxval(abs(aimag(NDBCcom(:,:))))
    tmp1(3,3) = maxval(abs(NDBCcom(:,:)))
    
    write(71,200) param%tr,param%energy(ienergy),&
         tmp1(1,1),tmp1(1,2),tmp1(2,1), &
         tmp1(2,2),tmp1(3,1),tmp1(3,2), &
         tmp1(1,3),tmp1(2,3),tmp1(3,3)
    
200 format(11f20.7)
    ! Output = mean value of the error
    !    write(71,*) param%energy(ienergy), &
    !             real( sum(abs(NDBC(:,:,3))) / (size(NDBC,1)*size(NDBC,2)), sp)
    
    
    

    !******************************************************
    !*** Calculations finished
    !******************************************************


    !------------------------------------------------------
    !---  Write to file
    !------------------------------------------------------
    If (exportdata) then
       open(unit=90, file=trim(adjustl(filename1))//'_export.dat')
       open(unit=91, file=trim(adjustl(filename2))//'_export.dat')
       
       write(90,*)  Ntheta
       write(90,*)  Nphi
       write(90,*)  real(PARAM%theta0_calc*180/pi,sp)
       write(90,*)  real(PARAM%TR,sp)

       do itheta = 1,size(theta,1)
          do iphi = 1,size(phi,1)
             if ( (phi(iphi)>=0._wp) .and. (phi(iphi)<pi) ) then
                write(90,*) real(theta(itheta),sp), real(phi(iphi),sp), real(BC(itheta,iphi,3),sp)
                write(91,*) real(theta(itheta),sp), real(phi(iphi),sp), real(NDBC(itheta,iphi,3),sp)
             endif
          enddo
       enddo
    Endif
 
!!$    !------------------------------------------------------
!!$    !---  Plot the results to screen
!!$    !------------------------------------------------------
!!$        write(*,*) 'theta = ', theta(25)   * 180/pi
!!$    call plot(real(phi,sp),BC(25,:,1),yr2=BC(25,:,2),title='PHI; real-part (F)')
!!$        write(*,*) 'theta = ', theta(75)   * 180/pi
!!$    call plot(real(phi,sp),BC(75,:,1),yr2=BC(75,:,2),title='PHI; real-part (F)')
!!$        write(*,*) 'phi = ', phi(1)   * 180/pi
!!$    call plot(real(theta,sp),BC(:,1,1),yr2=BC(:,1,2),title='THETA; real-part (F)')
!!$        write(*,*) 'phi = ', phi(25)   * 180/pi
!!$    call plot(real(theta,sp),BC(:,25,1),yr2=BC(:,25,2),title='THETA; real-part (F)')
!!$
!!$
!!$
!!$        write(*,*) 'theta = ', theta(25)   * 180/pi
!!$    call plot(real(phi,sp),NDBC(25,:,1),yr=2NDBC(25,:,2),title='PHI; real-part (ND)')
!!$        write(*,*) 'theta = ', theta(75)   * 180/pi
!!$    call plot(real(phi,sp),NDBC(75,:,1),yr2=NDBC(75,:,2),title='PHI; real-part (ND)')
!!$        write(*,*) 'phi = ', phi(1)   * 180/pi
!!$    call plot(real(theta,sp),NDBC(:,1,1),yr2=NDBC(:,1,2),title='THETA; real-part (ND)')
!!$        write(*,*) 'phi = ', phi(25)   * 180/pi
!!$    call plot(real(theta,sp),NDBC(:,25,1),yr2=NDBC(:,25,2),title='THETA; real-part(ND) ')
!!$
!!$    call contour(real(BC(:,:,3),sp))
!!$    call contour(real(NDBC(:,:,3),sp))
!!$
!!$
!!$    !------------------------------------------------------
!!$    !---  Write results to file
!!$    !------------------------------------------------------
!!$    do l=1,mpo
!!$       write(25,*) l, abs(A(l,0)), abs(A(l,1))
!!$       write(26,*) l, abs(B(l,0)), abs(B(l,1))
!!$    end do
!!$
!!$    itheta=25 
!!$    do iphi=1,Nphi
!!$       write(30,*) phi(iphi), BC(itheta,iphi,1),BC(itheta,iphi,2), NDBC(itheta,iphi,1), NDBC(itheta,iphi,2)
!!$    end do
!!$
!!$    itheta=75 
!!$    do iphi=1,Nphi
!!$       write(31,*) phi(iphi), BC(itheta,iphi,1),BC(itheta,iphi,2), NDBC(itheta,iphi,1), NDBC(itheta,iphi,2)
!!$    end do
!!$
!!$
!!$    iphi = 1
!!$    do itheta=1,Ntheta
!!$       write(40,*) theta(itheta), BC(itheta,iphi,1),BC(itheta,iphi,2), NDBC(itheta,iphi,1), NDBC(itheta,iphi,2)
!!$    end do
!!$
!!$
!!$    iphi = 25
!!$    do itheta=1,Ntheta
!!$       write(41,*) theta(itheta), BC(itheta,iphi,1),BC(itheta,iphi,2), NDBC(itheta,iphi,1), NDBC(itheta,iphi,2)
!!$    end do
!!$
!!$
!!$    write(*,*) 'MAX  : ', maxval(BC), maxloc(BC) 
!!$    write(*,*) 'MIN  : ', minval(BC), minloc(BC) 
!!$    write(*,*) 'MEAN : ', sum(BC) / (size(BC,1)*size(BC,2))
!!$

    ! Deallocation of  arrays
    Deallocate(theta,phi)
    Deallocate(BC,NDBC)
    Deallocate(BCcom,NDBCcom)
    Deallocate(x,x_mp,x_imp)
    Deallocate(r_mp,r_imp,gamma)
    
  End Subroutine boundary_conditions_above




!---------------------------------------------------------------------------------------------------


  Subroutine boundary_conditions_below(MPC,ienergy,int,int_tr1,zeta,param)
    !-------------------------------------
    ! Multipoles below the surface
    !-------------------------------------
    Use Legendre, Only : asslegpoly, Gausslegendre

    implicit none
    Type(parameters) :: param 
    Type(integrals)  :: Int(0:,0:,0:), int_tr1(0:,0:,0:) 
    integer          :: ienergy
    Real(wp)         :: zeta(0:,0:,0:)
    complex(wpc)     :: MPC(2*param%Mpole_order,0:1)

    ! Parameters
    integer, parameter              :: Ntheta=120,Nphi=200
    character(len=2),parameter      :: filename1='BC'
    character(len=4),parameter      :: filename2='NDBC'
    logical                         :: exportdata=.false.

    ! Local
    integer                         :: mpo,l,m,itheta,iphi
    real(sp)                        :: v0max
    real(wp)		            :: dtheta,dphi 
    real(wp),allocatable            :: theta(:),phi(:)
    real(sp),allocatable            :: BC(:,:,:),NDBC(:,:,:)
	complex(wpc),allocatable    :: BCcom(:,:),NDBCcom(:,:)
    real(wp),allocatable            :: x(:),x_mp(:),x_imp(:)
    real(wp)                        :: mu_mp,mu_imp,tmp,t,t0,p,p0,d
    real(wp),allocatable            :: prefactor(:,:)
    real(wp),allocatable            :: P_mp(:,:,:),P_imp(:,:,:),dPdx_mp(:,:,:),dPdx_imp(:,:,:)
    real(wp),allocatable            :: derivA_mp(:,:,:), derivA_imp(:,:,:)
    real(wp),allocatable            :: derivB_mp(:,:,:), derivB_imp(:,:,:)                            
    real(wp),allocatable            :: r_mp(:),r_imp(:),gamma(:)
    complex(wpc)                    :: e1,e2,e3,e4, v0,v1,v2,v3,v4
    complex(wpc)                    :: Y_mp,Y_imp,phase,b0,ctmp
    complex(wpc),allocatable        :: A(:,:),B(:,:),Ar(:,:),Br(:,:),At(:,:),Bt(:,:)
    real(wp)                        :: tmp1(3,3)
    integer                         :: dim

    ! Allocation of output arrays
    Allocate(theta(Ntheta),phi(Nphi))
    Allocate(BC(Ntheta,Nphi,3),NDBC(Ntheta,Nphi,3))
    Allocate(BCcom(Ntheta,Nphi),NDBCcom(Ntheta,Nphi))
    Allocate(x(Ntheta),x_mp(Ntheta),x_imp(Ntheta))
    Allocate(r_mp(Ntheta),r_imp(Ntheta),gamma(Ntheta))
    
    ! Output file
    open(70,file=Trim(Adjustl(filename1))//'.dat')
    open(71,file=Trim(Adjustl(filename2))//'.dat')
    
    ! Dielectric constants
    e1 = 1._wp
    e2 = param%eps_substrate(ienergy)
    e3 = param%eps_island(ienergy)
    e4 = e2

    !-------------------------------------------------
    ! Sets up the spherical coordinates
    !-------------------------------------------------
    dtheta    =  (3.14_wp-0.01_wp)/(Ntheta-1)
    dphi      =  2*pi/(Nphi-1)
    theta     =  arth(0.01_wp,dtheta,Ntheta)
    phi       =  arth(0._wp,dphi,Nphi)

    !-------------------------------------------------
    ! Some parameters 
    !-------------------------------------------------
    x          =  cos(theta)
    mu_mp      =  mu_func(param%tr,1,param)    ! Z-position
    mu_imp     =  mu_func(param%tr,2,param) 
    r_mp       =  sqrt( gamma_func(x,mu_mp) )  ! Radius
    r_imp      =  sqrt( gamma_func(x,mu_imp) )          
    x_mp       =  chi_func(x,mu_mp)            ! cos(theta)
    x_imp      =  chi_func(x,mu_imp)   
    v0max      =  0._sp

    !-------------------------------------------------
    ! Sets up the multipole expansion coefficients
    !-------------------------------------------------
    mpo = param%Mpole_order    
    allocate( A(mpo,-1:1),  B(mpo,-1:1)  ) 
    allocate( Ar(mpo,-1:1), Br(mpo,-1:1) )
    allocate( At(mpo,-1:1), Bt(mpo,-1:1) )
    ! m=0,1
    A(:,0:1)  =   MPC(1:mpo,0:1)
    B(:,0:1)  =   MPC(mpo+1:2*mpo,0:1)
    ! m=-1
    A(:,-1)   = - A(:,1) * exp(2*imu*param%phi0_calc)
    B(:,-1)   = - B(:,1) * exp(2*imu*param%phi0_calc)   

    ! Reflected multipole foefficients
    do l=1,mpo
       do m=-1,1
          Ar(l,m) = (-1._wp)**(l+m) * (e2 - e1)/(e1 + e2) * A(l,m)
          Br(l,m) = (-1._wp)**(l+m) * (e4 - e3)/(e3 + e4) * B(l,m)
       enddo
    enddo
    ! transmitted multipole foefficients
    At  =  2*e2/(e1+e2) * A
    Bt  =  2*e4/(e3+e4) * B


    !-------------------------------------------------
    ! Calculates the associated Legendre polymomials 
    ! and prefactors for the spherical harmonics
    !-------------------------------------------------
    allocate( P_mp(mpo,-1:1,Ntheta), P_imp(mpo,-1:1,Ntheta) )
    allocate( dPdx_mp(mpo,-1:1,Ntheta), dPdx_imp(mpo,-1:1,Ntheta) )
    allocate( prefactor(mpo,-1:1) )

    Do l=1,mpo
       Do m=1,-1,-1
          ! the prefactor
          select case(m)
          case(1)
             tmp              =  1._wp/(l*(l+1))
          case(0)
             tmp              =  1._wp                
          case(-1)
             tmp              =  1._wp*l*(l+1)                
          case default
             write(*,*) 'ERROR'
             Pause
			 stop
          end select
          prefactor(l,m)   =  sqrt( (2*l+1)/(4*pi) * tmp )
          ! The associated legendre polynomials and derivatived of those
          P_mp(l,m,:)  =  Ass_Legendre(l,m,x_mp)     ! Multipoles 
          P_imp(l,m,:) =  Ass_Legendre(l,m,x_imp)    ! Image multipoles
          dPdx_mp(l,m,:)  =  Deriv_Ass_Legendre(l,m,x_mp)  ! Multipoles     
          dPdx_imp(l,m,:) =  Deriv_Ass_Legendre(l,m,x_imp) ! Image multipoles   
       enddo
    enddo

          
    ! Calculates the derivatives of r*P()
    allocate( derivA_mp(mpo,-1:1,Ntheta), derivA_imp(mpo,-1:1,Ntheta) )
    allocate( derivB_mp(mpo,-1:1,Ntheta), derivB_imp(mpo,-1:1,Ntheta) )

    Do l=1,mpo
       Do m=1,-1,-1
          gamma = r_mp**2
          derivA_mp(l,m,:) =                                   &
                     -(l+1)*gamma**(-0.5_wp*l-1.5_wp)          &
                      *(1._wp-mu_mp*x)*P_mp(l,m,:)             &
                     + gamma**(-0.5_wp*l-2._wp)                &
                        *(x*gamma-(x-mu_mp)*(1._wp-mu_mp*x))   &
                           * dPdx_mp(l,m,:)

          gamma = r_imp**2
          derivA_imp(l,m,:) =                                  &
                     -(l+1)*gamma**(-0.5_wp*l-1.5_wp)          &
                      *(1._wp-mu_imp*x)*P_imp(l,m,:)           &
                      + gamma**(-0.5_wp*l-2._wp)               &
                        *(x*gamma-(x-mu_imp)*(1._wp-mu_imp*x)) &
                           * dPdx_imp(l,m,:)
          
          gamma =  r_mp**2         
          derivB_mp(l,m,:) =                                   &
                  l*gamma**(0.5_wp*l-1)                        &
                    *(1._wp-mu_mp*x)*P_mp(l,m,:)               &
                      +gamma**(0.5_wp*l-1.5_wp)                &
                        *(x*gamma-(x-mu_mp)*(1._wp-mu_mp*x))   &
                           * dPdx_mp(l,m,:)
          
          gamma =  r_imp**2         
          derivB_imp(l,m,:) =                                  & 
                  l*gamma**(0.5_wp*l-1)                        &
                    *(1._wp-mu_imp*x)*P_imp(l,m,:)             &
                      +gamma**(0.5_wp*l-1.5_wp)                &
                        *(x*gamma-(x-mu_imp)*(1._wp-mu_imp*x)) &
                           * dPdx_imp(l,m,:)
          
       Enddo
    Enddo


    



    !------------------------------------------------------
    !---  Calculates the constant b0
    !------------------------------------------------------
    t0 = param%theta0_calc  
    p0 = param%phi0_calc    
    d  = param%div_surface     
    ctmp  = (e1/e2-1._wp)*cos(t0) 
    b0  =    ctmp/sqrt(3._wp)*zeta(0,1,0)*int(0,1,0)%Q                                  &
         + param%tr*ctmp*zeta(0,0,0)*int(0,0,0)%Q
    do l=1,mpo
       ctmp  = A(l,0)*zeta(0,l,0)/sqrt(4*pi)
       ctmp  = ctmp                                                                     &
            *( (-1._wp)**l*Int(0,l,0)%K%mu(1)                                           &
            +(e2-e1)/(e2+e1)*Int(0,l,0)%K%mu(2)                                         &
            +(-1._wp)**l*2*e2/(e2+e1)*( Int_tr1(0,l,0)%K%mu(1) - Int(0,l,0)%K%mu(1) ) )
       b0 = b0 + ctmp
       
       ctmp  = B(l,0)*zeta(0,l,0)/sqrt(4*pi)
       ctmp  = ctmp                                                                    &
            *( (-1._wp)**l*Int(0,l,0)%M%mu(1)                                          &
            +(e4-e3)/(e4+e3)*Int(0,l,0)%M%mu(2)                                        &
            +(-1._wp)**l*2*e4/(e4+e3)*( Int_tr1(0,l,0)%M%mu(1) - Int(0,l,0)%M%mu(1) ) )
       b0 = b0 - ctmp
    end do
   



    BC    = 0._sp
    !************************************************************
    !*** Continuity of the potetial 
    !*** Note : We actually calculate pot./R
    !************************************************************

    !------------------------------------------------------
    !---  Loops over angles
    !------------------------------------------------------
    theta_loop : do itheta=1,size(theta,1)

       !------------------------------------------------------
       ! Upper sphere
       !------------------------------------------------------
       if (theta(itheta) >= acos(param%tr)) then

          ! Loop phi
          phi_loop_above : do iphi=1,size(phi,1)

             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             v1 = 0._wp
             v3 = 0._wp
             
             do l=1,mpo
                do m=-1,1
                   ! spherical harmonics
                   phase = (-1._wp)**m * exp(imu*m*phi(iphi)) 
                   Y_mp  =  prefactor(l,m) * P_mp(l,m,itheta)*phase  
                   ! Potentials                
                   v1 = v1 + At(l,m)*r_mp(itheta)**(-l-1)*Y_mp     
                   v3 = v3 + Bt(l,m)*r_mp(itheta)**(l)*Y_mp        
                enddo
             enddo
             
             ! Calculates the constant potetial v0
             t  = theta(itheta) 
             p  = phi(iphi)      
             v0 =    -   (   cos(t)*cos(t0)                 &     ! -
                           - sin(t)*cos(p)*sin(t0)*cos(p0)  &     ! +
                           - sin(t)*sin(p)*sin(t0)*sin(p0)  &     ! +
                         )
             v0max = max(v0max,real(abs(v0),sp))

             ! Add the constant potetials
             v1 = v1 +  v0
             v3 = v3 +  b0

             ! Sore the results
             BC(itheta,iphi,1) =  real(v1,sp) 
             BC(itheta,iphi,2) =  real(v3,sp) 
             BC(itheta,iphi,3) =  ( real(v1-v3,sp)   )   !/real(v1+v3,sp) )
             BCcom(itheta,iphi) =  v1 - v3
          end do phi_loop_above

          else
          !------------------------------------------------------
          ! lower sphere
          !------------------------------------------------------

          ! Loop phi
          phi_loop_below : do iphi=1,size(phi,1)

             v2 = 0._wp
             v4 = 0._wp
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             do l=1,mpo
                do m=-1,1
                   ! spherical harmonics
                   phase = (-1._wp)**m * exp(imu*m*phi(iphi)) 
                   Y_mp  = prefactor(l,m) * P_mp(l,m,itheta)*phase  
                   Y_imp = prefactor(l,m) * P_imp(l,m,itheta)*phase 
				   ! Potentials                
                   v2 = v2 + A(l,m)*r_mp(itheta)**(-l-1)*Y_mp + &
							 Ar(l,m)*r_imp(itheta)**(-l-1)*Y_imp
                   v4 = v4 + B(l,m)*r_mp(itheta)**(l)*Y_mp  + &
							 Br(l,m)*r_imp(itheta)**(l)*Y_imp
                enddo
             enddo

             ! Calculates the constant potetials
             t  = theta(itheta) 
             p  = phi(iphi)      
             v0 =    -   (   e1/e2*cos(t)*cos(t0)           &     ! -
                           - sin(t)*cos(p)*sin(t0)*cos(p0)  &     ! +
                           - sin(t)*sin(p)*sin(t0)*sin(p0)  &     ! +
                         )
             v0max = max(v0max,real(abs(v0),sp))

             ! Add constant potetials          
             v2 = v2 +  v0 + param%tr*(e1/e2-1._wp) * cos(t0)
             v4 = v4 +  b0

             ! Sore the results
             BC(itheta,iphi,1) =  real(v2,sp) 
             BC(itheta,iphi,2) =  real(v4,sp) 
             BC(itheta,iphi,3) =  ( real(v2-v4,sp)      )     !/real(v2+v4,sp) )
             BCcom(itheta,iphi) =  v2 - v4
          end do phi_loop_below

          endif 

    enddo theta_loop
    !------------------------------------------------------
    !---  END : Loops over angles
    !------------------------------------------------------

    ! Scaling
    Select Case(param%scaling)
    Case('none')
       continue
    Case('incident_field') 
       BCcom(:,:) =	 BCcom(:,:) / v0max 
    Case('mean_value')
       BCcom(:,:)	=	 2*BCcom(:,:)/(Abs(BC(:,:,1))+Abs(BC(:,:,2)))
    Case Default
       Write(6,*) 'Error in Boundary_conditions. Scaling : ', &
            param%scaling,' non existing yet !'
    End Select
    

    ! Output = mean value of the error 
    dim = size(BCcom,1)*size(BCcom,2)
    ! mean values
    tmp1(1,1) =  sum(abs(real(BCcom(:,:)))) 
    tmp1(2,1) =  sum(abs(aimag(BCcom(:,:))))
    tmp1(3,1) =  sum(abs(BCcom(:,:)))
    ! standard deviation of tmpmean
    tmp1(1,2) =  sum(abs(real(BCcom(:,:))**2)) 
    tmp1(2,2) =  sum(abs(aimag(BCcom(:,:))**2))
    tmp1(3,2) =  sum(abs(BCcom(:,:))**2)
    ! normalization
    tmp1 = tmp1 / dim
    ! deviation
    tmp1(1,2) = sqrt( tmp1(1,2) - tmp1(1,1)**2)
    tmp1(2,2) = sqrt( tmp1(2,2) - tmp1(2,1)**2)
    tmp1(3,2) = sqrt( tmp1(3,2) - tmp1(3,1)**2)
    
    write(70,200) param%tr,param%energy(ienergy), &
         tmp1(1,1),tmp1(1,2),tmp1(2,1),tmp1(2,2),tmp1(3,1),tmp1(3,2)
    
    

    v0max = 0._sp
    NDBC  = 0._sp
    !************************************************************
    !*** Continuity of the normal  derivative (times 1/eps)
    !************************************************************
    theta_loop_nd : do itheta=1,size(theta,1)

       !------------------------------------------------------
       ! Upper sphere
       !------------------------------------------------------
       if (theta(itheta) >= acos(param%tr)) then

          ! Loop phi
          phi_loop_above_nd : do iphi=1,size(phi,1)

             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             v1 = 0._wp
             v3 = 0._wp
             
             do l=1,mpo
                do m=-1,1
                   ! The Phase
                   phase = (-1._wp)**m * exp(imu*m*phi(iphi)) 
                   ! Potentials 
                   v1 = v1 + At(l,m) * derivA_mp(l,m,itheta)*prefactor(l,m)*phase   
                   v3 = v3 + Bt(l,m)* derivB_mp(l,m,itheta)*prefactor(l,m)*phase 
                enddo
             enddo

             ! Calculates the constant potetial v0
             t  = theta(itheta) 
             p  = phi(iphi)      
             v0 =    -   (   cos(t)*cos(t0)                 &     ! -
                           - sin(t)*cos(p)*sin(t0)*cos(p0)  &     ! +
                           - sin(t)*sin(p)*sin(t0)*sin(p0)  )     ! +
             v0max = max(v0max, real( max( abs(real(e3*v0)),abs(real(e1*v0)) ),sp ) )

             ! Add constant potetials          
             v1 = v1 +  v0 

             ! Multiply by eps
             v1 = v1*e1
             v3 = v3*e3             
             
             ! Store the results
             NDBC(itheta,iphi,1) =  real(v1,sp) 
             NDBC(itheta,iphi,2) =  real(v3,sp) 
             NDBC(itheta,iphi,3) =  ( real(v1-v3,sp)  )      !/real(v1+v3,sp) )
             NDBCcom(itheta,iphi) = v1 - v3


          end do phi_loop_above_nd

       else
          !------------------------------------------------------
          ! lower sphere
          !------------------------------------------------------

          ! Loop phi
          phi_loop_below_nd : do iphi=1,size(phi,1)

             v2 = 0._wp
             v4 = 0._wp
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             do l=1,mpo
                do m=-1,1

                   ! The Phase
                   phase = (-1._wp)**m * exp(imu*m*phi(iphi)) 
                   ! Potentials                
                   v2 = v2 + A(l,m) * derivA_mp(l,m,itheta)*prefactor(l,m)*phase  + &
							 Ar(l,m) * derivA_imp(l,m,itheta)*prefactor(l,m)*phase
                   v4 = v4 + B(l,m)* derivB_mp(l,m,itheta)*prefactor(l,m)*phase   + &
							 Br(l,m)* derivB_imp(l,m,itheta)*prefactor(l,m)*phase		
                enddo
             enddo

             ! Calculates the constant potetials
             t  = theta(itheta) 
             p  = phi(iphi)      
             v0 =    -   (   e1/e2*cos(t)*cos(t0)           &     ! -
                           - sin(t)*cos(p)*sin(t0)*cos(p0)  &     ! +
                           - sin(t)*sin(p)*sin(t0)*sin(p0)  &     ! +
                         )
             v0max = max( v0max,real( max(abs(real(e2*v0)),abs(real(e4*v0))) ,sp ) )
             ! Add constant potetials          
             v2 = v2 +  v0 
  
             ! Multiply by eps
             v2 = v2*e2
             v4 = v4*e4

             ! Store the results
             NDBC(itheta,iphi,1) =  real(v2,sp) 
             NDBC(itheta,iphi,2) =  real(v4,sp) 
             NDBC(itheta,iphi,3) =  ( real(v2-v4,sp))    !/real(v2+v4,sp) )
             NDBCcom(itheta,iphi) =  v2 - v4

          end do phi_loop_below_nd

       endif
       
    enddo theta_loop_nd
  
    ! Scaling
    Select Case(param%scaling)
    Case('none')
       continue
    Case('incident_field') 
       NDBCcom(:,:) =	 NDBCcom(:,:) / v0max 
    Case('mean_value')
       NDBCcom(:,:)	=	 2*NDBCcom(:,:)/(Abs(NDBC(:,:,1))+Abs(NDBC(:,:,2)))
    Case Default
       Write(6,*) 'Error in Boundary_conditions. Scaling : ', &
            param%scaling,' non existing yet !'
    End Select

    ! Output = mean value of the error 
    dim = size(NDBCcom,1)*size(NDBCcom,2)
    ! mean values
    tmp1(1,1) =  sum(abs(real(NDBCcom(:,:)))) 
    tmp1(2,1) =  sum(abs(aimag(NDBCcom(:,:))))
    tmp1(3,1) =  sum(abs(NDBCcom(:,:)))
    ! standard deviation of tmpmean
    tmp1(1,2) =  sum(abs(real(NDBCcom(:,:))**2)) 
    tmp1(2,2) =  sum(abs(aimag(NDBCcom(:,:))**2))
    tmp1(3,2) =  sum(abs(NDBCcom(:,:))**2)
    ! normalization
    tmp1 = tmp1 / dim
    ! deviation
    tmp1(1,2) = sqrt( tmp1(1,2) - tmp1(1,1)**2)
    tmp1(2,2) = sqrt( tmp1(2,2) - tmp1(2,1)**2)
    tmp1(3,2) = sqrt( tmp1(3,2) - tmp1(3,1)**2)
    
    write(71,200) param%tr,param%energy(ienergy), &
         tmp1(1,1),tmp1(1,2),tmp1(2,1),tmp1(2,2),tmp1(3,1),tmp1(3,2)
    

200 format(8f20.7)



    !******************************************************
    !*** Calculations finished
    !******************************************************


    !------------------------------------------------------
    !---  Write to file
    !------------------------------------------------------
    If (exportdata) then
       open(unit=90, file=trim(adjustl(filename1))//'.export_dat')
       open(unit=91,file=trim(adjustl(filename2))//'.export_dat')

       write(90,*)  Ntheta
       write(90,*)  Nphi
       write(90,*)  real(PARAM%theta0_calc*180/pi,sp)
       write(90,*)  real(PARAM%TR,sp)

       do itheta = 1,size(theta,1)
          do iphi = 1,size(phi,1)
             if ( (phi(iphi)>=0._wp) .and. (phi(iphi)<pi) ) then
                write(90,*) real(theta(itheta),sp), real(phi(iphi),sp), real(BC(itheta,iphi,3),sp)
                write(91,*) real(theta(itheta),sp), real(phi(iphi),sp), real(NDBC(itheta,iphi,3),sp)
             endif
          enddo
       enddo
    Endif
    

    !------------------------------------------------------
    !---  Plot the results
    !------------------------------------------------------
!        write(*,*) 'theta = ', theta(25)   * 180/pi
!    call plot(real(phi,sp),BC(25,:,1),BC(25,:,2),title='PHI; real-part (F)')
!        write(*,*) 'theta = ', theta(75)   * 180/pi
!    call plot(real(phi,sp),BC(75,:,1),BC(75,:,2),title='PHI; real-part (F)')
!        write(*,*) 'phi = ', phi(1)   * 180/pi
!    call plot(real(theta,sp),BC(:,1,1),BC(:,1,2),title='THETA; real-part (F)')
!        write(*,*) 'phi = ', phi(25)   * 180/pi
!    call plot(real(theta,sp),BC(:,25,1),BC(:,25,2),title='THETA; real-part (F)')



!        write(*,*) 'theta = ', theta(25)   * 180/pi
!    call plot(real(phi,sp),NDBC(25,:,1),NDBC(25,:,2),title='PHI; real-part (ND)')
!        write(*,*) 'theta = ', theta(75)   * 180/pi
!    call plot(real(phi,sp),NDBC(75,:,1),NDBC(75,:,2),title='PHI; real-part (ND)')
!        write(*,*) 'phi = ', phi(1)   * 180/pi
!    call plot(real(theta,sp),NDBC(:,1,1),NDBC(:,1,2),title='THETA; real-part (ND)')
!        write(*,*) 'phi = ', phi(25)   * 180/pi
!    call plot(real(theta,sp),NDBC(:,25,1),NDBC(:,25,2),title='THETA; real-part(ND) ')

!    call contour(real(BC(:,:,3),sp))
!    call contour(real(NDBC(:,:,3),sp))


    !------------------------------------------------------
    !---  Write results to file
    !------------------------------------------------------
!    do l=1,mpo
!       write(25,*) l, abs(A(l,0)), abs(A(l,1))
!       write(26,*) l, abs(B(l,0)), abs(B(l,1))
!    end do

!    itheta=25 
!    do iphi=1,Nphi
!       write(30,*) phi(iphi), BC(itheta,iphi,1),BC(itheta,iphi,2), NDBC(itheta,iphi,1), NDBC(itheta,iphi,2)
!    end do

!    itheta=75 
!    do iphi=1,Nphi
!       write(31,*) phi(iphi), BC(itheta,iphi,1),BC(itheta,iphi,2), NDBC(itheta,iphi,1), NDBC(itheta,iphi,2)
!    end do


!    iphi = 1
!    do itheta=1,Ntheta
!       write(40,*) theta(itheta), BC(itheta,iphi,1),BC(itheta,iphi,2), NDBC(itheta,iphi,1), NDBC(itheta,iphi,2)
!    end do


!    iphi = 25
!    do itheta=1,Ntheta
!       write(41,*) theta(itheta), BC(itheta,iphi,1),BC(itheta,iphi,2), NDBC(itheta,iphi,1), NDBC(itheta,iphi,2)
!    end do


!    write(*,*) 'MAX  : ', maxval(BC), maxloc(BC) 
!    write(*,*) 'MIN  : ', minval(BC), minloc(BC) 
!    write(*,*) 'MEAN : ', sum(BC) / (size(BC,1)*size(BC,2))


    ! Deallocation of  arrays
    Deallocate(theta,phi)
    Deallocate(BC,NDBC)
    Deallocate(BCcom,NDBCcom)
    Deallocate(x,x_mp,x_imp)
    Deallocate(r_mp,r_imp,gamma)
    
  End Subroutine boundary_conditions_below



  !==============================================================
  !==============================================================
  !==============================================================

   Function mu_func(tr,Type,param)  Result(mu)
     ! calculated the z-coordiate of the multipole (MP) ir image MP
     Implicit None
     Type(parameters)   :: param     
     Integer            :: Type
     Real(wp)           :: tr,mu

     Select Case(Type)
     Case(1)
        ! The MP
        mu = param%MPpos
     Case(2)
        ! The image MP
        mu = 2._wp*tr - param%MPpos
     Case default
        Pause 'ERROR in mu_func'
		stop
     End Select
     Return
   End Function mu_func

   !--------------------------------------------------

   Function gamma_func(x,mu)  Result(gamma)
     Implicit None
     Real(wp)   :: x(:), mu, gamma(Size(x,1))     
     gamma= 1._wp-2*x*mu+mu**2
     Return
   End Function gamma_func
   
   !--------------------------------------------------

   Function chi_func(x,mu)  Result(chi)
     Implicit None
     Real(wp)   :: x(:), mu, chi(Size(x,1))     
     chi = (x-mu)/Sqrt(gamma_func(x,mu))   
     Return
   End Function chi_func

   !---------------------------------------------------------------

   Function Ass_Legendre(l,m,x) Result(alp)
     ! Calculates the Associate Legendre polynomials
     Use Legendre, Only : asslegpoly
     Implicit None
     Integer  :: l,m, itmp, tmp
     Real(wp) :: x(:), alp(Size(x))
     ! NR-routine
     if (m>=0) then
        alp =  asslegpoly(l,m,x)
     else
        alp =  asslegpoly(l,-m,x)
        tmp=1
        do itmp=l,l-m
           tmp=tmp*itmp
        enddo
           alp =  (-1._wp)**m * alp / tmp
        end if
     Return
   End Function Ass_Legendre

   !---------------------------------------------------------------

   Function Deriv_Ass_Legendre(l,m,x) Result(dalp)
     ! Calculates the first derivative of the Associate Legendre polynomials
     Implicit None
     Integer  :: l, m
     Real(wp) :: x(:), dalp(Size(x))
     ! NR-routine
     dalp  = -m*x/(1-x**2)*Real(Ass_Legendre(l,m,x),wp)
     If (abs(m+1)<=l) Then
        dalp = dalp - Ass_Legendre(l,m+1,x)/Sqrt(1._wp-x**2)
     Endif
   End Function  Deriv_Ass_Legendre

   !---------------------------------------------------------------




End Module Boundary_mod





