Module Global_definitions

  ! Some operating system specific stuff
  Include 'os.h'


  !===========================================================
  ! This module mainly contains various kinds of definitions
  ! of derived types
  !===========================================================



  ! Definitions of precisions
  INTEGER, PARAMETER :: LGT = KIND(.true.) 
  INTEGER, PARAMETER :: I4B = SELECTED_INT_KIND(9) 
  INTEGER, PARAMETER :: SP = KIND(1.0) 
  INTEGER, PARAMETER :: DP = KIND(1.0D0) 
  INTEGER, PARAMETER :: WP = KIND(1.0D0) 
  INTEGER, PARAMETER :: SPC = KIND((1.0,1.0)) 
  INTEGER, PARAMETER :: DPC = KIND((1.0,1.0)) 
  INTEGER, PARAMETER :: WPC = KIND((1.0D0,1.0D0)) 
  REAL(SP), PARAMETER :: PI_S=3.141592653589793238462643383279502884197_sp 
  REAL(DP), PARAMETER :: PI_D=3.141592653589793238462643383279502884197_dp 
  REAL(WP), PARAMETER :: PI=3.141592653589793238462643383279502884197_dp 
  
  Complex(wpc), Parameter  ::  imu = (0._wp,1._wp)

  !-------------------------------------------------------
  !------ Some type declarations -------------------------
  !------------------------------------------------------- 

  Type parameters
     ! Properties of the incident beam
     Real(wp)			::   energy_min              ! Photon energy range (eV)
     Real(wp)			::   energy_max   
     Real(wp)			::   theta0                  ! Incident angle of light
     Real(wp)			::   phi0_calc               ! Parameters for calculations : angle between field and substrate 
     Real(wp)			::   theta0_calc			                   
     Character(len=1)	        ::   polarization            ! Polarization of the incident beam
      
     ! Geometrical parameters 
     character(len=10)	        ::   geometry                ! Film geometry : film-island 
     Real(wp)                   ::   film_thickness1         ! Film thickness 1 
     Real(wp)                   ::   film_thickness2         ! Film thickness 2 
      
     ! Material parameters 
     Character(len=10)	        ::   island                  ! Island material 
     Character(len=10)	        ::   substrate               ! Substrate material 
     Character(len=10)	        ::   coating                 ! Coating material 
      
     ! Some control parameters for the metal 
     Character(len=20)	        ::   mean_free_path          ! Nature of mean free path 'none' 'finite_size' 'manual'  
     Logical			::   surface_effects         ! Surface effects        
     Real(wp)			::   temperature             ! Temperature 
     Logical			::   thermoreflectance       ! Thermoreflectance 
     
     ! Geometrical properties of the islands and the substrate
     Real(wp)			::   R                       ! Island radius (nm)
     Real(wp)			::   Rapparent               ! Apparent island radius (nm)
     Real(wp)			::   tr                      ! Truncation parameter
     Real(wp)			::   div_surface             ! Position of the dividing surface (nm) 
     Logical			::   above                   ! MP above substrate?
     Real(wp)			::   volume                  ! Volume of the cluster 
     Real(wp)			::   SR	                     ! Aspect ratio of the cluster 
     Logical			::   normalization           ! Normalization by the volume in the polarizability 
      
     
     ! "Island-island" parameters 
     Character(len=15)	        ::   network                 ! Type of network : MFT,RPT,square,hexagonal
     Real(wp)			::   coverage                ! The coverage (dimensionless) 
     Real(wp)			::   density                 ! The density(related to coverage) 
     Real(wp)			::   lattice_const           ! The lattice constant 
     Character(10)		::   interaction             ! Island-island interaction 
     Integer			::   m                       ! m=1 dipole calculations 
     ! m=2 quadrupole calculations
      
     ! Input and Output parameters 
     Character(len=200)	        ::   path                     ! Path for the dielectric constants 
     Character(len=200)         ::   outfilename              ! Output filename 
     Character(len=20)          ::   fresnel                  ! Formulas for optics 
     Character(len=4)           ::   out                      ! Nature of output dr dt r t a 
     Character(len=200)         ::   expfilename              ! Experimental filename
     Logical			::   compa                    ! Comparison Exp-Theory 
     Logical			::   strenght                 ! Output for the multipoles coefficients 
     Logical			::   polarizability           ! Output for the polarizabilties 
      
     ! Control parameters for the boundary conditions 
     Character(len=10)          ::   bound                    ! Type of calculation (none-BC-Pot) 
     Real(wp)			::   BC_energy                ! Energy for the calculation 
     Character(len=15)	        ::   scaling                  ! Nature of the scaling parameter : none,incident_field,mean_value 
      
     ! Numerics parameters
     Integer	                ::   Nenergy                  ! # energy 
     Integer			::   Levels                   ! # levels used in the lattice sum
     Integer			::   Mpole_order              ! # multipoles
     Real(wp)			::   MPpos                    ! Position of the MP 
      
     ! Integrations and linear system parameters 
     Character(len=20)	        ::   int_method	              ! Method of integration (gauleg,quadpack) 
     Integer		        ::   Nint                     ! # points in Gauss-Legendre integration  
     ! and maximun # of points in quadpack 
     Integer			::   limit                    ! Number of allowed subdivision in Quadpack 
     Real(wp)			::   epsabs                   ! Absolute precison in integral 
     Real(wp)			::   epsrel                   ! Relative precision in integral 
     Real(wp)			::   epslin                   ! Relative error in the solution of Ax = b 
     Integer			::   MP_precision             ! Number of digit for the multiple precision 
      
     ! Some physical constants
     Real(wp)			::   eps_vacuum 
     Real(wp)			::   hbar                     ! Plancks constant (eV s)
     Real(wp)			::   c                        ! Speed of light
     
     ! Various parameters
     Real(wp), Pointer, Dimension(:)      :: energy   
     Complex(wpc), Pointer, Dimension(:)  :: eps_island   
     Complex(wpc), Pointer, Dimension(:)  :: eps_substrate  
     
     
  End Type parameters
  
  !-------------------------------------------------------
  
  Type surface_constitutive
     Complex(wpc)  :: gamma
     Complex(wpc)  :: beta
     Complex(wpc)  :: delta
     Complex(wpc)  :: tau
  End Type surface_constitutive
  
  !-------------------------------------------------------
  
  Type invariants
     Complex(wpc)  :: e
     Complex(wpc)  :: delta_e
     Complex(wpc)  :: tau
     Real(wp)      :: c
  End Type invariants
  
  !-------------------------------------------------------
  
  Type mumubar
     Real(wp)		:: mu(2)   ! 1:mu  2:mubar
  End Type mumubar
  
  !-------------------------------------------------------
  
  Type Integrals
     Real(wp)       ::     Q
     Type(mumubar)  ::     K
     Type(mumubar)  ::     L
     Type(mumubar)  ::     M  
     Type(mumubar)  ::     N  
  End Type Integrals
  
  !-------------------------------------------------------
  
  Type Polynomials
     Real(wp)       ::        P        !   P(l,m)(x)
     Type(mumubar)  ::       PP        !   P(l,m)(chi)
     Type(mumubar)  ::     dPdx        !   dP/dx(l,m)(chi)      
  End Type Polynomials
   
  !------------------------------------------------------- 
  ! Generic routine interface for the arithmetic progression 
  Interface arth 
     Module Procedure arth_sp, arth_dp 
  End Interface 
   
   
  Contains 
   
  !------------------------------------------------------------------- 
  Function arth_sp(first,increment,n) 
     
    Implicit None 
    Real(sp)	              ::  first,increment 
    Integer                   ::  n,i 
    Real(sp),Dimension(n)     ::  arth_sp 
     
    arth_sp(1) = first 
    Do i=2,n 
       arth_sp(i) = arth_sp(i-1)+increment 
    Enddo 
    Return 
     
  End Function arth_sp 
  !------------------------------------------------------------------- 
  Function arth_dp(first,increment,n) 
     
    Implicit None 
    Real(dp)                   ::  first,increment 
    Integer                    ::  n,i 
    Real(dp),Dimension(n)      ::  arth_dp 
     
    arth_dp(1) = first 
    Do i=2,n 
       arth_dp(i) = arth_dp(i-1)+increment 
    Enddo 
    Return 
     
  End Function arth_dp 
   
  !---------------------------------------------------------------------- 
  Function locate(xx,x) 
    Implicit None 
    Real, Dimension(:), Intent(IN)	:: xx 
    Real, Intent(IN)                    :: x 
    Integer                             :: locate 
    Integer                             :: n,jl,jm,ju 
    Logical                             :: ascnd 
     
    n=size(xx) 
    ascnd = (xx(n) >= xx(1)) 
    jl=0 
    ju=n+1 
    do 
       if (ju-jl <= 1) exit 
       jm=(ju+jl)/2 
       if (ascnd .eqv. (x >= xx(jm))) then 
          jl=jm 
       else 
          ju=jm 
       end if 
    end do 
    if (x == xx(1)) then 
       locate=1 
    else if (x == xx(n)) then 
       locate=n-1 
    else 
       locate=jl 
    end if 
     
  End Function locate 
  
End Module Global_definitions


