
!----------------------------------------------------------------------
!              SPHERE part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: integral_mod.f90,v 1.1 2001/08/11 15:42:12 ingves Exp ingves $
!-----------------------------------------------------------------------



Module Integral_mod

Use Global_definitions
! Use Graphics

Implicit None
 
Contains

!----------------------------------------------------------------
!
!			The main routine for getting the integrals
!
!----------------------------------------------------------------
	Subroutine get_integrals(tr,int,param)

	 Implicit None
	 Real(wp)						  :: tr
     Type(parameters)                 :: param
     Type(integrals)                  :: Int(0:,0:,0:)
	 
	 ! Choose the integration scheme
	 Select Case(Trim(Adjustl(param%int_method)))
	 !--------------------------------------------
	 ! Gauss-Legendre with fixed numbers of points
	 !--------------------------------------------
	 Case('gauleg')
	  Call get_integrals_gauleg(tr,int,param)
	 !--------------------------------------------
	 ! Quadpack : iterative quadrature
	 !--------------------------------------------
	 Case('quadpack')
	  Call get_integrals_quadpack(tr,int,param)
	 Case Default
	  Write(6,*) ' Error in get_integrals for param%int_method'
	  Pause
	  Stop
	 End Select 	 
		
	 Return

	End Subroutine get_integrals



!----------------------------------------------------------------
!
! Fixed Gauss-Legendre integration method using NUMERICAL RECIPES
!
!----------------------------------------------------------------
	

   Subroutine get_integrals_gauleg(tr,int,param)

     Use Legendre, Only : Gausslegendre
     Implicit None     
     Type(parameters)                 :: param
     Type(integrals)                  :: Int(0:,0:,0:)
     Type(polynomials), Allocatable   :: poly(:,:,:)
     Integer                          :: l1,l2,l,m,imu,mpo,mm
     Real(wp)                         :: tr,mu(2),integrand(param%Nint) 
     Real(wp), Dimension(param%Nint)  :: w,x
     Real(wp), Dimension(2,param%Nint):: gamma, chi
	 	
     ! Some abbreviations
     mpo      = param%Mpole_order
	 mm		  = param%m

     ! Get the weights and abscissas for the Gauss-quadrature
     ! w:weights, x:abscissas
     Call gausslegendre(-1._wp,tr,x,w)

     ! Calculate mu and gamma 
     Do imu =1,2
        mu(imu)      = mu_func(param%tr,imu,param%MPpos)
        gamma(imu,:) = gamma_func(x,mu(imu))
        chi(imu,:)   = chi_func(x,mu(imu))
     Enddo

	 ! Initialization of integrals
	 Int(:,:,:)%Q			= 0._wp
	 Do m=1,2
	  Int(:,:,:)%K%mu(m)	= 0._wp
	  Int(:,:,:)%L%mu(m)	= 0._wp
	  Int(:,:,:)%M%mu(m)	= 0._wp
	 Enddo

     !----------------------------------------------
     !---  Calculates the Polynomials
     !----------------------------------------------
     ! Allocate space for the assosiated legendre polynomials
     Allocate( poly(0:mpo,0:mm,param%Nint) )
	 ! Initialize the polynomials
	 poly(:,:,:)%P			= 0._wp
	 Do m=1,2
	  poly(:,:,:)%PP%mu(m)	= 0._wp
	  poly(:,:,:)%dPdx%mu(m)= 0._wp
	 Enddo

     ! Only needs the P(l,m)(x) for m=0 l=0
     poly(0,0,:)%P  = 1._wp

     ! Loops over all "quantum numbers" for m=0,1
     Do l=1,mpo
        Do m=0,1
		   ! P(l,m)(x)
           poly(l,m,:)%P  = Ass_Legendre(l,m,x)
!           call plot(real(x(:),sp),real(poly(l,m,:)%P,kind(1.0)))
           Do imu=1,2  ! Loop over mu    
              ! P(l,m)(chi)
              poly(l,m,:)%PP%mu(imu) =  Ass_Legendre(l,m,chi(imu,:)) 
              ! The derivative of P(l,m)(chi)
              poly(l,m,:)%dPdx%mu(imu) = Deriv_Ass_Legendre(l,m,chi(imu,:))
!			call plot(real(x(:),sp),real(poly(l,m,:)%dPdx%mu(imu),kind(1.0)))
            Enddo
        Enddo
     Enddo

	 If(mm==2) Then
	 ! Loops over all "quantum numbers" for m=2 if necessary
	 Do l=2,mpo
        Do imu=1,2  ! Loop over mu
           ! P(l,m)(x)
            poly(l,2,:)%P  = Ass_Legendre(l,2,x)    
           ! P(l,m)(chi)
            poly(l,2,:)%PP%mu(imu) =  Ass_Legendre(l,2,chi(imu,:)) 
           ! The derivative of P(l,m)(chi)
            poly(l,2,:)%dPdx%mu(imu) = Deriv_Ass_Legendre(l,2,chi(imu,:))
!           call plot(real(poly(l,m,:)%dPdx%mu(imu),kind(1.0))
        Enddo
     Enddo
	 Endif

     !----------------------------------------------
     !---  Calculates the Integrals
     !----------------------------------------------

	 !-------------------------------
	 ! Special Case for l1=0,l2=0,m=0 
	 !-------------------------------
	  Int(0,0,0)%Q = tr + 1._wp	 

	 !--------------------------
	 ! Special Case for l1=0,m=0 
	 !--------------------------
	 Do l2=0,mpo
              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
              integrand = poly(l2,0,:)%P 
              Int(0,l2,0)%Q = Sum( w*integrand )

              Do imu=1,2  ! Loop over mu-type
                 !------------------------------------------------
                 ! --- K-integrals 
                 !------------------------------------------------
                 integrand = poly(l2,0,:)%PP%mu(imu)    &
                           * gamma(imu,:)**(-(l2+1._wp)/2._wp)          
                 Int(0,l2,0)%K%mu(imu) = Sum( w * integrand )
                 !------------------------------------------------
                 ! --- L-integrals 
                 !------------------------------------------------
                 integrand = -(l2+1._wp)*gamma(imu,:)**(-0.5_wp*l2-1.5_wp)     &
                            *(1._wp-mu(imu)*x)*poly(l2,0,:)%PP%mu(imu)       &
                           + gamma(imu,:)**(-0.5_wp*l2-2._wp)                &
                              *(x*gamma(imu,:)-(x-mu(imu))*(1._wp-mu(imu)*x))&
                              *poly(l2,0,:)%dPdx%mu(imu) 
                 Int(0,l2,0)%L%mu(imu)   =  Sum(w * integrand )
                 !------------------------------------------------
                 ! --- M-integrals 
                 !------------------------------------------------
                 integrand = poly(l2,0,:)%PP%mu(imu)    &
                           * gamma(imu,:)**(l2/2._wp)          
                 Int(0,l2,0)%M%mu(imu) = Sum( w * integrand )
                 !------------------------------------------------
                 ! --- N-integrals 
                 !------------------------------------------------
                 integrand =  l2*gamma(imu,:)**(0.5_wp*l2-1._wp)              &
                              *(1._wp-mu(imu)*x)*poly(l2,0,:)%PP%mu(imu)     &
                            +gamma(imu,:)**(0.5_wp*l2-1.5_wp)                &
                              *(x*gamma(imu,:)-(x-mu(imu))*(1._wp-mu(imu)*x))&
                              *poly(l2,0,:)%dPdx%mu(imu)                                               
                 Int(0,l2,0)%N%mu(imu)   =  Sum(w * integrand )
             Enddo	 
		Enddo

	 !--------------------------
	 ! Special Case for l2=0,m=0 
	 !--------------------------
       Do l1=1,mpo
              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
              integrand = poly(l1,0,:)%P 
              Int(l1,0,0)%Q = Sum( w*integrand )

              Do imu=1,2  ! Loop over mu-type
                 !------------------------------------------------
                 ! --- K-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,0,:)%P * gamma(imu,:)**(-0.5_wp)          
                 Int(l1,0,0)%K%mu(imu) = Sum( w * integrand )
                 !------------------------------------------------
                 ! --- L-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,0,:)%P       &
                         *(-gamma(imu,:)**(-1.5_wp)*(1._wp-mu(imu)*x)   &
                           + gamma(imu,:)**(-2._wp)                &
                              *(x*gamma(imu,:)-(x-mu(imu))*(1._wp-mu(imu)*x)) )
                 Int(l1,0,0)%L%mu(imu)   =  Sum(w * integrand )
                 !------------------------------------------------
                 ! --- M-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,0,:)%P          
                 Int(l1,0,0)%M%mu(imu) = Sum( w * integrand )
                 !------------------------------------------------
                 ! --- N-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,0,:)%P   & 
                          *( gamma(imu,:)**(-1.5_wp) &
                           *( x*gamma(imu,:)-(x-mu(imu))*(1._wp-mu(imu)*x) ) &
                               )                                                 
                 Int(l1,0,0)%N%mu(imu)   =  Sum(w * integrand )
             Enddo
		Enddo
			 
	 !-------------------------------------------		    
     ! Loops over all "quantum numbers" for m=0,1
	 !-------------------------------------------
     Do l1=1,mpo
        Do l2=1,mpo
           Do m=0,1
              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
              integrand = poly(l1,m,:)%P*poly(l2,m,:)%P 
              Int(l1,l2,m)%Q = Sum( w*integrand )

              Do imu=1,2  ! Loop over mu-type
                 !------------------------------------------------
                 ! --- K-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,m,:)%P*poly(l2,m,:)%PP%mu(imu)    &
                           * gamma(imu,:)**(-(l2+1._wp)/2._wp)          
                 Int(l1,l2,m)%K%mu(imu) = Sum( w * integrand )
                 !------------------------------------------------
                 ! --- L-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,m,:)%P                                  &
                         *(-(l2+1._wp)*gamma(imu,:)**(-0.5_wp*l2-1.5_wp)     &
                            *(1._wp-mu(imu)*x)*poly(l2,m,:)%PP%mu(imu)       &
                           + gamma(imu,:)**(-0.5_wp*l2-2._wp)                &
                              *(x*gamma(imu,:)-(x-mu(imu))*(1._wp-mu(imu)*x))&
                              *poly(l2,m,:)%dPdx%mu(imu) )
                 Int(l1,l2,m)%L%mu(imu)   =  Sum(w * integrand )
                 !------------------------------------------------
                 ! --- M-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,m,:)%P*poly(l2,m,:)%PP%mu(imu)    &
                           * gamma(imu,:)**(l2/2._wp)          
                 Int(l1,l2,m)%M%mu(imu) = Sum( w * integrand )
                 !------------------------------------------------
                 ! --- N-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,m,:)%P                                  & 
                          *( l2*gamma(imu,:)**(0.5_wp*l2-1._wp)              &
                              *(1._wp-mu(imu)*x)*poly(l2,m,:)%PP%mu(imu)     &
                            +gamma(imu,:)**(0.5_wp*l2-1.5_wp)                &
                              *(x*gamma(imu,:)-(x-mu(imu))*(1._wp-mu(imu)*x))&
                              *poly(l2,m,:)%dPdx%mu(imu) )                                                 
                 Int(l1,l2,m)%N%mu(imu)   =  Sum(w * integrand )
             Enddo
          Enddo
       Enddo
    Enddo

	If(mm==2) Then
     ! Loops over all "quantum numbers" for m=2 if necessary
	  Do l1=2,mpo
        Do l2=2,mpo
              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
              integrand = poly(l1,2,:)%P*poly(l2,2,:)%P 
              Int(l1,l2,2)%Q = Sum( w*integrand )

              Do imu=1,2  ! Loop over mu-type
                 !------------------------------------------------
                 ! --- K-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,2,:)%P*poly(l2,2,:)%PP%mu(imu)    &
                           * gamma(imu,:)**(-(l2+1._wp)/2._wp)          
                 Int(l1,l2,2)%K%mu(imu) = Sum( w * integrand )
                 !------------------------------------------------
                 ! --- L-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,2,:)%P                                  &
                         *(-(l2+1._wp)*gamma(imu,:)**(-0.5_wp*l2-1.5_wp)     &
                            *(1._wp-mu(imu)*x)*poly(l2,2,:)%PP%mu(imu)       &
                           + gamma(imu,:)**(-0.5_wp*l2-2._wp)                &
                              *(x*gamma(imu,:)-(x-mu(imu))*(1._wp-mu(imu)*x))&
                              *poly(l2,2,:)%dPdx%mu(imu))
                 Int(l1,l2,2)%L%mu(imu)   =  Sum(w * integrand )
                 !------------------------------------------------
                 ! --- M-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,2,:)%P*poly(l2,2,:)%PP%mu(imu)    &
                           * gamma(imu,:)**(l2/2._wp)          
                 Int(l1,l2,2)%M%mu(imu) = Sum( w * integrand )
                 !------------------------------------------------
                 ! --- N-integrals 
                 !------------------------------------------------
                 integrand = poly(l1,2,:)%P                                  & 
                          *( l2*gamma(imu,:)**(0.5_wp*l2-1._wp)              &
                              *(1._wp-mu(imu)*x)*poly(l2,2,:)%PP%mu(imu)     &
                            +gamma(imu,:)**(0.5_wp*l2-1.5_wp)                &
                              *(x*gamma(imu,:)-(x-mu(imu))*(1._wp-mu(imu)*x))&
                              *poly(l2,2,:)%dPdx%mu(imu) )                                                 
                 Int(l1,l2,2)%N%mu(imu)   =  Sum(w * integrand )
				Enddo
			Enddo
		 Enddo
		Endif

    Return

  End Subroutine get_integrals_gauleg

  !---------------------------------------------------------------

   Function mu_func(tr,Type,mupos)  Result(mu)
     ! calculated the z-coordiate of the multipole (MP) and image MP
     Implicit None
     Real(wp)		    :: mupos     
     Integer            :: Type
     Real(wp)           :: tr,mu

     Select Case(Type)
     Case(1)
        ! The MP
        mu = mupos
     Case(2)
        ! The image MP
        mu = 2._wp*tr - mupos
     Case default
        Pause 'ERROR in mu_func'
		Stop
     End Select
     Return
   End Function mu_func

   !--------------------------------------------------

   Function gamma_func(x,mu)  Result(gamma)
     Implicit None
     Real(wp)   :: x(:), mu, gamma(Size(x,1))     
     gamma= 1._wp-2*x*mu+mu**2
     Return
   End Function gamma_func
   
   !--------------------------------------------------

   Function chi_func(x,mu)  Result(chi)
     Implicit None
     Real(wp)   :: x(:), mu, chi(Size(x,1))     
     chi = (x-mu)/Sqrt(gamma_func(x,mu))   
     Return
   End Function chi_func

   !---------------------------------------------------------------

   Function Ass_Legendre(l,m,x) Result(alp)
     ! Calculates the Associate Legendre polynomials
     Use Legendre, Only : asslegpoly
     Implicit None
     Integer  :: l,m
     Real(wp) :: x(:), alp(Size(x))
     ! NR-routine
     alp =  asslegpoly(l,m,x)
     Return
   End Function Ass_Legendre

   !---------------------------------------------------------------

   Function Deriv_Ass_Legendre(l,m,x) Result(dalp)
     ! Calculates the first derivative of the Associate Legendre polynomials
     Implicit None
     Integer  :: l, m
     Real(wp) :: x(:), dalp(Size(x))

     ! Recursive relation
     dalp  = (l-m+1)*Ass_Legendre(l+1,m,x)-(l+1)*x*Ass_Legendre(l,m,x)
     dalp = dalp/(x**2-1)
	 return
   End Function  Deriv_Ass_Legendre


!----------------------------------------------------------------
!
!	Adaptative quadrature integration method using QUADPACK
!
!----------------------------------------------------------------

   Subroutine get_integrals_quadpack(tr,int,param)

     Implicit None     
     Type(parameters)                 :: param
     Type(integrals)                  :: Int(0:,0:,0:)
     Integer                          :: l1,l2,m
     Real(wp)                         :: tr,mu(2),mupos
	 Integer						  :: imu,mpo,mm

	 
	 Common /index/				l1,l2,m
	 Common /mupoint/			mupos

     ! Some abbreviations
     mpo      = param%Mpole_order
	 mm		  = param%m

     ! Calculate mu position in the main coordiante system
     Do imu =1,2
        mu(imu)  =  mu_func(param%tr,imu,param%MPpos)
     Enddo

	 ! Initialization of integrals
	 Int(:,:,:)%Q			= 0._wp
	 Do m=1,2
	  Int(:,:,:)%K%mu(m)	= 0._wp
	  Int(:,:,:)%L%mu(m)	= 0._wp
	  Int(:,:,:)%M%mu(m)	= 0._wp
	 Enddo

     !----------------------------------------------
     !---  Calculates the Integrals
     !---------------------------------------------- 
	 
	 !-------------------------------
	 ! Special Case for l1=0,l2=0,m=0 
	 !-------------------------------
	  Int(0,0,0)%Q = tr + 1._wp	 
	  
	 !--------------------------
	 ! Special Case for l1=0,m=0 
	 !--------------------------
	   l1	=	0
	   m	=	0	   	  	 
       Do l2=1,mpo
              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
			  Call Quadpack(Int_Q,tr,Int(l1,l2,m)%Q,param)
              Do imu=1,2  ! Loop over mu-type
				 mupos = mu(imu)
                 !------------------------------------------------
                 ! --- K-integrals 
                 !------------------------------------------------         
                 Call Quadpack(Int_K,tr,Int(l1,l2,m)%K%mu(imu),param)
                 !------------------------------------------------
                 ! --- L-integrals 
                 !------------------------------------------------
				 Call Quadpack(Int_L,tr,Int(l1,l2,m)%L%mu(imu),param)
                 !------------------------------------------------
                 ! --- M-integrals 
                 !------------------------------------------------
				 Call Quadpack(Int_M,tr,Int(l1,l2,m)%M%mu(imu),param)
                 !------------------------------------------------
                 ! --- N-integrals 
                 !------------------------------------------------
				Call Quadpack(Int_N,tr,Int(l1,l2,m)%N%mu(imu),param)
             Enddo	 
		Enddo
	 
	 !--------------------------
	 ! Special Case for l2=0,m=0 
	 !--------------------------
	   l2	=	0
	   m	=	0	   	  	 
       Do l1=1,mpo
              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
			  Call Quadpack(Int_Q,tr,Int(l1,l2,m)%Q,param)
              Do imu=1,2  ! Loop over mu-type
				 mupos = mu(imu)
                 !------------------------------------------------
                 ! --- K-integrals 
                 !------------------------------------------------         
                 Call Quadpack(Int_K,tr,Int(l1,l2,m)%K%mu(imu),param)
                 !------------------------------------------------
                 ! --- L-integrals 
                 !------------------------------------------------
				 Call Quadpack(Int_L,tr,Int(l1,l2,m)%L%mu(imu),param)
                 !------------------------------------------------
                 ! --- M-integrals 
                 !------------------------------------------------
				 Call Quadpack(Int_M,tr,Int(l1,l2,m)%M%mu(imu),param)
                 !------------------------------------------------
                 ! --- N-integrals 
                 !------------------------------------------------
				Call Quadpack(Int_N,tr,Int(l1,l2,m)%N%mu(imu),param)
             Enddo	 
		Enddo	 
	 
	 
	 !-------------------------------------------   
     ! Loops over all "quantum numbers" for m=0,1
	 !-------------------------------------------
     Do l1=1,mpo
        Do l2=1,mpo
           Do m=0,1
              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
			  Call Quadpack(Int_Q,tr,Int(l1,l2,m)%Q,param)
              Do imu=1,2  ! Loop over mu-type
				 mupos = mu(imu)
                 !------------------------------------------------
                 ! --- K-integrals 
                 !------------------------------------------------         
                 Call Quadpack(Int_K,tr,Int(l1,l2,m)%K%mu(imu),param)
                 !------------------------------------------------
                 ! --- L-integrals 
                 !------------------------------------------------
				 Call Quadpack(Int_L,tr,Int(l1,l2,m)%L%mu(imu),param)
                 !------------------------------------------------
                 ! --- M-integrals 
                 !------------------------------------------------
				 Call Quadpack(Int_M,tr,Int(l1,l2,m)%M%mu(imu),param)
                 !------------------------------------------------
                 ! --- N-integrals 
                 !------------------------------------------------
				Call Quadpack(Int_N,tr,Int(l1,l2,m)%N%mu(imu),param)
             Enddo
          Enddo
       Enddo
    Enddo

	If(mm==2) Then
     ! Loops over all "quantum numbers" for m=2 if necessary
	  m = mm
	  Do l1=2,mpo
        Do l2=2,mpo
              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
			  Call Quadpack(Int_Q,tr,Int(l1,l2,m)%Q,param)
              Do imu=1,2  ! Loop over mu-type
				 mupos = mu(imu)
                 !------------------------------------------------
                 ! --- K-integrals 
                 !------------------------------------------------         
                 Call Quadpack(Int_K,tr,Int(l1,l2,m)%K%mu(imu),param)
                 !------------------------------------------------
                 ! --- L-integrals 
                 !------------------------------------------------
				 Call Quadpack(Int_L,tr,Int(l1,l2,m)%L%mu(imu),param)
                 !------------------------------------------------
                 ! --- M-integrals 
                 !------------------------------------------------
				 Call Quadpack(Int_M,tr,Int(l1,l2,m)%M%mu(imu),param)
                 !------------------------------------------------
                 ! --- N-integrals 
                 !------------------------------------------------
				Call Quadpack(Int_N,tr,Int(l1,l2,m)%N%mu(imu),param)
				Enddo
			Enddo
		 Enddo
		Endif

    Return

  End Subroutine get_integrals_quadpack

  !---------------------------------------------------------------
   Function gamma_func1(x,mu)  Result(gamma)
     Implicit None
     Real(wp)   :: x, mu, gamma     

     gamma= 1._wp-2*x*mu+mu**2
     Return
   End Function gamma_func1
   
   !--------------------------------------------------

   Function chi_func1(x,mu)  Result(chi)
     Implicit None
     Real(wp)   :: x, mu, chi
     chi = (x-mu)/Sqrt(gamma_func1(x,mu))   
     Return
   End Function chi_func1

   !---------------------------------------------------------------

   Function Ass_Legendre1(l,m,x) Result(alp)
     ! Calculates the Associate Legendre polynomials
     Use Legendre, Only : asslegpoly1
     Implicit None
     Integer  :: l,m
     Real(wp) :: x, alp
     ! NR-routine
     alp =  asslegpoly1(l,m,x)
     Return
   End Function Ass_Legendre1

   !---------------------------------------------------------------

   Function Deriv_Ass_Legendre1(l,m,x) Result(dalp)
     ! Calculates the first derivative of the Associate Legendre polynomials
     Implicit None
     Integer  :: l, m
     Real(wp) :: x, dalp
     ! Recursive relation
     dalp  = (l-m+1)*Ass_Legendre1(l+1,m,x)-(l+1)*x*Ass_Legendre1(l,m,x)
     dalp = dalp/(x**2-1)
	 
	 Return	 
   End Function  Deriv_Ass_Legendre1

  !---------------------------------------------------------------
    Function Int_Q(x) Result(Q_integrand)

	Implicit None
	Real(wp)					::	x,Q_integrand
	Integer						::	l1,l2,m

	Common /index/				l1,l2,m

	Q_integrand	=	Ass_Legendre1(l1,m,x)*Ass_Legendre1(l2,m,x)
	Return

    End Function Int_Q

   !--------------------------------------------------------------- 
    Function Int_K(x) Result(K_integrand)

	Implicit None
	Real(wp)					::	x,K_integrand
	Integer						::	l1,l2,m
	Real(wp)					::	mu
	Real(wp)					::	chi,gamma

	Common /index/				l1,l2,m
	Common /mupoint/			mu

	chi			=	chi_func1(x,mu)
	gamma		=	gamma_func1(x,mu)
	K_integrand	=	&
	Ass_Legendre1(l1,m,x)*Ass_Legendre1(l2,m,chi)* &
	gamma**(-(l2+1._wp)/2._wp)

	Return

   End Function Int_K

   !--------------------------------------------------------------- 
    Function Int_L(x) Result(L_integrand)

	Implicit None
	Real(wp)					::	x,L_integrand
	Integer						::	l1,l2,m
	Real(wp)					::	mu
	Real(wp)					::	chi,gamma

	Common /index/				l1,l2,m
	Common /mupoint/			mu

	chi			=	chi_func1(x,mu)
	gamma		=	gamma_func1(x,mu)
	L_integrand	=	&
	Ass_Legendre1(l1,m,x) * ( &
	-(l2+1._wp)*gamma**(-0.5_wp*l2-1.5_wp)*(1._wp-mu*x)*Ass_Legendre1(l2,m,chi) + &
	gamma**(-0.5_wp*l2-2._wp)*(x*gamma-(x-mu)*(1._wp-mu*x))* &
	Deriv_Ass_Legendre1(l2,m,chi) &
	)

	Return

   End Function Int_L
   !--------------------------------------------------------------- 
    Function Int_M(x) Result(M_integrand)

	Implicit None
	Real(wp)					::	x,M_integrand
	Integer						::	l1,l2,m
	Real(wp)					::	mu
	Real(wp)					::	chi,gamma

	Common /index/				l1,l2,m
	Common /mupoint/			mu

	chi			=	chi_func1(x,mu)
	gamma		=	gamma_func1(x,mu)
	M_integrand	=	&
	Ass_Legendre1(l1,m,x)*Ass_Legendre1(l2,m,chi)* &
	gamma**(l2/2._wp)

	Return
   End Function Int_M

    !--------------------------------------------------------------- 
    Function Int_N(x) Result(N_integrand)

	Implicit None
	Real(wp)					::	x,N_integrand
	Integer						::	l1,l2,m
	Real(wp)					::	mu
	Real(wp)					::	chi,gamma

	Common /index/				l1,l2,m
	Common /mupoint/			mu

	chi			=	chi_func1(x,mu)
	gamma		=	gamma_func1(x,mu)
	N_integrand	=	&
	Ass_Legendre1(l1,m,x) * ( &
	l2*gamma**(0.5_wp*l2-1._wp)*(1._wp-mu*x)*Ass_Legendre1(l2,m,chi) + &
	gamma**(0.5_wp*l2-1.5_wp)*(x*gamma-(x-mu)*(1._wp-mu*x))* &
	Deriv_Ass_Legendre1(l2,m,chi) &
	)

	Return

   End Function Int_N

  !--------------------------------------------------------------- 
   Subroutine Quadpack(F,tr,Int,param)

	! Subroutine for calling the package of quadrature adaptative 
	! integration
	Implicit None
	Real(wp),External				::	F
	Real(wp)						::	tr
	Type(parameters)				::	param
	Real(wp)						::	Int
	Real(wp)						::	epsrel,epsabs,abserr
	Integer,Parameter				::	key=4
	Integer							::	neval,ier,limit,lenw,last
	Integer,Allocatable				::	iwork(:)
	Real(wp),Allocatable			::	work(:)


	! Some initial abreviations
	epsabs	=	param%epsabs
	epsrel	=	param%epsrel
	limit	=	param%limit
	lenw	=	4*limit
	! Allocation of the working arrays
	Allocate(iwork(limit),work(lenw))

	!-----------------------------
	! Call of the quapack routines
	!-----------------------------
	Select Case(wp)
	!---------------------------
	! Single precision version
	!---------------------------
	Case(sp)
	Call qag(F,-1._wp,tr,epsabs,epsrel,key,Int,abserr, &
			neval,ier,limit,lenw,last,iwork,work)
	!---------------------------
	! Double precision version
	!---------------------------
	Case(dp)
	Call dqag(F,-1._wp,tr,epsabs,epsrel,key,Int,abserr, &
			neval,ier,limit,lenw,last,iwork,work)
	End Select
	
	! Error code in Quadpack
	If(ier/=0) Then
	! Output of some control parameters
	 Open(53,file='error_int.dat',status='unknown')
	 Write(53,'(i2,3(E15.5),2i5)') &
	   ier,Int,abserr/Abs(Int),abserr,neval,last
	Endif


	! Deallocation of arrays
	Deallocate(iwork,work)

   End Subroutine Quadpack

  !--------------------------------------------------------------- 
   Subroutine Quadpack_I(F,bound,Int,param)

	! Subroutine for calling the package of quadrature adaptative 
	! integration over infinite interval : a,+inf
	Implicit None
	Real(wp),External				::	F
	Real(wp)						::	bound
	Type(parameters)				::	param
	Real(wp)						::	Int
	Real(wp)						::	epsrel,epsabs,abserr
	Integer							::	neval,ier,limit,lenw,last
	Integer,Allocatable				::	iwork(:)
	Real(wp),Allocatable			::	work(:)


	! Some initial abreviations
	epsabs	=	param%epsabs
	epsrel	=	param%epsrel
	limit	=	param%limit
	lenw	=	4*limit
	! Allocation of the working arrays
	Allocate(iwork(limit),work(lenw))

	!-----------------------------
	! Call of the quapack routines
	!-----------------------------
	Select Case(wp)
	!---------------------------
	! Single precision version
	!---------------------------
	Case(sp)
	Call qagi(F,bound,1,epsabs,epsrel,Int,abserr, &
			  neval,ier,limit,lenw,last,iwork,work)
	!---------------------------
	! Double precision version
	!---------------------------
	Case(dp)
	Call dqagi(F,bound,1,epsabs,epsrel,Int,abserr, &
			  neval,ier,limit,lenw,last,iwork,work)
	End Select
	
	! Error code in Quadpack
	If(ier/=0) Then
	! Output of some control parameters
	 Open(53,file='error_int_inf.dat',status='unknown')
	 Write(53,'(i2,3(E15.5),2i5)') &
	   ier,Int,abserr/Abs(Int),abserr,neval,last
	Endif


	! Deallocation of arrays
	Deallocate(iwork,work)

   End Subroutine Quadpack_I

 
End Module Integral_mod

