
!----------------------------------------------------------------------
!              SPHERE part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: matrix_system_mod.f90,v 1.1 2001/08/11 15:44:26 ingves Exp ingves $
!-----------------------------------------------------------------------


Module Matrix_system_mod   
   

Use Global_definitions
! Use graphics

Contains
   
   !---------------------------------------------------------------
   Subroutine Matrix_system(m,A,Ienergy,param,int,int_tr1,zeta)

    ! NOTICE : Generic routine for an uniform or quadrupolar field
    ! This routine sets up the reduced dimensionless linear system 
    ! for the determination of the multipole coeficients.
    Implicit None
    Type(parameters)       :: param
    Integer                :: m       
    Type(integrals)        :: Int(0:,0:,0:), int_tr1(0:,0:,0:)
    Integer                :: Ienergy
    Complex(wpc)           :: A(:,:)
    Real(wp)               :: zeta(0:,0:,0:)

    If (param%above) Then
       ! MP ABOVE substrate 
       Call Matrix_system_above( m,A,Ienergy,param,int,int_tr1,zeta )
    Else
       ! MP BELOW substrate
       Call Matrix_system_below( m,A,Ienergy,param,int,int_tr1,zeta )
    End If

   End Subroutine Matrix_system

   !---------------------------------------------------------------

   Subroutine Right_Dipole(m,b,Ienergy,param,int,zeta)

     ! NOTICE :
     ! This routine sets up the reduced dimensionless right
     ! hand side of the linear system for the determination 
     ! of the multipole coeficients in the case of an uniform  
     ! incident electric field
     
     Implicit None
     Type(parameters)      ::  param
     Integer               ::  m,ienergy
     Complex(wpc)          ::  b(:)
     Type(integrals)       ::  Int(0:,0:,0:)
     Real(wp)              ::  zeta(0:,0:,0:)


     If(param%above) Then
        ! MP ABOVE substrate 
        Call Right_Dipole_above(m,b,Ienergy,param,int,zeta)
     Else
        ! MP BELOW substrate 
        Call Right_Dipole_below(m,b,Ienergy,param,int,zeta)
     Endif
     
     Return
     
   End subroutine Right_Dipole
   
   !---------------------------------------------------------------
	
   Subroutine Right_Quadrupole(m,b,Ienergy,param,int,zeta)

     ! NOTICE :
     ! This routine sets up the reduced dimensionless right
     ! hand side of the linear system for the determination 
     ! of the multipole coeficients in the case of a non uniform  
     ! field (quadrupolar)
     
     Implicit None
     Type(parameters)     ::  param
     Integer              ::  m,ienergy
     Complex(wpc)         ::  b(:)
     Type(integrals)      ::  Int(0:,0:,0:)
     Real(wp)             ::  zeta(0:,0:,0:)
     
     
     If(param%above) Then
        ! MP ABOVE substrate 
        Call Right_Quadrupole_above(m,b,Ienergy,param,int,zeta)
     Else
        ! MP BELOW substrate 
        Call Right_Quadrupole_below(m,b,Ienergy,param,int,zeta)
     Endif
     
     Return
     
   End subroutine Right_Quadrupole
   
   !---------------------------------------------------------------

   Subroutine Matrix_system_above(m,A,Ienergy,param,int,int_tr1,zeta)
     
     ! NOTICE : 
     ! This routine sets up the reduced dimensionless linear system 
     ! for the determination of the multipole coeficients, when 
     ! the multipoles are located ABOVE the substrate. 
     
     Implicit None
     Integer                :: m
     Type(parameters)       :: param       
     Type(integrals)        :: Int(0:,0:,0:), int_tr1(0:,0:,0:)
     Integer                :: Ienergy
     Complex(wpc)           :: A(:,:)
     Integer                :: mpo,l1,l2
     Real(wp)               :: zeta(0:,0:,0:)
     Complex(wpc)           :: e1,e2,e3,e4,eps(4)

     ! Some initial abrevations
     mpo  = param%Mpole_order 
     
     ! The dieletctric constants
     e1  = param%eps_vacuum 
     e2  = param%eps_substrate(ienergy)
     e3  = param%eps_island(ienergy)
     e4  = e2
     
     ! Build the matrix system
     A(:,:) = 0._wp
     Do l1=1,mpo
        Do l2=1,mpo
           
           eps  = (/  (e1-e2),2*e1,2*e1*e2,e1*(e1-e2) /) /(e1+e2) 
           ! --- C
           A(l1,l2)     =    Int(l1,l2,m)%K%mu(1)                         &
                + (-1._wp)**(l2+m)*eps(1)*Int(l1,l2,m)%K%mu(2)            & 
                + eps(2)*( int_tr1(l1,l2,m)%K%mu(1)-Int(l1,l2,m)%K%mu(1) )  
           A(l1,l2) =  zeta(l1,l2,m) * A(l1,l2) 
           
           ! --- F
           A(mpo+l1,l2) = eps(3)*int_tr1(l1,l2,m)%L%mu(1)                &
                + eps(4)*(  Int(l1,l2,m)%L%mu(1)                         &
                +(-1._wp)**(l2+m)*Int(l1,l2,m)%L%mu(2) )
           A(mpo+l1,l2) = zeta(l1,l2,m) * A(mpo+l1,l2)
           
           eps = (/ (e3-e4),2*e3,2*e3*e4,e3*(e3-e4) /) /(e3+e4)
           ! --- D
           A(l1,mpo+l2) =  Int(l1,l2,m)%M%mu(1)                          &  
                +(-1._wp)**(l2+m)*eps(1)*Int(l1,l2,m)%M%mu(2)            & 
                +eps(2)*( int_tr1(l1,l2,m)%M%mu(1)-Int(l1,l2,m)%M%mu(1) )
           A(l1,mpo+l2) =  - zeta(l1,l2,m) * A(l1,mpo+l2)
           
           ! --- G
           A(mpo+l1,mpo+l2) = eps(3)*int_tr1(l1,l2,m)%N%mu(1)            &
                + eps(4)*(  Int(l1,l2,m)%N%mu(1)                         &
                +(-1._wp)**(l2+m)*Int(l1,l2,m)%N%mu(2) )
           A(mpo+l1,mpo+l2) = - zeta(l1,l2,m) * A(mpo+l1,mpo+l2)
           
        Enddo   ! End loop over l2
     Enddo      ! End loop over l1
     
     Return
     
   End Subroutine Matrix_system_above
   
   !---------------------------------------------------------------
   
   Subroutine Matrix_system_below(m,A,Ienergy,param,int,int_tr1,zeta)
     ! NOTICE :
     ! This routine sets up the reduced dimensionless linear system 
     ! for the determination of the multipole coeficients, when 
     ! the multipoles are located BELOW the substrate. 
     
     Implicit None
     Type(parameters)       :: param
     Integer                :: m       
     Type(integrals)        :: Int(0:,0:,0:), int_tr1(0:,0:,0:)
     Integer                :: Ienergy
     Complex(wpc)           :: A(:,:)
     Integer                :: mpo,l1,l2
     Real(wp)               :: zeta(0:,0:,0:)
     Real(wp)               :: factor
     Complex(wpc)           :: e1,e2,e3,e4,eps(4)

    ! Some initial abrevations
    mpo  = param%Mpole_order 
    
    ! The dieletctric constants
    e1  = param%eps_vacuum
    e2  = param%eps_substrate(ienergy)
    e3  = param%eps_island(ienergy)
    e4  = e2

    ! Build the matrix system   
    ! Notice :  As compared to the above case (get_linear_system_above)
    !           e1 <--> e2
    !           e3 <--> e4
    A(:,:) = 0._wp
    Do l1=1,mpo
       Do l2=1,mpo
          
          eps  = (/  (e2-e1),2*e2,2*e2*e1,e2*(e2-e1) /) /(e2+e1)
          ! Unique for this MP below subsrate 
          factor =  (-1._wp)**(l1+l2)
          ! --- C
          A(l1,l2)     =    Int(l1,l2,m)%K%mu(1)                         &
               + (-1._wp)**(l2+m)*eps(1)*Int(l1,l2,m)%K%mu(2)            & 
               + eps(2)*( int_tr1(l1,l2,m)%K%mu(1)-Int(l1,l2,m)%K%mu(1) )  
          A(l1,l2) =  factor * zeta(l1,l2,m) * A(l1,l2) 
          
          ! --- F
          A(mpo+l1,l2) = eps(3)*int_tr1(l1,l2,m)%L%mu(1)                &
               + eps(4)*(  Int(l1,l2,m)%L%mu(1)                         &
               +(-1._wp)**(l2+m)*Int(l1,l2,m)%L%mu(2) )
          A(mpo+l1,l2) = factor * zeta(l1,l2,m) * A(mpo+l1,l2)  
          
          
          eps = (/ (e4-e3),2*e4,2*e4*e3,e4*(e4-e3) /) /(e4+e3)
          ! --- D
          A(l1,mpo+l2) =  Int(l1,l2,m)%M%mu(1)                          &  
               +(-1._wp)**(l2+m)*eps(1)*Int(l1,l2,m)%M%mu(2)            & 
               +eps(2)*( int_tr1(l1,l2,m)%M%mu(1)-Int(l1,l2,m)%M%mu(1) )
          A(l1,mpo+l2) =  - factor*zeta(l1,l2,m) * A(l1,mpo+l2)
          
          ! --- G
          A(mpo+l1,mpo+l2) = eps(3)*int_tr1(l1,l2,m)%N%mu(1)            &
               + eps(4)*(  Int(l1,l2,m)%N%mu(1)                         &
               +(-1._wp)**(l2+m)*Int(l1,l2,m)%N%mu(2)    )
          A(mpo+l1,mpo+l2) = - factor * zeta(l1,l2,m) * A(mpo+l1,mpo+l2) 
          
       Enddo   ! End loop over l2
    Enddo      ! End loop over l1
    
    Return
    
  End Subroutine Matrix_system_below

   !---------------------------------------------------------------
  Subroutine Right_Dipole_above(m,b,Ienergy,param,int,zeta)

    Implicit None
    Integer              ::   m,ienergy
    Complex(wpc)         ::   b(:)
    Type(parameters)     ::   param
    Type(integrals)      ::   Int(0:,0:,0:)
    Real(wp)             ::   zeta(0:,0:,0:)
    Integer              ::   l1,mpo
    Real(wp)             ::   tr,sin0,cos0,kronl
    Complex(wpc)         ::   e1,e2,exp0

    ! Some initial abrevations
    tr   = param%tr
    sin0 = Sin(param%theta0_calc)
    cos0 = Cos(param%theta0_calc)
    exp0 = Exp(-imu*param%phi0_calc)
    mpo  = param%Mpole_order 
    
    ! The dieletctric constants
    e1  = param%eps_vacuum 
    e2  = param%eps_substrate(ienergy)
    
    Do l1=1,mpo      
       ! Calculated the kronecker delta
       If (l1==1) Then
          kronl=1._wp
       Else
          kronl=0._wp
       Endif
       Select Case(m)
       Case(0)
          !--- H
          b(l1) = Sqrt(4._wp*pi/3._wp)*cos0* &
               ( e1/e2*kronl + (e1-e2)/e2*   &
               (Sqrt(3._wp)*tr*zeta(l1,0,0)*Int(l1,0,0)%Q &
               -zeta(l1,1,0)*Int(l1,1,0)%Q ) )   
          !--- J 
          b(mpo+l1) = Sqrt(4._wp*pi/3._wp)*e1*cos0*kronl   
          
       Case(1)
          !--- H 
          b(l1) = -Sqrt(2._wp*pi/3._wp)*sin0*kronl*exp0       
          !--- J
          b(mpo+l1) = -Sqrt(2._wp*pi/3._wp)*sin0*exp0* &
               ( e2*kronl+(e1-e2)*zeta(l1,1,1)*Int(l1,1,1)%Q )   
       End Select
    Enddo
    
  End Subroutine Right_Dipole_above
  
  !---------------------------------------------------------------
	
  Subroutine Right_Dipole_below(m,b,Ienergy,param,int,zeta)

    Implicit None
    Integer              ::  m,ienergy
    Complex(wpc)         ::  b(:)
    Type(parameters)     ::  param
    Type(integrals)      ::  Int(0:,0:,0:)
    Real(wp)             ::  zeta(0:,0:,0:)
    Integer              ::  l1,mpo
    Real(wp)             ::  tr,sin0,cos0,kronl,factor
    Complex(wpc)         ::  e1,e2,exp0
  
    ! Some initial abrevations
    tr   = param%tr 
    sin0 = Sin(param%theta0_calc)
    cos0 = Cos(param%theta0_calc)
    exp0 = Exp(-imu*param%phi0_calc)
    mpo  = param%Mpole_order 
    
    ! The dieletctric constants
    e1  = param%eps_vacuum 
    e2  = param%eps_substrate(ienergy)
    
    
    ! Notice :  As compared to the above case (get_linear_system_above)
    !           e1 <--> e2
    !           e3 <--> e4
    Do l1=1,mpo       
       ! Calculated the kronecker delta
       If (l1==1) Then
          kronl=1._wp
       Else
          kronl=0._wp
       Endif
       
       ! Unique for this MP below subsrate 
       factor =  (-1._wp)**(l1+1)
       Select Case(m)
       Case(0)
          !--- H
          b(l1) = Sqrt(4._wp*pi/3._wp)*cos0*   &
               ( kronl + (e2-e1)/e2*factor*   &
               ( -Sqrt(3._wp)*tr*zeta(l1,0,0)*Int(l1,0,0)%Q - &
               zeta(l1,1,0)*Int(l1,1,0)%Q ) )  
          !--- J 
          b(mpo+l1) = Sqrt(4._wp*pi/3._wp)*e1*cos0*kronl   
          
       Case(1)
              !--- H 
          b(l1) = -Sqrt(2._wp*pi/3._wp)*sin0*kronl*exp0      
          !--- J
          b(mpo+l1) = -Sqrt(2._wp*pi/3._wp)*sin0*exp0*   &
               ( kronl*e1+(e2-e1)*factor*zeta(l1,1,1)*Int(l1,1,1)%Q )   
       End Select
    Enddo
    
  End Subroutine Right_Dipole_below
  
  !---------------------------------------------------------------
  
  Subroutine Right_Quadrupole_above( m,b,Ienergy,param,int,zeta )

    Implicit None
    Type(parameters)     ::  param
    Complex(wpc)         ::  b(:)
    Integer              ::  m,Ienergy
    Type(integrals)      ::  Int(0:,0:,0:)
    Real(wp)             ::  zeta(0:,0:,0:)
    Integer              ::  l1,mpo
    Real(wp)             ::  kronl1,kronl2,tr
    Complex(wpc)         ::  e1,e2,sqr
    
    
    ! Some initial abrevations
    tr   = param%tr
    mpo  = param%Mpole_order
    sqr	 = sqrt(2._wp*pi/15._wp) 	   
    ! The dieletctric constants
    e1   = param%eps_vacuum 
    e2   = param%eps_substrate(ienergy)
    
    Do l1=1,mpo 
       
       ! Calculate the kronecker delta
       If (l1==1) Then
          kronl1=1._wp
       Else
          kronl1=0._wp
       Endif
       If (l1==2) Then
          kronl2=1._wp
       Else
          kronl2=0._wp
       Endif
       ! Formulas 10-71 10-72 10-73 10-74 10-76 10-77
       Select Case(m)
       Case(0)
          ! --- K
          b(l1)	 = -2._wp*sqrt(4._wp*pi/3._wp)* &
               ( sqrt(3._wp/5._wp)/2*kronl2 + (e1/e2-1._wp)*tr* &
               (sqrt(3._wp)*tr*zeta(l1,0,0)*Int(l1,0,0)%Q + kronl1 - &
               zeta(l1,1,0)*Int(l1,1,0)%Q ) )
          ! --- L
          b(l1+mpo)  = 4._wp*sqrt(pi/5._wp)* &
               ( (e2-e1)*zeta(l1,2,0)*Int(l1,2,0)%Q - e2*kronl2 + &
               sqrt(5._wp/3._wp)*(e2-e1)*tr*(kronl1-zeta(l1,1,0)*Int(l1,1,0)%Q) )
          
       Case(1)
          ! --- K
          b(l1)		= -sqr*(1._wp-imu)* &
               ( e1/e2*kronl2 - (e1/e2-1._wp)*( zeta(l1,2,1)*Int(l1,2,1)%Q + &
               sqrt(5._wp)*tr*(kronl1-zeta(l1,1,1)*Int(l1,1,1)%Q) ) )
          ! --- L
          b(l1+mpo)	= -sqr*(1._wp-imu)* &
               ( 2._wp*e1*kronl2 + sqrt(5._wp)*(e2-e1)*tr* &
               (kronl1-zeta(l1,1,1)*Int(l1,1,1)%Q) )
       Case(2)
          If(l1==1) cycle
          ! --- K
          b(l1)		= -sqr*imu*kronl2
          ! --- L
          b(l1+mpo)	= -2._wp*sqr*imu* &
               ( e2*kronl2 - (e2-e1)*zeta(l1,2,2)*Int(l1,2,2)%Q )
          
       End Select
       
    Enddo
    
  End Subroutine Right_Quadrupole_above

  !---------------------------------------------------------------
  
  Subroutine Right_Quadrupole_below( m,b,Ienergy,param,int,zeta )
    
    Implicit None
    Type(parameters)     ::  param
    Complex(wpc)         ::  b(:)
    Integer              ::  m,Ienergy
    Type(integrals)      ::  Int(0:,0:,0:)
    Real(wp)             ::  zeta(0:,0:,0:)
    Integer              ::  l1,mpo
    Real(wp)             ::  kronl1,kronl2,tr
    Complex(wpc)         ::  e1,e2,sqr
    
    
    ! Some initial abrevations
    tr   = -Abs(param%tr) ! Inversion of the system axis in the case param%above=.false.
    mpo  = param%Mpole_order
    sqr	 = sqrt(2._wp*pi/15._wp) 	   
    ! The dieletctric constants
    e1   = param%eps_vacuum 
    e2   = param%eps_substrate(ienergy)
    
    Do l1=1,mpo 
       
       ! Calculate the kronecker delta
       If (l1==1) Then
          kronl1=1._wp
       Else
          kronl1=0._wp
       Endif
       If (l1==2) Then
          kronl2=1._wp
       Else
          kronl2=0._wp
       Endif
       ! Formulas 10-71 10-72 10-73 10-74 10-76 10-77
       Select Case(m)
       Case(0)
          ! --- K
          b(l1)		= -2._wp*sqrt(4._wp*pi/3._wp)* &
               ( sqrt(3._wp/5._wp)/2*kronl2 + (e2/e1-1._wp)*tr* &
               (sqrt(3._wp)*tr*zeta(l1,0,0)*Int(l1,0,0)%Q + kronl1 - &
               zeta(l1,1,0)*Int(l1,1,0)%Q ) )
          ! --- L
          b(l1+mpo)	= 4._wp*sqrt(pi/5._wp)* &
               ( (e1-e2)*zeta(l1,2,0)*Int(l1,2,0)%Q - e1*kronl2 + &
               sqrt(5._wp/3._wp)*(e1-e2)*tr*(kronl1-zeta(l1,1,0)*Int(l1,1,0)%Q) )
          
       Case(1)
          ! --- K
          b(l1)		= -sqr*(1._wp-imu)* &
               ( e2/e1*kronl2 - (e2/e1-1._wp)*( zeta(l1,2,1)*Int(l1,2,1)%Q + &
               sqrt(5._wp)*tr*(kronl1-zeta(l1,1,1)*Int(l1,1,1)%Q) ) )
          ! --- L
          b(l1+mpo)	= -sqr*(1._wp-imu)* &
               ( 2._wp*e2*kronl2 + sqrt(5._wp)*(e1-e2)*tr* &
               (kronl1-zeta(l1,1,1)*Int(l1,1,1)%Q) )
       Case(2)
          If(l1==1) cycle
          ! --- K
          b(l1)		= -sqr*imu*kronl2
          ! --- L
          b(l1+mpo)	= -2._wp*sqr*imu* &
               ( e1*kronl2 - (e1-e2)*zeta(l1,2,2)*Int(l1,2,2)%Q )
          
       End Select
       
    Enddo
    
    
  End Subroutine Right_Quadrupole_below

  
End Module Matrix_system_mod
