
!----------------------------------------------------------------------
!              SPHERE part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: potential_mod.f90,v 1.1 2001/08/11 15:45:47 ingves Exp ingves $
!-----------------------------------------------------------------------


Module Potential_mod


  Use Global_definitions
  Use Graphics, only	: contour
  
Contains
  
  !-----------------------------------------------------------------------

  Subroutine Potential(MPC,ienergy,int,int_tr1,zeta,param)
    
    Implicit None
    Type(parameters) :: param 
    Type(integrals)  :: Int(0:,0:,0:), int_tr1(0:,0:,0:) 
    Integer          :: ienergy
    Real(wp)         :: zeta(0:,0:,0:)
    Complex(wpc)     :: MPC(2*param%Mpole_order,0:1)
    
    Select Case(param%above)
    Case(.true.)
       Call Potential_above(MPC,ienergy,int,int_tr1,zeta,param)
    Case(.false.)
       Call Potential_below(MPC,ienergy,int,int_tr1,zeta,param)	 
    End Select
    
  End Subroutine Potential
  

  !------------------------------------------------------------------------
  
  Subroutine Potential_above(MPC,ienergy,int,int_tr1,zeta,param)
    
    !------------------------------------
    ! Multipole located above the surface
    !------------------------------------
    
    implicit none
    ! Global
    Type(parameters) :: param 
    Type(integrals)  :: Int(0:,0:,0:), int_tr1(0:,0:,0:) 
    Integer          :: ienergy
    Real(wp)         :: zeta(0:,0:,0:)
    Complex(wpc)     :: MPC(2*param%Mpole_order,0:1)

    ! Local
    ! The multipole coefficients
    Complex(wpc),allocatable	:: A(:,:),B(:,:),Ar(:,:),Br(:,:),At(:,:),Bt(:,:)
    ! The corresponding spherical coordinates 
    Real(wp),allocatable        :: radius(:),cost(:),phi(:)
    ! Distances from the *-points
    Real(wp),allocatable        :: r_mp(:),r_imp(:),gamma(:)
    ! cos(theta_*) cos(theta_*bar)
    Real(wp),allocatable        :: cost_mp(:),cost_imp(:)
    ! Values of the Legendre polynomials
    Real(wp),allocatable        :: P_mp(:,:,:),P_imp(:,:,:)
    ! Prefactor of the spherical harmonics                             
    Real(wp),allocatable        :: prefactor(:,:)
    Integer                     :: mpo,l,m,ipoint
    Real(wp)                    :: mu_mp,mu_imp,tmp
    Real(wp)                    :: t0,cost0,sint0,p0,cosp0,sinp0,p
    Real(wp)                    :: costmp,sintmp
    Complex(wp)                 :: e1,e2,e3,e4, v0,v
    Complex(wp)                 :: Y_mp,Y_imp,phase,b0,ctmp
    ! Parameters for the grid
    Real(sp),allocatable        :: Grid(:,:)
    Real(wp),allocatable        :: x1(:),x2(:)
    Integer,parameter           :: Nx1=60
    Integer,parameter           :: Nx2=60
    Integer,parameter           :: Npoints=Nx1*Nx2
    Character(len=1)            :: cut
    Real(wp)                    :: value
    Real(wp), parameter         :: x1min=-2._wp
    Real(wp), parameter         :: x1max=+2._wp
    Real(wp), parameter         :: x2min=-2._wp
    Real(wp), parameter         :: x2max=+2._wp
    Real(wp)                    :: dx1,dx2
    Real(wp)                    :: x,y,z
    Integer                     :: i,j
    Integer,Parameter           :: Ns=500
    Logical                     :: realimag
    

    ! Open the output file
    Open(77,file='potreal.dat',status='unknown')
    Write(77,'(3a25)') 'x1','x2','potential'
    Open(78,file='potaimag.dat',status='unknown')
    Write(78,'(3a25)') 'x1','x2','potential'
    
    !-------------------------------------------------
    ! Allocation of the necessary space
    !-------------------------------------------------
    Allocate(Grid(Nx2,Nx1))
    Allocate(x1(Nx1),x2(Nx2))
    Allocate(radius(Npoints),cost(Npoints),phi(Npoints))
    Allocate(r_mp(Npoints),r_imp(Npoints),gamma(Npoints))
    Allocate(cost_mp(Npoints),cost_imp(Npoints))
    
    
    ! Dielectric constants
    e1 = param%eps_vacuum
    e2 = param%eps_substrate(ienergy)
    e3 = param%eps_island(ienergy)
    e4 = e2

    ! Type of surface of cut 
10  Write(6,'(a\)') '  Enter the plane of cut (x-y-z) : '
    Read(5,*,err=10) cut
20  Write(6,'(a\)') '  Enter the value of cut : '
    Read(5,*,err=20) value
30  Write(6,'(a\)') '  Real(T) or Imaginary(F) part : '
    Read(5,*,err=30) realimag
    
    !-------------------------------------------------
    ! Sets up the grid and the spherical coordinates
    !-------------------------------------------------
    dx1 = (x1max-x1min)/(Nx1-1)
    dx2 = (x2max-x2min)/(Nx2-1)
    x1  = arth(x1min,dx1,Nx1)
    x2  = arth(x2min,dx2,Nx2)
    
    Select Case(cut)
    Case('x')
       x = value
       do i=1,Nx1
	  y = x1(i)
	  do j=1,Nx2
             ipoint = (i-1)*Nx2+j
             z = x2(j)
             Call Sph_Coord(x,y,z,radius(ipoint),cost(ipoint),phi(ipoint))
	  enddo
       enddo
       x2 = -x2
    Case('y')
       y = value
       do i=1,Nx1
	  x = x1(i)
	  do j=1,Nx2
             ipoint = (i-1)*Nx2+j
             z = x2(j)
             Call Sph_Coord(x,y,z,radius(ipoint),cost(ipoint),phi(ipoint))
	  enddo
       enddo
       x2 = -x2
    Case('z')
       z = value
       do i=1,Nx1
	  x = x1(i)
	  do j=1,Nx2
	   ipoint = (i-1)*Nx2+j
	   y = x2(j)
	   Call Sph_Coord(x,y,z,radius(ipoint),cost(ipoint),phi(ipoint))
        enddo
     enddo
  Case Default
     Write(6,*) 'Error in the choice of plane-cut'
     pause
     stop
  End Select
      
  
  !-------------------------------------------------
  ! Some parameters 
  !-------------------------------------------------
  mu_mp      =  mu_func(param%tr,1,param)    ! Z-position
  mu_imp     =  mu_func(param%tr,2,param) 
  r_mp       =  sqrt( gamma_func(radius,cost,mu_mp) )  ! Radius
  r_imp      =  sqrt( gamma_func(radius,cost,mu_imp) )          
  cost_mp       =  chi_func(radius,cost,mu_mp)            ! cos(theta)
  cost_imp      =  chi_func(radius,cost,mu_imp)
  t0 = param%theta0_calc  
    p0 = param%phi0_calc
    cost0 = cos(t0)
    sint0 = sin(t0)
    cosp0 = cos(p0)
    sinp0 = sin(p0)    
    
    !-------------------------------------------------
    ! Sets up the multipole expansion coefficients
    !-------------------------------------------------
    mpo = param%Mpole_order    
    allocate( A(mpo,-1:1),  B(mpo,-1:1)  ) 
    allocate( Ar(mpo,-1:1), Br(mpo,-1:1) )
    allocate( At(mpo,-1:1), Bt(mpo,-1:1) )
    ! m=0,1
    A(:,0:1)  =   MPC(1:mpo,0:1)
    B(:,0:1)  =  MPC(mpo+1:2*mpo,0:1)
    ! m=-1
    A(:,-1)   = - A(:,1) * exp(2*imu*param%phi0_calc)
    B(:,-1)   = - B(:,1) * exp(2*imu*param%phi0_calc)   
    
    ! Reflected multipole foefficients
    do l=1,mpo
       do m=-1,1
          Ar(l,m) = (-1._wp)**(l+m) * (e1 - e2)/(e1 + e2) * A(l,m)
          Br(l,m) = (-1._wp)**(l+m) * (e3 - e4)/(e3 + e4) * B(l,m)
       enddo
    enddo
    ! transmitted multipole foefficients
    At  =  2*e1/(e1+e2) * A
    Bt  =  2*e3/(e3+e4) * B


    !-------------------------------------------------
    ! Calculates the associated Legendre polymomials 
    ! and prefactors for the spherical harmonics
    !-------------------------------------------------
    allocate( P_mp(mpo,-1:1,Npoints), P_imp(mpo,-1:1,Npoints) )
    allocate( prefactor(mpo,-1:1) )
    
    Do l=1,mpo
       Do m=1,-1,-1
          ! the prefactor
          select case(m)
          case(1)
             tmp              =  1._wp/(l*(l+1))
          case(0)
             tmp              =  1._wp                
          case(-1)
             tmp              =  1._wp*l*(l+1)                
          case default
             write(*,*) 'ERROR'
             Pause
             Stop
          end select
          prefactor(l,m)   =  sqrt( (2*l+1)/(4*pi) * tmp )
          ! The associated legendre polynomials and derivatived of those
          P_mp(l,m,:)  =  Ass_Legendre(l,m,cost_mp)     ! Multipoles 
          P_imp(l,m,:) =  Ass_Legendre(l,m,cost_imp)    ! Image multipoles  
       enddo
    enddo


    !------------------------------------------------------
    !---  Calculates the constant b0
    !-----------------------------------------------------  	   
    ctmp  = (e1/e2-1._wp)*cost0 
    b0    =    ctmp/sqrt(3._wp)*zeta(0,1,0)*int(0,1,0)%Q          &
               +  param%tr*ctmp * (1._wp-zeta(0,0,0)*int(0,0,0)%Q)
    do l=1,mpo
       ctmp  = A(l,0)*zeta(0,l,0)/sqrt(4*pi)
       ctmp  = ctmp                                                       &
            *( Int(0,l,0)%K%mu(1)                                         &
            +(-1._wp)**l*((e1-e2)/(e1+e2))*Int(0,l,0)%K%mu(2)              &
            +2*e1/(e1+e2)*( Int_tr1(0,l,0)%K%mu(1) - Int(0,l,0)%K%mu(1) ) )
       b0 = b0 + ctmp
       
       ctmp  = B(l,0)*zeta(0,l,0)/sqrt(4*pi)
       ctmp  = ctmp                                                       &
            *( Int(0,l,0)%M%mu(1)                                         &
            +(-1._wp)**l*((e3-e4)/(e3+e4))*Int(0,l,0)%M%mu(2)              &
            +2*e3/(e3+e4)*( Int_tr1(0,l,0)%M%mu(1) - Int(0,l,0)%M%mu(1) ) &
            )
       b0 = b0 - ctmp
    end do
   

    !************************************************************
    !********************   POTENTIAL  **************************
    !************************************************************

    !------------------------------------------------------
    !---  Loops over points in the grid
    !------------------------------------------------------
    grid_loop : do ipoint=1,size(radius,1)
       
       v = 0._wp
       
       
       !***********************************************
       !				Upper space 1-3
       !***********************************************
       Upper_Lower : if (radius(ipoint)*cost(ipoint) <= param%tr) then
         


          !-----------------------------------------------------
          !					Space 1
          !-----------------------------------------------------
          Inside_Outside1 : if (radius(ipoint) >= 1._wp) then		 
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             
             do l=1,mpo
                do m=-1,1
                   ! spherical harmonics
                   phase = (-1._wp)**m * exp(imu*m*phi(ipoint)) 
                   Y_mp  = prefactor(l,m) * P_mp(l,m,ipoint)*phase  
                   Y_imp = prefactor(l,m) * P_imp(l,m,ipoint)*phase  
                   ! Potentials                
                   v = v + A(l,m)*r_mp(ipoint)**(-l-1)*Y_mp     &
                        + Ar(l,m)*r_imp(ipoint)**(-l-1)*Y_imp 
                enddo
             enddo
             
             ! Calculates the constant potetial v0
             costmp	=	cost(ipoint)
             sintmp	=	sqrt(1._wp  - cost(ipoint)**2) 
             p		=	phi(ipoint)      
             v0         =    -   (   costmp*cost0   &     ! -
                  - sintmp*cos(p)*sint0*cosp0  &          ! +
                  - sintmp*sin(p)*sint0*sinp0  &          ! +
                  )
             
             ! Add the constant potentials
             v = v +  v0
             
             
          else
             
             !-----------------------------------------------------
             !						Space 3
             !-----------------------------------------------------
             
             do l=1,mpo
                do m=-1,1
                   ! spherical harmonics
                   phase = (-1._wp)**m * exp(imu*m*phi(ipoint)) 
                   Y_mp  = prefactor(l,m) * P_mp(l,m,ipoint)*phase  
                   Y_imp = prefactor(l,m) * P_imp(l,m,ipoint)*phase  
                   ! Potentials                
                   v = v + B(l,m)*r_mp(ipoint)**(l)*Y_mp        &
                        + Br(l,m)*r_imp(ipoint)**(l)*Y_imp 
                   
                enddo
             enddo
             
             ! Add the constant potentials
             v = v +  b0
             
          endif Inside_Outside1
          
          
          
          !***************************************************
          !				Lower space 2-4
          !***************************************************
       else
          
          !-----------------------------------------------------
          ! Space 2
          !-----------------------------------------------------
          Inside_Outside2 : if (radius(ipoint) >= 1._wp) then		 
             
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             do l=1,mpo
                do m=-1,1
                   ! spherical harmonics
                   phase = (-1._wp)**m * exp(imu*m*phi(ipoint)) 
                   Y_mp  = prefactor(l,m) * P_mp(l,m,ipoint)*phase  
                   ! Potentials                
                   v = v + At(l,m)*r_mp(ipoint)**(-l-1)*Y_mp        
                enddo
             enddo
             
             ! Calculates the constant potetials
             costmp	=	cost(ipoint) 
             sintmp	=	sqrt(1._wp - cost(ipoint)**2)
             p		=	phi(ipoint)      
             v0 =    -   (   e1/e2*costmp*cost0         &     ! -
                  - sintmp*cos(p)*sint0*cosp0  &     ! +
                  - sintmp*sin(p)*sint0*sinp0  &     ! +
                  )
             
             ! Add constant potetials          
             v = v + v0 +param%tr*(e1/e2-1._wp) * cost0
             
             !-----------------------------------------------------
             ! Space 4
             !-----------------------------------------------------
          else	 
             
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             do l=1,mpo
                do m=-1,1
                   ! spherical harmonics
                   phase = (-1._wp)**m * exp(imu*m*phi(ipoint)) 
                   Y_mp  = prefactor(l,m) * P_mp(l,m,ipoint)*phase  
                   ! Potentials                   
                   v = v + Bt(l,m)*r_mp(ipoint)**(l)*Y_mp    
                enddo
             enddo
             
             ! Add constant potential          
             v = v +  b0
             
          endif Inside_Outside2
          
          !------------------------------------------------------
          !---  End Condition Upper-Lower
          !------------------------------------------------------
          
       endif Upper_Lower
       
       
       !------------------------------------------------------
       !---  Storage of the results
       !------------------------------------------------------
       i = (ipoint-1)/Nx2+1
       j = ipoint-(i-1)*Nx2
       ! Graphical output
       If(realimag) Then
          Grid(Nx2+1-j,i)=real(v,4)
	Else
           Grid(Nx2+1-j,i)=real(aimag(v),4)
	Endif
	Write(77,'(3f25.5)') x1(i),x2(j),real(v,4)
	Write(78,'(3f25.5)') x1(i),x2(j),real(aimag(v),4)
        
        !------------------------------------------------------
        !---  End loops over points in the grid
        !------------------------------------------------------
     enddo grid_loop
     
     !------------------------------------------------
     !		Contour plot of isolines
     !------------------------------------------------
     Call contour(Grid,title='Potential',flag=2)
     
     !------------------------------------------------
     !	Deallocation of space
     !------------------------------------------------
     Deallocate(grid,x1,x2,radius,cost,phi,r_mp,r_imp,gamma,cost_mp,cost_imp)
     
     !-------------------------------------------------
     ! Set up the contour line for the cluster and
     ! the substrate surface
     !-------------------------------------------------
     Allocate(x1(Ns))
     Write(77,'(2a12)') 'xc','yc'
     Write(78,'(2a12)') 'xc','yc'
     Select Case(cut)
     Case('x')
        dx1 = (x1max-x1min)/(Ns-1)
        x1  = arth(x1min,dx1,Ns)
        Do i=1,Ns
           Write(77,'(2f12.5)') x1(i),-param%tr
           Write(78,'(2f12.5)') x1(i),-param%tr
        Enddo
        dx1 = 2*(pi-Acos(param%tr))/(Ns-1)
        x1  = arth(Acos(param%tr),dx1,Ns)
        Do i=1,Ns
	   Write(77,'(2f12.5)') Sin(x1(i)),-Cos(x1(i))
	   Write(78,'(2f12.5)') Sin(x1(i)),-Cos(x1(i))
        Enddo
     Case('y')
        dx1 = (x1max-x1min)/(Ns-1)
        x1  = arth(x1min,dx1,Ns)
        Do i=1,Ns
           Write(77,'(2f12.5)') x1(i),-param%tr
           Write(78,'(2f12.5)') x1(i),-param%tr
        Enddo
        dx1 = 2*(pi-Acos(param%tr))/(Ns-1)
        x1  = arth(Acos(param%tr),dx1,Ns)
        Do i=1,Ns
	   Write(77,'(2f12.5)') Sin(x1(i)),-Cos(x1(i))
	   Write(78,'(2f12.5)') Sin(x1(i)),-Cos(x1(i))
        Enddo
     Case('z')
        If(value<=param%tr.and.value>=-1._wp) Then
           dx1 = 2*pi/(Ns-1)
           x1  = arth(0._wp,dx1,Ns)
           Do i=1,Ns
              Write(77,'(2f12.5)') Sqrt(1._wp-value**2)*Cos(x1(i)), &
                   Sqrt(1._wp-value**2)*Sin(x1(i))
              Write(78,'(2f12.5)') Sqrt(1._wp-value**2)*Cos(x1(i)), &
                   Sqrt(1._wp-value**2)*Sin(x1(i))
           Enddo
        Endif
     Case Default
        Write(6,*) 'Error in the choice of plane-cut'
     End Select
     Deallocate(x1)
     
     Close(77,status='keep')
     Close(78,status='keep')
     
   End Subroutine Potential_above
   
   !------------------------------------------------------------------------
   
   Subroutine Potential_below(MPC,ienergy,int,int_tr1,zeta,param)
     
     !------------------------------------
     ! Multipole located below the surface
     !------------------------------------
     
     implicit none
     ! Global
     Type(parameters) :: param 
     Type(integrals)  :: Int(0:,0:,0:), int_tr1(0:,0:,0:) 
     Integer          :: ienergy
     Real(wp)         :: zeta(0:,0:,0:)
     Complex(wpc)     :: MPC(2*param%Mpole_order,0:1)
     
     ! Local
     ! The multipole coefficients
     Complex(wpc),allocatable	:: A(:,:),B(:,:),Ar(:,:),Br(:,:),At(:,:),Bt(:,:)
     ! The corresponding spherical coordinates 
     Real(wp),allocatable       :: radius(:),cost(:),phi(:)
     ! Distances from the *-points
     Real(wp),allocatable       :: r_mp(:),r_imp(:),gamma(:)
     ! cos(theta_*) cos(theta_*bar)
     Real(wp),allocatable       :: cost_mp(:),cost_imp(:)
     ! Values of the Legendre polynomials
     Real(wp),allocatable       :: P_mp(:,:,:),P_imp(:,:,:)
     ! Prefactor of the spherical harmonics                             
     Real(wp),allocatable       :: prefactor(:,:)
     Integer                    :: mpo,l,m,ipoint
     Real(wp)                   :: mu_mp,mu_imp,tmp
     Real(wp)                   :: t0,cost0,sint0,p0,cosp0,sinp0,p
     Real(wp)                   :: costmp,sintmp
     Complex(wp)                :: e1,e2,e3,e4, v0,v
     Complex(wp)                :: Y_mp,Y_imp,phase,b0,ctmp
     ! Parameters for the grid
     Real(sp),allocatable       :: Grid(:,:)
     Real(wp),allocatable       :: x1(:),x2(:)
     Integer,parameter          :: Nx1=60
     Integer,parameter          :: Nx2=60
     Integer,parameter          :: Npoints=Nx1*Nx2
     Character(len=1)           :: cut
     Real(wp)                   :: value
     Real(wp), parameter        :: x1min=-2._wp
     Real(wp), parameter        :: x1max=+2._wp
     Real(wp), parameter        :: x2min=-2._wp
     Real(wp), parameter        :: x2max=+2._wp
     Real(wp)                   :: dx1,dx2
     Real(wp)                   :: x,y,z
     Integer                    :: i,j
     Integer,Parameter          :: Ns=500
     Logical                    :: realimag
     
     ! Open the output file
     Open(77,file='potreal.dat',status='unknown')
     Write(77,'(3a25)') 'x1','x2','potential'
     Open(78,file='potaimag.dat',status='unknown')
     Write(78,'(3a25)') 'x1','x2','potential'
     
     !-------------------------------------------------
     ! Allocation of the necessary space
     !-------------------------------------------------
     Allocate(x1(Nx1),x2(Nx2))
     Allocate(radius(Npoints),cost(Npoints),phi(Npoints))
     Allocate(r_mp(Npoints),r_imp(Npoints),gamma(Npoints))
     Allocate(cost_mp(Npoints),cost_imp(Npoints))
     

     ! Dielectric constants
     e1 = param%eps_vacuum
     e2 = param%eps_substrate(ienergy)
     e3 = param%eps_island(ienergy)
     e4 = e2
     
     ! Type of surface of cut 
10   Write(6,'(a\)') '  Enter the plane of cut (x-y-z) : '
     Read(5,*,err=10) cut
20   Write(6,'(a\)') '  Enter the value of cut : '
     Read(5,*,err=20) value
30   Write(6,'(a\)') '  Real(T) or Imaginary(F) part : '
     Read(5,*,err=30) realimag
     
     
     !-------------------------------------------------
     ! Sets up the grid and the spherical coordinates
     !-------------------------------------------------
     dx1 = (x1max-x1min)/(Nx1-1)
     dx2 = (x2max-x2min)/(Nx2-1)
     x1  = arth(x1min,dx1,Nx1)
     x2  = arth(x2min,dx2,Nx2)
     
     Select Case(cut)
     Case('x')
        x = value
        do i=1,Nx1
           y = x1(i)
           do j=1,Nx2
              ipoint = (i-1)*Nx2+j
              z = x2(j)
              Call Sph_Coord(x,y,z,radius(ipoint),cost(ipoint),phi(ipoint))
           enddo
        enddo
        x2 = -x2
     Case('y')
        y = value
        do i=1,Nx1
           x = x1(i)
           do j=1,Nx2
              ipoint = (i-1)*Nx2+j
	   z = x2(j)
	   Call Sph_Coord(x,y,z,radius(ipoint),cost(ipoint),phi(ipoint))
        enddo
     enddo
     x2 = -x2
  Case('z')
     z = value
     do i=1,Nx1
        x = x1(i)
        do j=1,Nx2
	   ipoint = (i-1)*Nx2+j
	   y = x2(j)
	   Call Sph_Coord(x,y,z,radius(ipoint),cost(ipoint),phi(ipoint))
        enddo
     enddo
  Case Default
     Write(6,*) 'Error in the choice of plane-cut'
     pause
     stop
  End Select
  
  
    !-------------------------------------------------
  ! Some parameters 
  !-------------------------------------------------
  mu_mp      =  mu_func(param%tr,1,param)    ! Z-position
  mu_imp     =  mu_func(param%tr,2,param) 
  r_mp       =  sqrt( gamma_func(radius,cost,mu_mp) )  ! Radius
  r_imp      =  sqrt( gamma_func(radius,cost,mu_imp) )          
  cost_mp    =  chi_func(radius,cost,mu_mp)            ! cos(theta)
  cost_imp   =  chi_func(radius,cost,mu_imp)
  t0 = param%theta0_calc  
  p0 = param%phi0_calc
  cost0 = cos(t0)
  sint0 = sin(t0)
  cosp0 = cos(p0)
  sinp0 = sin(p0)    
  
  !-------------------------------------------------
  ! Sets up the multipole expansion coefficients
  !-------------------------------------------------
  mpo = param%Mpole_order    
  allocate( A(mpo,-1:1),  B(mpo,-1:1)  ) 
  allocate( Ar(mpo,-1:1), Br(mpo,-1:1) )
  allocate( At(mpo,-1:1), Bt(mpo,-1:1) )
  ! m=0,1
  A(:,0:1)  =   MPC(1:mpo,0:1)
  B(:,0:1)  =  MPC(mpo+1:2*mpo,0:1)
  ! m=-1
  A(:,-1)   = - A(:,1) * exp(2*imu*param%phi0_calc)
  B(:,-1)   = - B(:,1) * exp(2*imu*param%phi0_calc)   
  
  ! Reflected multipole foefficients
  do l=1,mpo
     do m=-1,1
        Ar(l,m) = (-1._wp)**(l+m) * (e2 - e1)/(e2 + e1) * A(l,m)
        Br(l,m) = (-1._wp)**(l+m) * (e4 - e3)/(e4 + e3) * B(l,m)
     enddo
  enddo
  ! transmitted multipole foefficients
  At  =  2*e2/(e2+e1) * A
  Bt  =  2*e4/(e4+e3) * B
  
  
  !-------------------------------------------------
  ! Calculates the associated Legendre polymomials 
  ! and prefactors for the spherical harmonics
  !-------------------------------------------------
  allocate( P_mp(mpo,-1:1,Npoints), P_imp(mpo,-1:1,Npoints) )
  allocate( prefactor(mpo,-1:1) )
    
  Do l=1,mpo
     Do m=1,-1,-1
        ! the prefactor
        select case(m)
        case(1)
           tmp              =  1._wp/(l*(l+1))
        case(0)
           tmp              =  1._wp                
        case(-1)
           tmp              =  1._wp*l*(l+1)                
        case default
           write(*,*) 'ERROR'
           Pause
           Stop
        end select
        prefactor(l,m)   =  sqrt( (2*l+1)/(4*pi) * tmp )
        ! The associated legendre polynomials and derivatived of those
        P_mp(l,m,:)  =  Ass_Legendre(l,m,cost_mp)     ! Multipoles 
        P_imp(l,m,:) =  Ass_Legendre(l,m,cost_imp)    ! Image multipoles  
     enddo
  enddo
  
  
  !------------------------------------------------------
  !---  Calculates the constant b0
  !-----------------------------------------------------  	   
  ctmp  = (e1/e2-1._wp)*cost0 
  b0  =    ctmp/sqrt(3._wp)*zeta(0,1,0)*int(0,1,0)%Q          &
       + param%tr*ctmp*zeta(0,0,0)*int(0,0,0)%Q
  do l=1,mpo
     ctmp  = A(l,0)*zeta(0,l,0)/sqrt(4*pi)
     ctmp  = ctmp                                                       &
          *( (-1._wp)**l*Int(0,l,0)%K%mu(1)                                         &
          +(e2-e1)/(e2+e1)*Int(0,l,0)%K%mu(2)              &
          +(-1._wp)**l*2*e2/(e2+e1)*( Int_tr1(0,l,0)%K%mu(1) - Int(0,l,0)%K%mu(1) ) )
     b0 = b0 + ctmp
     
     ctmp  = B(l,0)*zeta(0,l,0)/sqrt(4*pi)
     ctmp  = ctmp                                                       &
          *( (-1._wp)**l*Int(0,l,0)%M%mu(1)                                         &
          +(e4-e3)/(e4+e3)*Int(0,l,0)%M%mu(2)              &
          +(-1._wp)**l*2*e4/(e4+e3)*( Int_tr1(0,l,0)%M%mu(1) - Int(0,l,0)%M%mu(1) ) )
     b0 = b0 - ctmp
  end do
  
  
  !************************************************************
  !********************   POTENTIAL  **************************
  !************************************************************
  
  !------------------------------------------------------
  !---  Loops over points in the grid
  !------------------------------------------------------
  grid_loop : do ipoint=1,size(radius,1)
     
     v = 0._wp
     
     
     !***********************************************
     !				Upper space 1-3
     !***********************************************
     Upper_Lower : if (radius(ipoint)*cost(ipoint) <= param%tr) then
       
        
        !-----------------------------------------------------
        !					Space 1
        !-----------------------------------------------------
        Inside_Outside1 : if (radius(ipoint) >= 1._wp) then		 
           
           !------------------------------------------------------
           !---  Loops over quantum numbers
           !------------------------------------------------------
           
           do l=1,mpo
              do m=-1,1
                 ! spherical harmonics
                 phase = (-1._wp)**m * exp(imu*m*phi(ipoint)) 
                 Y_mp  = prefactor(l,m) * P_mp(l,m,ipoint)*phase  
                 ! Potentials                
                 v = v + At(l,m)*r_mp(ipoint)**(-l-1)*Y_mp  
              enddo
           enddo
           
           ! Calculates the constant potetial v0
           costmp	=	cost(ipoint)
           sintmp	=	sqrt(1._wp  - cost(ipoint)**2) 
           p		=	phi(ipoint)      
           v0 =    -   (   costmp*cost0               &     ! -
                - sintmp*cos(p)*sint0*cosp0  &     ! +
                - sintmp*sin(p)*sint0*sinp0  &     ! +
                )
           
           ! Add the constant potentials
           v = v +  v0
           
           
        else
           
           !-----------------------------------------------------
           !						Space 3
           !-----------------------------------------------------
           
           do l=1,mpo
              do m=-1,1
                 ! spherical harmonics
                 phase = (-1._wp)**m * exp(imu*m*phi(ipoint)) 
                 Y_mp  = prefactor(l,m) * P_mp(l,m,ipoint)*phase  
                 ! Potentials                
                 v = v + Bt(l,m)*r_mp(ipoint)**(l)*Y_mp    
              enddo
           enddo
           
           ! Add the constant potentials
           v = v +  b0
           
        endif Inside_Outside1
        
        
        
	!***************************************************
	!				Lower space 2-4
	!***************************************************
     else

        !-----------------------------------------------------
        ! Space 2
        !-----------------------------------------------------
        Inside_Outside2 : if (radius(ipoint) >= 1._wp) then		 
           
           
           !------------------------------------------------------
           !---  Loops over quantum numbers
           !------------------------------------------------------
           do l=1,mpo
              do m=-1,1
                 ! spherical harmonics
                 phase = (-1._wp)**m * exp(imu*m*phi(ipoint)) 
                 Y_mp  = prefactor(l,m) * P_mp(l,m,ipoint)*phase  
                 Y_imp = prefactor(l,m) * P_imp(l,m,ipoint)*phase  
                 ! Potentials                
                 v = v + A(l,m)*r_mp(ipoint)**(-l-1)*Y_mp   +&
                      Ar(l,m)*r_imp(ipoint)**(-l-1)*Y_imp
              enddo
           enddo
           
           ! Calculates the constant potetials
           costmp	=	cost(ipoint) 
           sintmp	=	sqrt(1._wp - cost(ipoint)**2)
           p		=	phi(ipoint)      
           v0 =    -   (   e1/e2*costmp*cost0         &     ! -
                - sintmp*cos(p)*sint0*cosp0  &     ! +
                - sintmp*sin(p)*sint0*sinp0  &     ! +
                )
           
           ! Add constant potetials          
           v = v + v0 +param%tr*(e1/e2-1._wp) * cost0
                          
           !-----------------------------------------------------
           ! Space 4
           !-----------------------------------------------------
        else	 
           

           !------------------------------------------------------
           !---  Loops over quantum numbers
           !------------------------------------------------------
           do l=1,mpo
              do m=-1,1
                 ! spherical harmonics
                 phase = (-1._wp)**m * exp(imu*m*phi(ipoint)) 
                 Y_mp  = prefactor(l,m) * P_mp(l,m,ipoint)*phase  
                 Y_imp = prefactor(l,m) * P_imp(l,m,ipoint)*phase 
                 ! Potentials                   
                 v = v + B(l,m)*r_mp(ipoint)**(l)*Y_mp + &
                      Br(l,m)*r_imp(ipoint)**(l)*Y_imp
              enddo
           enddo
           
           ! Add constant potential          
           v = v +  b0
           
        endif Inside_Outside2
        
        !------------------------------------------------------
        !---  End Condition Upper-Lower
        !------------------------------------------------------
        
     endif Upper_Lower
     
     
     !------------------------------------------------------
     !---  Storage of the results
    !------------------------------------------------------
     i = (ipoint-1)/Nx2+1
     j = ipoint-(i-1)*Nx2
     ! Graphical output
     If(realimag) Then
        Grid(Nx2+1-j,i)=real(v,4)
     Else
	 Grid(Nx2+1-j,i)=real(aimag(v),4)
	Endif
	Write(77,'(3f25.5)') x1(i),x2(j),real(v,4)
	Write(78,'(3f25.5)') x1(i),x2(j),real(aimag(v),4)
        
        !------------------------------------------------------
        !---  End loops over points in the grid
        !------------------------------------------------------
     enddo grid_loop
     
	!------------------------------------------------
	!	Deallocation of space
	!------------------------------------------------
     Deallocate(grid,x1,x2,radius,cost,phi,r_mp,r_imp,gamma,cost_mp,cost_imp)
     
     !-------------------------------------------------
     ! Set up the contour line for the cluster and
     ! the substrate surface
     !-------------------------------------------------
     Allocate(x1(Ns))
     Write(77,'(2a12)') 'xc','yc'
     Write(78,'(2a12)') 'xc','yc'
     Select Case(cut)
     Case('x')
        dx1 = (x1max-x1min)/(Ns-1)
        x1  = arth(x1min,dx1,Ns)
        Do i=1,Ns
           Write(77,'(2f12.5)') x1(i),-param%tr
           Write(78,'(2f12.5)') x1(i),-param%tr
        Enddo
        dx1 = 2*(pi-Acos(param%tr))/(Ns-1)
        x1  = arth(Acos(param%tr),dx1,Ns)
        Do i=1,Ns
	   Write(77,'(2f12.5)') Sin(x1(i)),-Cos(x1(i))
	   Write(78,'(2f12.5)') Sin(x1(i)),-Cos(x1(i))
        Enddo
     Case('y')
        dx1 = (x1max-x1min)/(Ns-1)
        x1  = arth(x1min,dx1,Ns)
        Do i=1,Ns
           Write(77,'(2f12.5)') x1(i),-param%tr
           Write(78,'(2f12.5)') x1(i),-param%tr
        Enddo
        dx1 = 2*(pi-Acos(param%tr))/(Ns-1)
        x1  = arth(Acos(param%tr),dx1,Ns)
        Do i=1,Ns
	   Write(77,'(2f12.5)') Sin(x1(i)),-Cos(x1(i))
	   Write(78,'(2f12.5)') Sin(x1(i)),-Cos(x1(i))
        Enddo
     Case('z')
        If(value<=param%tr.and.value>=-1._wp) Then
           dx1 = 2*pi/(Ns-1)
           x1  = arth(0._wp,dx1,Ns)
           Do i=1,Ns
              Write(77,'(2f12.5)') Sqrt(1._wp-value**2)*Cos(x1(i)), &
                   Sqrt(1._wp-value**2)*Sin(x1(i))
              Write(78,'(2f12.5)') Sqrt(1._wp-value**2)*Cos(x1(i)), &
                   Sqrt(1._wp-value**2)*Sin(x1(i))
           Enddo
        Endif
     Case Default
        Write(6,*) 'Error in the choice of plane-cut'
     End Select
     Deallocate(x1)
     
     Close(77,status='keep')
     Close(78,status='keep')
     
   End Subroutine Potential_below
   
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================
  !==============================================================


   !==============================================================
   
   Function mu_func(tr,Type,param)  Result(mu)
     ! calculated the z-coordiate of the multipole (MP) ir image MP
     Implicit None
     Type(parameters)   :: param     
     Integer            :: Type
     Real(wp)           :: tr,mu
     
     Select Case(Type)
     Case(1)
        ! The MP
        mu = param%MPpos
     Case(2)
        ! The image MP
        mu = 2._wp*tr - param%MPpos
     Case default
        Pause 'ERROR in mu_func'
        Stop
     End Select
     Return
   End Function mu_func
   
   !--------------------------------------------------
   
   Function gamma_func(x,y,mu)  Result(gamma)
     Implicit None
     Real(wp)   :: x(:), y(:), mu, gamma(Size(x,1))     
     gamma= x**2-2*x*y*mu+mu**2
     Return
   End Function gamma_func
   
   !--------------------------------------------------

   Function chi_func(x,y,mu)  Result(chi)
     Implicit None
     Real(wp)   :: x(:),y(:), mu, chi(Size(x,1))     
     chi = (x*y-mu)/Sqrt(gamma_func(x,y,mu))   
     Return
   End Function chi_func

   !---------------------------------------------------------------

   Function Ass_Legendre(l,m,x) Result(alp)
     ! Calculates the Associate Legendre polynomials
     Use Legendre, Only : asslegpoly
     Implicit None
     Integer  :: l,m, itmp, tmp
     Real(wp) :: x(:), alp(Size(x))
     ! NR-routine
     if (m>=0) then
        alp =  asslegpoly(l,m,x)
     else
        alp =  asslegpoly(l,-m,x)
        tmp=1
        do itmp=l,l-m
           tmp=tmp*itmp
        enddo
        alp =  (-1._wp)**m * alp / tmp
     end if
     Return
   End Function Ass_Legendre
   
   !---------------------------------------------------------------

   Subroutine Sph_Coord(x,y,z,r,cost,phi)
     
     Implicit None
     Real(wp)				:: x,y,z
     Real(wp)				:: r,cost,phi
     
     r = sqrt(x**2+y**2+z**2)
     cost = z/r
     phi = atan(abs(y/x))
     if(x>=0) then
        if(y>=0) then
           phi = phi
        else
           phi = 2*pi - phi
        endif
     else
        if(y>=0) then
           phi = pi - phi
        else
           phi = pi + phi
        endif
     endif
     
     
   End Subroutine Sph_Coord
   
   
   
 End Module Potential_mod





