
!----------------------------------------------------------------------
!              SPHERE part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: sphere.f90,v 1.1 2001/08/11 15:46:29 ingves Exp ingves $
!-----------------------------------------------------------------------



!====================================================================
!========	 MAIN PROGRAM             ===========================
!====================================================================

Program Sphere

  ! Modules used by the main program
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! THESE LINES CAN BE SUPPRESSED SAFELY
  Use Windows_menu_mod, only :  Windows_menu
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  Use Global_definitions
  Use Initialize_mod
  Use Integral_mod,        only	 : get_integrals
  Use Matrix_system_mod
  Use linsolver_mod,       only	 : linsolver
  Use Optics_mod 
  Use Write_mod
  Use Boundary_mod,        only  : boundary_conditions
  Use Potential_mod,       only	 : potential
  
  ! Global variables
  Implicit None
  Type(parameters)                     :: param    
  Type(integrals),Pointer              :: Int(:,:,:), int_tr1(:,:,:)
  Type(surface_constitutive), Pointer  :: surf_const_coef(:)
  Integer                              :: ienergy,m,itmp(1),mpo,i
  Real(wp), Pointer                    :: zeta(:,:,:), Fresnel_coeff(:)
  Real(wp)		               :: BC_energy
  Complex(wpc),Pointer                 :: A(:,:),Atmp(:,:)
  Complex(wpc),Pointer                 :: bd(:,:),bq(:,:),bqtmp(:)
  Complex(wpc),Pointer		       :: mpoled(:,:,:)
  Complex(wpc),Pointer                 :: mpoleq(:,:)
  Character(len=1)                     :: rep

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! THESE LINES CAN BE SUPPRESSED SAFELY		
  ! Main windows
   Call Windows_menu
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  ! ---------------------------------------------------------------------
  !        Initialize all the parameters and the pointers
  ! ---------------------------------------------------------------------
10 Call initialize( A,bd,bq,param,Fresnel_coeff,mpoled,mpoleq, int,int_tr1,surf_const_coef,zeta )   
  itmp = minloc(abs(param%energy-param%BC_energy))
  BC_energy = param%energy(itmp(1))
  

  Select case (trim(adjustl(param%geometry))) 
  Case('film')
     ! ---------------------------------------------------------------------
     !      Film Geometry
     ! ---------------------------------------------------------------------
     ! For consistency reasons the surface const. coef. method is used 
     ! for the film-geometry.
     param%fresnel  = 'constitutive' 
     param%R        =  1._wp
     call surf_const_coef_film(surf_const_coef, param)
     
     
  Case('2film')
     ! ---------------------------------------------------------------------
     !	     Stacking of 2 films
     ! ---------------------------------------------------------------------
     ! For consistency reasons the surface const. coef. method is used 
     ! for the film-geometry.
     param%fresnel  = 'constitutive' 
     param%R        =  1._wp
     call surf_const_coef_2film(surf_const_coef, param)
     
     
  Case('thin_cap')
     ! ---------------------------------------------------------------------
     !	      Thin Spherical Cap
     ! ---------------------------------------------------------------------
     call surf_const_coef_cap(surf_const_coef, param)	
     
     
  Case('island')
     ! ---------------------------------------------------------------------
     !	       Island Geometry
     ! ---------------------------------------------------------------------
     ! Always let param%above=.true. in order to use the same expansion for the potentials
     !       param%above = .true.
     mpo	=	param%MPole_order
     
     Write(6,*) ' Setting up all the integrals ! Wait...'
     ! Get the various integrals 
     If (param%above) Then
        Call get_integrals( 1._wp,int_tr1,param )       ! tr = 1
        Call get_integrals(  param%tr ,int,param )
     Else
        ! For the calculation of the integrals : tr -> -tr;  mu -> -mu
        param%tr	 = - param%tr
        param%MPpos = - param%MPpos 
        Call get_integrals( 1._wp,int_tr1,param )       ! tr = 1
        Call get_integrals( param%tr,int,param )
        param%tr	 = - param%tr
        param%MPpos = - param%MPpos 
     End if
     
     
     !----------------------------------------------------------------------
     !	     Energy Loop
     !-----------------------------------------------------------------------
     Write(6,*) ' Evaluating the polarizabilites ! Wait...'
     Energy_loop : Do ienergy=1,param%Nenergy
        
        Write(6,'(a,f8.3)') '   Energy (eV) : ',param%energy(ienergy)
        
        !----------------------------------------------------------------------
        !    Get the multipoles coefficients
        !----------------------------------------------------------------------
	  ! Get the generic right hand side m=0
	   Call Matrix_system(0,A,Ienergy,param,int,int_tr1,zeta)
	  ! Dipole right hand side for m=0 (perpendicular component)
	   Call Right_Dipole(0,bd(:,0),Ienergy,param,int,zeta)
	   ! Solve the linear system
	   Call linsolver( A(:,:),bd(:,0),param%epslin )	 
           ! Store the dipole and quadrupole coefficients
           mpoled(:,0,ienergy) = bd(1:2,0)


	   If(param%interaction=='quadrupole') Then
              ! Quadrupole right hand side for m=0 
              Call Right_Quadrupole( 0,bq(:,0),Ienergy,param,int,zeta )
              ! Solve the linear system
              Call linsolver( A(:,:),bq(:,0),param%epslin )
              ! Store the dipole and quadrupole coefficients
              mpoleq(1,ienergy) = bq(2,0)
	   Endif
           

           ! Get the generic right hand side m=1
	   Call Matrix_system(1,A,Ienergy,param,int,int_tr1,zeta)
           ! Dipole right hand side for m=1 (paralel component)
	   Call Right_Dipole(1,bd(:,1),Ienergy,param,int,zeta)
           ! Solve the linear system	 
	   Call linsolver( A(:,:),bd(:,1),param%epslin )
           ! Store the dipole and quadrupole coefficients
           mpoled(:,1,ienergy) = bd(1:2,1)
           
	   If(param%interaction=='quadrupole') Then
              ! Quadrupole right hand side for m=1
              Call Right_Quadrupole( 1,bq(:,1),Ienergy,param,int,zeta )
              ! Solve the linear system	
              Call linsolver( A(:,:),bq(:,1),param%epslin )
              ! Store the dipole and quadrupole coefficients
              mpoleq(2,ienergy) = bq(2,1)
	   Endif
           
           
	   If(param%interaction=='quadrupole') Then
              ! Get the generic right hand side m=2
              Call Matrix_system(2,A,Ienergy,param,int,int_tr1,zeta)
              ! Quadrupole right hand side for m=2		 
              Call Right_Quadrupole( 2,bq(:,2),Ienergy,param,int,zeta )
              ! Solve the linear system
              Allocate(Atmp(1:2*mpo-2,1:2*mpo-2),bqtmp(2*mpo-2))
              ! The m=2 system does not contain the l=1 equation	
              Atmp(1:mpo-1,1:mpo-1)            =    A(2:mpo,2:mpo)
              Atmp(1:mpo-1,mpo:2*mpo-2)        =    A(2:mpo,mpo+2:2*mpo)
              Atmp(mpo:2*mpo-2,1:mpo-1)        =    A(mpo+2:2*mpo,2:mpo)
              Atmp(mpo:2*mpo-2,mpo:2*mpo-2)    =    A(mpo+2:2*mpo,mpo+2:2*mpo)
              bqtmp(1:mpo-1)                   =    bq(2:mpo,2)
              bqtmp(mpo:2*mpo-2)               =    bq(mpo+2:2*mpo,2)
              Call linsolver( Atmp(:,:),bqtmp(:),param%epslin )
              ! Store the dipole and quadrupole coefficients
              mpoleq(3,ienergy) = bqtmp(1) 
              Deallocate(Atmp,bqtmp)
	   Endif

           !----------------------------------------------------------------------
           !      Output for the oscillators strenghts
           !----------------------------------------------------------------------
           ! Output of the oscillators strenght if necessary
	   If(param%strenght) then
              Open(48,file='Multipole_Strenght.dat',status='unknown')
              Write(48,'(f10.5,8x,71(2f10.5,5x))')  &
                   param%energy(ienergy), ( ( Real(bd(i,m)),Aimag(bd(i,m)),m=0,1 ) ,i=1,param%Mpole_order)
	   Endif
           
           !----------------------------------------------------------------------
           !       Check the boundary conditions or calculate the potential 
           !	       for an uniform incident field
           !----------------------------------------------------------------------
           ! For testing the boundary conditions or the potential
           If (ienergy==itmp(1)) then
              Select Case (param%bound)
              Case('none')
                 Continue
              Case('BC')
                 Write(6,*) ' Calculating the boundaries conditions ! Wait...'
                 Write(6,'(a,f5.3,a)') &
                      '  Boundary conditions at : ',param%energy(ienergy),' eV'
                 Call boundary_conditions(bd(:,:),ienergy,int,int_tr1,zeta,param)
              Case('Pot')
                 Write(6,*) ' Calculating the potential ! Wait...'
                 Write(6,'(a,f5.3,a)') &
                      ' Potential at : ',param%energy(ienergy),' eV'
                 Call potential(bd(:,:),ienergy,int,int_tr1,zeta,param)
              Case Default
                 Write(6,*) 'Error in the choice of param%bound ! (',param%bound,')'
              End Select
           Endif
           
           
        Enddo Energy_loop
        !----------------------------------------------------------------------
        !      End Energy Loop
        !----------------------------------------------------------------------

        ! Free the unecessary allocated space
        Deallocate(A)
        Deallocate(bd,bq)

        !-----------------------------------------------------------------------
        !       Calculates the surface constitutive coefficients	
        !-----------------------------------------------------------------------
        Call surf_const_coef_island(surf_const_coef,mpoled,mpoleq,param)
        
        ! Free the unecessary allocated space
        Deallocate(Int)
        Deallocate(int_tr1)
        Deallocate(zeta)
        Deallocate(mpoled,mpoleq)
        
        
        !-----------------------------------------------------------------------
        !        Default Value		
        !-----------------------------------------------------------------------
	  Case Default
	   Write(6,*) 'ERROR in Bedeaux : Geometry : ', param%geometry, &
                ' not defined ! '		        
        End Select


        ! ---------------------------------------------------------------------
        !         Calculates the optical properties of the layer
        ! ---------------------------------------------------------------------
        Write(6,*) ' Calculating the Fresnel coefficient ! Wait...'
        Call Fresnel_calc(Fresnel_coeff,surf_const_coef,param)
        ! Free the unecessary allocated space 
        Deallocate(surf_const_coef)
        

        ! ---------------------------------------------------------------------
        !	  Plots and writes the results
        ! ---------------------------------------------------------------------
        Write(6,*)
        Call Write2screen2file(Fresnel_coeff,param)
        
        ! ---------------------------------------------------------------------
        !          Frees the allocated space
        ! ---------------------------------------------------------------------
        Deallocate(param%energy)
        Deallocate(param%eps_substrate)
        Deallocate(param%eps_island) 
        Deallocate(Fresnel_coeff)
        
        ! ---------------------------------------------------------------------
        !          Other calculation ?
        ! ---------------------------------------------------------------------
20	Write(6,*)
	Write(6,'(a\)') '  Other simulation (y-n) ?  '
	Read(5,*) rep
	Write(6,*)
	Select Case(rep)
        Case('y')
           Goto 10
        Case('n')
        Case Default
           Write(6,*) 'Band answer !'
           Goto 20
        End Select
        
        
      End Program Sphere







