
!--------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: global_def.f90,v 1.1 2001/07/29 12:37:18 ingves Exp ingves $
!---------------------------------------------------------------------




Module Global_definitions


  ! Some operating system specific stuff
  Include 'os.h'


  !===========================================================
  ! This module mainly contains various kinds of definitions
  ! of derived types
  !===========================================================



  ! Definitions of precision
  INTEGER, PARAMETER :: I4B = SELECTED_INT_KIND(9)
  INTEGER, PARAMETER :: SP = KIND(1.0)
  INTEGER, PARAMETER :: DP = KIND(1.0D0)
  INTEGER, PARAMETER :: WP = KIND(1.0D0)
  INTEGER, PARAMETER :: SPC = KIND((1.0,1.0))
  INTEGER, PARAMETER :: DPC = KIND((1.0,1.0))
  INTEGER, PARAMETER :: WPC = KIND((1.0D0,1.0D0))
  REAL(SP), PARAMETER :: PI_S=3.141592653589793238462643383279502884197_sp
  REAL(DP), PARAMETER :: PI=3.141592653589793238462643383279502884197_dp
  Complex(wpc), Parameter  ::  imu = (0._wp,1._wp)


 
  !-------------------------------------------------------
  !------ Some type declarations -------------------------
  !-------------------------------------------------------
  Type parameters
     ! Properties of the incident beam
     Real(wp)           ::  energy_min              ! Photon energy range (eV)
     Real(wp)           ::  energy_max   
     Real(wp)           ::  theta0                  ! Incident angles
     Real(wp)           ::  phi0_calc               ! Parameters for calculations : angle between field and substrate
     Real(wp)           ::  theta0_calc	                  
     Character(len=1)	::  polarization            ! Polarization of the incident beam

     ! Type of calculations
     character(len=10)  ::  geometry                ! Film geometry : island-Yamaguchi-coated
     
     ! Material parameters
     Character(len=10)  ::  island                  ! Island material
     Character(len=10)  ::  substrate               ! Substrate material
     Character(len=10)  ::  coating                 ! Coating material

     ! Some control parameters for the metal
     Character(len=20)  ::  mean_free_path          ! Nature of mean free path 'none' 'finite_size' 'manual' 
     Logical            ::  surface_effects         ! Surface effects 
     Real(wp)           ::  temperature             ! Temperature
     Logical            ::  thermoreflectance       ! Thermoreflectance of the substrate   
	                 
     ! Properties of the islands, the substrate and the layer
     Real(wp)           ::   R_par                  ! Island radius parallel to the substrate (nm)
     Real(wp)           ::   R_per                  ! Island radius perpendicular to the substrate (nm)
     Real(wp)           ::   R                      ! Scale parameter
     Real(wp)           ::   Rapparent              ! Apparent radius
     Real(wp)           ::   xi0                    ! "Spheroidal" radius
     Real(wp)           ::   a                      !  Ring foci distance 
     Real(wp)           ::   tr                     ! Truncation parameter
     Real(wp)           ::   div_surface            ! Pos of the dividing surface (nm)
     Logical            ::   above                  ! MP above substrate ?
     Character(len=10)	::   island_type            ! island type : oblate or prolate ?
     Real(wp) 	        ::   thickness              ! Thichkness of the layer (Yamaguchi coating)
     Real(wp) 	        ::   volume                 ! Volume of the truncated particle
     Real(wp) 	        ::   SR                     ! The shape ratio SR
     Real(wp) 	        ::   mean_thickness         ! Mean thichkness of the island layer
     
     ! "Island-island" parameters
     Character(len=15)  ::   network                ! Type of network : MFT,RPT,square,hexagonal
     Real(wp) 	        ::   coverage               ! The coverage (dimensionless)
     Real(wp) 	        ::   density                ! The density(related to coverage)
     Real(wp) 	        ::   lattice_const          ! The lattice constant
     Character(10)      ::   interaction            ! Island-island interaction
     
     ! Input and Output parameters
     Character(len=200)	::   path                   ! Path for the dielectric constants
     Character(len=200)	::   outfilename            ! Output filename
     Character(len=20)	::   fresnel                ! Formulas for optics
     Character(len=4)	::   out                    ! Nature of output dr dt r t a
     Character(len=200)	::   expfilename            ! Experimental filename
     Logical            ::   compa                  ! Comparison Exp-Theory
     Logical            ::   strenght               ! Output for the multipoles coefficients
     Logical            ::   polarizability         ! Output for the polarizability
     
     ! Values for the potential
     Character(len=10)  ::   bound                  ! Type of calculation (none-BC-Pot)
     Real(wp)           ::   BC_energy              ! Energy for the calculation
     
     
     ! Numerics parameters
     Integer            ::   Nenergy                ! # energy 
     Integer            ::   Levels                 ! # levels used in the lattice sum
     Integer            ::   Mpole_order            ! # multipoles
     Integer            ::   MP_precision           ! The Precision used in the multiple precision module 

     ! Integrations and linear system parameters
     Character(len=20)  ::   int_method             ! Method of integration (gauleg,quadpack)
     Integer            ::   Nint                   ! Number of points for Gauss quadrature
     Integer            ::   limit                  ! Number of allowed subdivision in Quadpack
     Real(wp)           ::   epsabs                 ! Absolute precison in integral
     Real(wp)           ::   epsrel                 ! Relative precision in integral
     Real(wp)           ::  epslin                  ! Relative error in the solution of Ax = b

     ! Some physical constants
     Real(wp) 	::  eps_vacuum 
     Real(wp) 	::  hbar                           ! Plancks constant (eV s)
     Real(wp) 	::  c                              ! Speed of light

     ! Various parameters for the dielectric constants
     Real(wp), Pointer, Dimension(:)      :: energy   
     Complex(wpc), Pointer, Dimension(:)  :: eps_island   
     Complex(wpc), Pointer, Dimension(:)  :: eps_substrate 
     Complex(wpc), Pointer, Dimension(:)  :: eps_coating

  End Type parameters


  !-------------------------------------------------------
  Type Integrals
     Real(wp) ::   Q
     Real(wp) ::   V
     Real(wp) ::   dVdx
     Real(wp) ::   W
     Real(wp) ::   dWdx
  End Type Integrals

  !-------------------------------------------------------
  Type XZ_type
     Real(wp) ::  X
     Real(wp) ::  dXdxi
     Real(wp) ::  Z
     Real(wp) ::  dZdxi
  End Type XZ_type

  !-------------------------------------------------------
  Type surface_constitutive
     Complex(wpc)  :: gamma
     Complex(wpc)  :: beta
     Complex(wpc)  :: delta
     Complex(wpc)  :: tau
  End Type surface_constitutive

  !-------------------------------------------------------
  Type invariants
     Complex(wpc)  :: e
     Complex(wpc)  :: delta_e
     Complex(wpc)  :: tau
     Real(wp)      :: c
  End Type invariants


End Module Global_definitions










