
!------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: graphics.f90,v 1.1 2001/07/29 12:37:53 ingves Exp ingves $
!------------------------------------------------------------------


Module Graphics

  Use Global_definitions, only :  pgplot_device

  Contains


    SUBROUTINE PLOT(XR,YR1,YR2,YR3,xlabel,ylabel,title,flag)
      !----------------------------------------------------------------------
      ! Routine for plotting yr versus xr
      !----------------------------------------------------------------------
      
      IMPLICIT NONE
      INTEGER	                       ::  I,npts
      REAL                             ::  XR(:), YR1(:)
      REAL, OPTIONAL                   ::  YR2(:),YR3(:)
      REAL                             ::  XRMIN,XRMAX,YRMIN,YRMAX
      REAL                             ::  X1,X2,Y1,Y2
      REAL                             ::  XZ1,XZ2,YZ1,YZ2
      CHARACTER(LEN=*), OPTIONAL       ::  xlabel,ylabel,title
      INTEGER                          ::  PGOPEN
      !	VARIABLES FOR CURSOR POSITIONS
      LOGICAL, OPTIONAL                :: FLAG
      LOGICAL                          :: FLAGOUT	
      INTEGER                          :: PGBEG, PGBAND
      INTEGER                          :: JUNK
      CHARACTER(LEN=1)                 :: CH
      REAL                             :: X,Y
      
      
      NPTS = SIZE(XR,1)
      if (.not.present(flag)) then
         flagout=.true.
      else
         flagout=flag
      endif
      
      ! OPEN THE GRAPH WINDOW
      ! '?' FOR INTERACTIVE OPENING
      IF (PGOPEN(trim(adjustl(pgplot_device))) .LE. 0)  STOP
      
      ! SEARCH FOR MAXIMUN AND MINIMUM
      xrmin = minval(xr)
      xrmax = maxval(xr)
      yrmin = minval(yr1)
      yrmax = maxval(yr1)
      if (present(yr2))  then
         yrmin = min(yrmin,minval(yr2))
         yrmax = max(yrmax,maxval(yr2))
      endif
      if (present(yr3))  then
         yrmin = min(yrmin,minval(yr3))
         yrmax = max(yrmax,maxval(yr3))
      endif
      
      ! Call PGENV to initialize the viewport and window; the
      ! AXIS argument is -2, so no frame or labels will be drawn.
10    X1 = XRMIN
      X2 = XRMAX
      Y1 = YRMIN
      Y2 = YRMAX
30    CALL PGERAS
      ! white = 0 background
      ! black =1
      CALL PGSCR(0,1.0,1.0,1.0)
      CALL PGERAS	
      CALL PGSCR(1,0.0,0.0,0.0)
      CALL PGSVP(0.15,0.85,0.25,0.75)
      CALL PGSWIN(x1,x2,y1,y2)
      
      !	BOXES AND AXES
      CALL PGSCI(1)
      CALL PGSLW(1)
      CALL PGSLS(2)
      CALL PGBOX('G',0,0,'G',0,0)
      CALL PGSLS(1)
      CALL PGSLW(4)
      CALL PGBOX('ABCTSN',0,0,'ABCTSNV',0,0)
      
      ! Call PGLAB to label the graph in a different color (1=black).
      !COLOR OF LABELS
      ! SIZE OF LABELS
      if (present(xlabel).and.present(ylabel).and.present(title)) then
         CALL PGSCI(1)
         CALL PGSLW(2)
         CALL PGSCH(1.2)   
         CALL PGSCF(2)
         CALL PGLAB(xlabel,ylabel,title)
      endif
      
      ! Change the color (2 = red), line-style (1=line), and line
      ! width and draw the function.
      CALL PGSLW(1)
      CALL PGSLS(1)
      CALL PGSCI(2)
      CALL PGLINE(NPTS,XR,YR1)
      if (present(yr2)) then
         CALL PGSCI(3)
         CALL PGLINE(NPTS,XR,YR2)	 
      endif
      if (present(yr3)) then
         CALL PGSCI(4)
         CALL PGLINE(NPTS,XR,YR3)	 
      endif
      
      ! CURSOR OR NOT
      !$IS$! if (flagout .eq. .false.) goto 80    
      if (.not.flagout) goto 80    
      
      ! CONTROL OF THE CURSOR
      CALL PGSCI(1)
50    CONTINUE
      JUNK = PGBAND(0,1,X1,Y1,X,Y,CH)
      IF (CH.EQ.'c') THEN
         WRITE (*, '(a,2F8.3)') 'Cursor position : ',X,Y
      ENDIF
      !         Check for exit
      IF (CH.EQ.'/'.OR.CH.EQ.CHAR(4).OR.CH.EQ.'q') GOTO 80
      !		Check for zooming
      IF (CH.EQ.'z') THEN
         XZ1 = X
         YZ1 = Y
         JUNK = PGBAND(2,1,X,Y,X,Y,CH)
         !           WRITE (*, '(a,2F8.3)') ,'Cursor position : ',X,Y
         XZ2 = X
         YZ2 = Y
         X1 = MIN(XZ1,XZ2)
         Y1 = MIN(YZ1,YZ2)
         X2 = MAX(XZ1,XZ2)
         Y2 = MAX(YZ1,YZ2)
         GOTO 30
      ENDIF
      IF (CH.EQ.'u') GOTO 10
      GOTO 50
      
      ! Finally, call PGCLOS to terminate things properly.
      !
80    CALL PGASK(.True.)
      !	CALL PGCLOS
      CALL PGEND
      RETURN
      
    End Subroutine Plot
    !-------------------------------------------------



    SUBROUTINE MULTIPLOT(XR,YR,xlabel,ylabel,title,flag)
      !----------------------------------------------------------------------
      ! Routine for multiple plot
      !----------------------------------------------------------------------
      
      IMPLICIT NONE
      INTEGER                          ::  I,npts
      REAL                             ::  XR(:), YR(:,:)
      REAL                             ::  XRMIN,XRMAX,YRMIN,YRMAX
      REAL                             ::  X1,X2,Y1,Y2
      REAL                             ::  XZ1,XZ2,YZ1,YZ2
      CHARACTER(LEN=*), OPTIONAL       ::  xlabel,ylabel,title
      INTEGER                          ::  PGOPEN
      !	VARIABLES FOR CURSOR POSITIONS
      LOGICAL, OPTIONAL                :: FLAG
      LOGICAL                          :: FLAGOUT	
      INTEGER                          :: PGBEG, PGBAND
      INTEGER                          :: JUNK
      CHARACTER(LEN=1)                 :: CH
      REAL                             :: X,Y
      
      
      NPTS = SIZE(XR,1)
      if (.not.present(flag)) then
         flagout=.true.
      else
         flagout=flag
      endif
      
      ! OPEN THE GRAPH WINDOW
      ! '?' FOR INTERACTIVE OPENING
      IF (PGOPEN(trim(adjustl(pgplot_device))) .LE. 0)  STOP
      
      ! SEARCH FOR MAXIMUN AND MINIMUM
      xrmin = minval(xr)
      xrmax = maxval(xr)
      yrmin = minval(yr)
      yrmax = maxval(yr)
      
      ! Call PGENV to initialize the viewport and window; the
      ! AXIS argument is -2, so no frame or labels will be drawn.
10    X1 = XRMIN
      X2 = XRMAX
      Y1 = YRMIN
      Y2 = YRMAX
30    CALL PGERAS
      ! white = 0 background
      ! black =1
      CALL PGSCR(0,1.0,1.0,1.0)
      CALL PGERAS
      CALL PGSCR(1,0.0,0.0,0.0)
      CALL PGSVP(0.15,0.85,0.25,0.75)
      CALL PGSWIN(x1,x2,y1,y2)
      
      !	BOXES AND AXES
      CALL PGSCI(1)
      CALL PGSLW(1)
      CALL PGSLS(2)
      CALL PGBOX('G',0,0,'G',0,0)
      CALL PGSLS(1)
      CALL PGSLW(4)
      CALL PGBOX('ABCTSN',0,0,'ABCTSNV',0,0)
      
      ! Call PGLAB to label the graph in a different color (1=black).
      !COLOR OF LABELS
      ! SIZE OF LABELS
      if (present(xlabel).and.present(ylabel).and.present(title)) then
         CALL PGSCI(1)
         CALL PGSLW(2)
         CALL PGSCH(1.2)   
         CALL PGSCF(2)
         CALL PGLAB(xlabel,ylabel,title)
      endif
      
      ! Change the color (2 = red), line-style (1=line), and line
      ! width and draw the function.
      CALL PGSLW(3)
      DO I=1,SIZE(YR,2)
         CALL PGSLS(SIZE(YR,2)+1-I)
         CALL PGSCI(SIZE(YR,2)-I+2)
         CALL PGLINE(NPTS,XR,YR(:,I))
      ENDDO
      
      
      ! CURSOR OR NOT
      !$IS$ if (flagout .eq. .false.) goto 80    
      if (.not. flagout) goto 80    
      
      ! CONTROL OF THE CURSOR
      CALL PGSLW(1)
      CALL PGSCI(1)
50    CONTINUE
      JUNK = PGBAND(0,1,X1,Y1,X,Y,CH)
      IF (CH .EQ.'c') THEN
         WRITE (*, '(a,2F8.3)') 'Cursor position : ',X,Y
      ENDIF
      !         Check for exit
      IF (CH.EQ.'/'.OR.CH.EQ.CHAR(4).OR.CH.EQ.'q') GOTO 80
      !		Check for zooming
      IF (CH.EQ.'z') THEN
         XZ1 = X
         YZ1 = Y
         JUNK = PGBAND(2,1,X,Y,X,Y,CH)
         !           WRITE (*, '(a,2F8.3)') ,'Cursor position : ',X,Y
         XZ2 = X
         YZ2 = Y
         X1 = MIN(XZ1,XZ2)
         Y1 = MIN(YZ1,YZ2)
         X2 = MAX(XZ1,XZ2)
         Y2 = MAX(YZ1,YZ2)
         GOTO 30
      ENDIF
      IF (CH.EQ.'u') GOTO 10
      GOTO 50
      
      ! Finally, call PGCLOS to terminate things properly.
      !
80    CALL PGASK(.True.)
      !	CALL PGCLOS
      CALL PGEND
      RETURN
      
    End Subroutine MultiPlot
    
    
    !-------------------------------------------------

    Subroutine Contour(A,title,flag)
      !--------------------------------------------------
      ! Program for displaying a matrix as an image 
      ! - simple display flag=0
      ! - with isolines  flag=1
      ! - with color palettes flag=2
      ! - with cursor zooming flag=3
      !--------------------------------------------------
      
      implicit none
      real                      ::  A(:,:)
      integer, optional         ::  flag
      character(len=*),optional	::  title
      real                      ::  tr(6)
      integer                   ::  flag1
      integer                   ::  PGOPEN
      integer                   ::  ni,nj
      integer                   ::  i,l,c1,c2,nc
      real                      ::  Amin,Amax
      character*16              ::  val
      real                      ::  bright,contra
      real                      ::  ALEV(1)
      integer                   ::  junk,pgband
      character(len=1)          ::  ch
      real                      ::  x,y
      integer                   ::  ni1,ni2,nj1,nj2
      integer                   ::  niz1,njz1,niz2,njz2
      
      
      
      ! Control of the flag
      if (.not.present(flag)) then
         flag1 = 0
      else
         flag1 = flag
      endif
      
      ! Open device for graphics.
      if (PGOPEN('?').lt.0) STOP
      
      ! Ask for the number of colors
      CALL PGQCIR(c1,c2)
      nc = max(0, c2-c1+1)
      !	write(6,*)'Number of color indices used for image: ', nc
      if (nc.lt.8) THEN 
	 write(6,*)'Not enough colors available on this device'
	 stop
      endif
      
      ! Set the coordinate transformation matrix: 
      ! world coordinate = pixel number.
      tr(1) = 0.0
      tr(2) = 0.0
      tr(3) = 1.0
      tr(4) = 0.0
      tr(5) = 1.0
      tr(6) = 0.0
      
      ! Various sizes
      ni = size(a,1)
      nj = size(a,2)
10    ni1 = 1
      ni2 = ni
      nj1 = 1
      nj2 = nj
      
      ! Find the extrema of the matrix and erase the screen
30    Amax = maxval(A(ni1:ni2,nj1:nj2))
      Amin = minval(A(ni1:ni2,nj1:nj2))
      write(6,*) ' Matrix maximun :',Amax
      write(6,*) ' Matrix minimum :',Amin
      
      ! Erase the screen and prepare the screen
      CALL PGERAS
      ! white = 0 background
      ! black =1
      CALL PGSCR(0,1.0,1.0,1.0)
      CALL PGERAS
      CALL PGSCR(1,0.0,0.0,0.0)
      CALL SETVP
      CALL PGWNAD(ni1+0.0, ni2+1.0, nj1+0.0, nj2+1.0)
      !	CALL PGWNAD(imin, imax, nj1+0.0, nj2+1.0)
      ! Set up the color map.
      bright = 0.5
      contra  = 1.0
      CALL PALETT(2, contra, bright)
      ! Draw the map with PGIMAG.  
      CALL PGIMAG(A,ni,nj,ni1,ni2,nj1,nj2,Amin,Amax,tr)
      ! Annotate the plot.
      CALL PGSCI(1)
      CALL PGSLW(3)
      CALL PGSCH(2.0)
      CALL PGMTXT('t',0.5,0.5,0.5,title)
      CALL PGSCH(0.6)
      CALL PGBOX('bcntsi',0.0,0,'bcntsiv',0.0,0)
      CALL PGMTXT('b',3.0,1.0,1.0,'array number')
      ! Draw a wedge.
      CALL PGWEDG('BI', 4.0, 5.0, Amin, Amax, 'array value')
      

      
!!! CHOICE OF OPTIONS
      
      SelectCase(flag1)
      Case(0)
         Goto 80
      Case(1)
         ! Color table
         ! If the device has a cursor, allow user to fiddle with color table.
         !
         CALL PGQINF('CURSOR', VAL, L)
         if (val(:l).eq.'YES') then
            CALL FIDDLE
            CALL PGASK(.True.)
         end if
      Case(2)
         ! Isolines 
         ! Overlay contours in black.
         CALL PGSCI(1)
         do i=1,21
            alev(1) = Amin + (i-1)*(Amax-Amin)/20.0
            if (mod(i,5).eq.0) then
               CALL PGSLW(3)
            else
               CALL PGSLW(1)
            end if
            if (i.lt.10) then
               CALL PGSLS(2)
            else
               CALL PGSLS(1)
            end if
            CALL PGCONT(a,ni,nj,1,ni,1,nj,ALEV,-1,TR)
         enddo
         CALL PGSLS(1)
         CALL PGSLW(1)
      Case(3)
         ! Control the cursor position and zooming
50       CONTINUE
      JUNK = PGBAND(0, 1, ni1, nj1, X,Y,CH)
      WRITE (*, '(a,2F8.3)') 'Cursor position : ',X,Y
      !         Check for exit
      IF (CH.EQ.'/'.OR.CH.EQ.CHAR(4).OR.CH.EQ.'q') GOTO 80
      !		Check for zooming
      IF (CH.EQ.'z') THEN
         niz1 = int(x)
         njz1 = int(y)
         JUNK = PGBAND(2, 1, X, Y, X,Y,CH)
         WRITE (*, '(a,2F8.3)') 'Cursor position : ',X,Y
         niz2 = int(X)
         njz2 = int(Y)
         ni1 = min(niz1,niz2)
         ni2 = max(niz1,niz2)
         nj1 = min(njz1,njz2)
         nj2 = max(njz1,njz2)
         GOTO 30
      ENDIF
      IF (CH.EQ.'u') GOTO 10
      GOTO 50
   Case Default
      ! Defaults
      write(6,*) 'Error in Contour : value of flag'
   End Select
   
   
80 CALL PGASK(.true.)
   !
   ! Close the device and exit.
   !
   CALL PGEND
   
 End Subroutine Contour
 
 
 
 ! --------------------------------------------------------------
 
 SUBROUTINE PALETT(TYP, CONTRA, BRIGHT)
   !-----------------------------------------------------------------------
   ! Set a "palette" of colors in the range of color indices used by
   ! PGIMAG.
   !-----------------------------------------------------------------------
   INTEGER                 ::  TYP
   REAL                    ::  CONTRA, BRIGHT
   REAL,DIMENSION(2)       ::  GL, GR, GG, GB
   REAL,DIMENSION(9)       ::  RL, RR, RG, RB
   REAL,DIMENSION(5)       ::  HL, HR, HG, HB
   REAL,DIMENSION(10)      ::  WL, WR, WG, WB
   REAL,DIMENSION(20)      ::  AL, AR, AG, AB
   !
   DATA GL /0.0, 1.0/
   DATA GR /0.0, 1.0/
   DATA GG /0.0, 1.0/
   DATA GB /0.0, 1.0/
   !
   DATA RL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
   DATA RR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
   DATA RG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
   DATA RB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
   !
   DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
   DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
   DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
   DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/
   !
   DATA WL /0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0/
   DATA WR /0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0/
   DATA WG /0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0/
   DATA WB /0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0/
   !
   DATA AL /0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5, &
        0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0/
   DATA AR /0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0, &
        0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/
   DATA AG /0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8, &
        0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0/
   DATA AB /0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9, &
        0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/
   !
   SELECT CASE(TYP)
   CASE(1)
      !		-- gray scale
      CALL PGCTAB(GL, GR, GG, GB, 2, CONTRA, BRIGHT)
   CASE(2)
      !		-- rainbow
      CALL PGCTAB(RL, RR, RG, RB, 9, CONTRA, BRIGHT)
   CASE(3)
      !		-- heat
      CALL PGCTAB(HL, HR, HG, HB, 5, CONTRA, BRIGHT)
   CASE(4)
      !		-- weird IRAF
      CALL PGCTAB(WL, WR, WG, WB, 10, CONTRA, BRIGHT)
   CASE(5)
      !		-- AIPS
      CALL PGCTAB(AL, AR, AG, AB, 20, CONTRA, BRIGHT)
   END SELECT
   
 END SUBROUTINE PALETT
 
 SUBROUTINE SETVP
   !-----------------------------------------------------------------------
   ! Set the viewport, allowing margins around the edge for annotation.
   ! (This is similar in effect to PGVSTD, but has different margins.)
   ! The routine determines the view-surface size and allocates margins
   ! as fractions of the minimum of width and height.
   !-----------------------------------------------------------------------
   REAL	  :: D, VPX1, VPX2, VPY1, VPY2
   
   CALL PGSVP(0.1, 0.9, 0.1, 0.9)
   CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
   D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
   VPX1 = VPX1 + 5.0*D
   VPX2 = VPX2 - 2.0*D
   VPY1 = VPY1 + 8.0*D
   VPY2 = VPY2 - 2.0*D
   CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
   
 END SUBROUTINE SETVP
 
 SUBROUTINE FIDDLE
   
   INTEGER     :: P, IER, PGCURS
   REAL        :: CONTRA, BRIGHT, X, Y, SIGN
   REAL        :: X1, Y1, X2, Y2, B1, B2, C1, C2
   CHARACTER   :: CH
   
   WRITE (*,*) ' Use cursor to adjust color table:'
   WRITE (*,*) ' Keys 1,2,3,4,5 select different palettes'
   WRITE (*,*) ' Key P cycles through available palettes'
   WRITE (*,*) ' Key F adjusts contrast and brightness, with'
   WRITE (*,*) '  cursor x position setting brightness [0.0 - 1.0]'
   WRITE (*,*) '   and y position setting contrast [0.0 - 10.0]'
   WRITE (*,*) '  (Hold down F key while moving cursor to change'
   WRITE (*,*) '  contrast and brightness continuously)'
   WRITE (*,*) ' Key C resets contrast=1.0, brightness=0.5'
   WRITE (*,*) ' Key - reverses color palette'
   WRITE (*,*) ' Key X or right mouse button exits program' 
   
   P = 2
   CONTRA = 1.0
   BRIGHT = 0.5
   X = 0.5
   Y = 1.0
   SIGN = +1.0
   
   CALL PGQWIN(X1, X2, Y1, Y2)
   B1 = 0.0
   B2 = 1.0
   C1 = 0.0
   C2 = 10.0
   CALL PGSWIN(B1, B2, C1, C2)
10 IER = PGCURS(X, Y, CH)
   IF (CH.EQ.CHAR(0) .OR. CH.EQ.'x' .OR. CH.EQ.'X') THEN
      CALL PGSWIN(X1, X2, Y1, Y2)
      RETURN
   ELSE IF (CH.EQ.'F' .OR. CH.EQ.'f') THEN
      BRIGHT = MAX(B1, MIN(B2,X))
      CONTRA = MAX(C1, MIN(C2,Y))
   ELSE IF (CH.EQ.'C' .OR. CH.EQ.'c') THEN
      CONTRA = 1.0
      Y = 1.0
      BRIGHT = 0.5
      X = 0.5
   ELSE IF (CH.EQ.'-') THEN
      SIGN = -SIGN
   ELSE IF (CH.EQ.'1') THEN
      P = 1
   ELSE IF (CH.EQ.'2') THEN
      P = 2
   ELSE IF (CH.EQ.'3') THEN
      P = 3
   ELSE IF (CH.EQ.'4') THEN
      P = 4
   ELSE IF (CH.EQ.'5') THEN
      P = 5
   ELSE IF (CH.EQ.'P' .OR. CH.EQ.'p') THEN
      P = 1 + MOD(P,5)
   END IF
   CALL PALETT(P, SIGN*CONTRA, BRIGHT)
   GOTO 10
 END SUBROUTINE FIDDLE
 



End Module Graphics


