
!----------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: initialize_mod.f90,v 1.1 2001/07/29 12:38:44 ingves Exp ingves $
!-----------------------------------------------------------------------------------



Module Initialize_mod

Use Global_definitions

Contains

!------------------------------------------------------------------
  Subroutine get_parameters(param)

    Implicit None    
    Type(parameters)   :: param

     ! Properties of the incident light
     param%polarization      = 'p'               ! Polarization of the incident beam
     param%energy_min        = 1.5_wp            ! Photon energy range (eV)
     param%energy_max        = 5._wp
     param%theta0            = pi/4._wp          ! Incident angles (rad)
     param%theta0_calc	     = pi/4._wp
     param%phi0_calc         = 0._wp      

     ! Type of calculations
     param%geometry          = 'island'		 ! Geometry of the system : island-Yamaguchi-coated
     
     ! Material parameters
     param%island            = 'ag'              ! Island material
     param%substrate         = 'mgo'             ! Substrate material
     param%coating           = 'ag'	         ! Coating material
     
     ! Parameters for the dielectric constant of metal
     param%mean_free_path    = 'none'            ! Mean free path of electrons
     param%surface_effects   = .false.           ! Surface effects 
     param%temperature       = 300		 ! Temperature(K)
     param%thermoreflectance = .false.		 ! Thermoreflectance	
     
     ! Properties of the islands and the layer
     param%R_par             =  8.8_wp           ! Island radius parallel to substr. (nm) 
     param%R_per             =  7._wp            ! Island radius perpendicular  - " - 
     param%tr                =  0.99_wp          ! Truncation parameter
     param%thickness         =  1._wp            ! Thickness of the layer or coating layer
     
     ! "Island-island" parameters
     param%network           = 'square'          ! The type of network for interactions
     param%coverage          = 0.3922219_wp      ! Coverage 
     param%lattice_const     = 19.245_wp         ! The lattice constant
     param%interaction       = 'none'            ! Island-island interaction 
     
     ! Output parameters
     param%path	             = 'c:\00Remi\Fortran\Dielectric' ! Path for the dielectrics
     param%outfilename       = 'results'         ! Output filename (basename)
     param%fresnel           = 'constitutive'    ! Type of formulae for DR
     param%out               = 'DR'              ! Nature of output
     param%compa             = .false.           ! Comparison with experiment 
     param%expfilename       = 'experi'          ! Experimental filename
     param%strenght          = .false.           ! Output for the multipoles coefficients
     param%polarizability    = .false.           ! Output for the polarizabilities
     param%bound             = 'none'            ! Potential (Pot) or not (none)
     param%BC_energy         = 3._wp             ! Value of energy for the potential
     
     ! Numerics and method parameters 
     param%Nenergy           = 30                ! # energies  
     param%Mpole_order       = 5                 ! # multipole
     param%Levels            = 500               ! # levels
     param%int_method        = 'quadpack'        ! Integration method
     param%Nint              = 250               ! # points in integration in Gauleg and Quadpack
     param%limit             = 30                ! Number of interval subdivision in Quadpack
     param%epsabs            = 1e-12             ! Absolute precison in the integrals
     param%epsrel            = 1e-9              ! Relative precision in integration
     param%epslin            = 1e-5              ! Relative error in Ax=b
     param%MP_precision      = 20                ! Number of digits in multiple precision
     
     ! Physical constants
     param%hbar               = 6.582122E-16     ! Plancks constant /2pi (eV s)
     param%c                  = 2.99792458E17    ! Speed of light (nm/s)
     param%eps_vacuum          = 1._wp           ! Dielectric function of vacuum
     
     !-----------------------------------
     ! READ PARAMETERS FROM FILE
     !----------------------------------- 
     Call read_param(param)
     
     !-----------------------------------
     ! PROMPTS FOR SOME OF THE PARAMETERS
     !----------------------------------- 
     !     Call prompt_param(param)
     
     ! ---------------------------------
     ! --- Derived Quantities
     ! ---------------------------------
     
     ! Calculate the apparent island radius
     If (param%tr>=0._wp) Then
        param%Rapparent = param%R_par
     Else
        param%Rapparent = param%R_par * Sqrt(1._wp - param%tr**2)
     Endif
     
     If(param%interaction=='none') then
        param%density         =  param%coverage/(pi*param%Rapparent**2)
        param%lattice_const   =  1._wp/(sqrt(param%density))
     EndIf
     
     ! Nature of the network and density
     Select case(param%network)
     Case('square')
        param%density         =  1._wp/(param%lattice_const**2)
     Case('hexagonal')
        param%density         =  1._wp/(sqrt(3._wp)/2._wp*param%lattice_const**2)
     Case('MFT','RPT')
        param%density         =  param%coverage/(pi*param%Rapparent**2)
        param%lattice_const	=  1._wp/(sqrt(param%density))
     Case Default
        Write(6,*) 'Error in the type of network : ',param%network, &
             ' not supported yet !'
        Pause
        stop
     End Select
     
     ! Check for cluster overlapping
     If(2*param%Rapparent>param%lattice_const) Then
        Write(6,*) 'Error in the input : Clusters Overlapping !'
        Pause
     Endif
     
     ! Define the coverage
     param%coverage =  param%density*pi*param%Rapparent**2
     
     ! Decide if the island is of oblate or prolate
     ! and calculate the  spheroidal parameters xi0,a
     ! and the scale parameter
     If(param%R_par>param%R_per) Then 
        ! Oblate
        param%island_type	=	'oblate'
        param%xi0			=	param%R_per / sqrt(param%R_par**2-param%R_per**2)
        param%a				=	param%R_per/param%xi0
     Else
        If(param%R_par<param%R_per) Then
           ! Prolate
           param%island_type	=	'prolate'
           param%xi0			=	param%R_per / sqrt(param%R_per**2-param%R_par**2)
           param%a				=	param%R_per/param%xi0
        Else
	   ! Sphere
	   Write(6,*) '-----------------------------------------'
	   Write(6,*) '              WARNING'
	   Write(6,*) ' The case of a sphere is impossible to   '
	   Write(6,*) ' handle by the spheroidal model          '  
	   Write(6,*) ' Use the SPHERE program insted           '  
	   Write(6,*) '-----------------------------------------'
	   Pause
	   Stop
        End if
     End if
     
     ! This must be defined before tr->|tr|
     ! Volume of the particle
     param%volume	=	pi*param%a**3*param%xi0*(2._wp/3._wp+param%tr-param%tr**3/3._wp)
     Select Case(param%island_type)
     Case('oblate')
        param%volume	=	param%volume*(param%xi0**2+1._wp)
     Case('prolate')
        param%volume	=	param%volume*(param%xi0**2-1._wp)
     End Select
     
     ! Mean thickness of the island layer and shape ratio
     param%mean_thickness =  param%density * param%volume
     param%SR =	 2*param%Rapparent/(param%R_per*(1+param%tr))
     
     ! The position of the deviding surface 
     param%div_surface     =   param%tr*param%R_per
     
     ! Give tr and the right value
     ! NOTICE : In the Bedeaux model tr is always positive
     if (abs(param%tr)>1._wp) then
        write(*,*) 'ERROR : abs(Truncation ration) too big '
        pause
        stop
     endif
     if (param%tr>=0._wp) then
        param%above=.true.        
     else
        param%above=.false.
        param%tr = abs( param%tr )
     endif

     ! The scale parameter
     param%R = param%R_per
    
     Return

   End Subroutine get_parameters

   !---------------------------------------------------------------

   Subroutine Read_Param(param)
     Implicit None
     Type(parameters)   ::  param
     Logical            ::  exi
     Integer            ::  i	
     
     ! Read the input file for the various parameters
     Inquire(file='Spheroid.inc',exist=exi)
     If(.not.exi) Then
        Write(6,*) 'Error in Read_Param ! File Sphereoid.inc non existing'
        Pause
        stop
     Endif
     
     Open(10,file='Spheroid.inc',status='unknown')
     i = 1
     Read(10,*,err=20)	param%path
     i = i + 1
     Read(10,*,err=20)	param%polarization
     i = i + 1
     Read(10,*,err=20)	param%energy_min
     i = i + 1
     Read(10,*,err=20)	param%energy_max
     i = i + 1
     Read(10,*,err=20)	param%theta0
     i = i + 1
     Read(10,*,err=20)	param%geometry
     i = i + 1 
     Read(10,*,err=20)	param%island
     i = i + 1 
     Read(10,*,err=20)	param%substrate
     i = i + 1
     Read(10,*,err=20)	param%coating
     i = i + 1
     Read(10,*,err=20)	param%mean_free_path
     i = i + 1
     Read(10,*,err=20)	param%surface_effects
     i = i + 1
     Read(10,*,err=20)	param%R_par
     i = i + 1
     Read(10,*,err=20)	param%R_per
     i = i + 1
     Read(10,*,err=20)	param%thickness
     i = i + 1
     Read(10,*,err=20)	param%tr
     i = i + 1
     Read(10,*,err=20)	param%network
     i = i + 1
     Read(10,*,err=20)	param%coverage
     i = i + 1
     Read(10,*,err=20)	param%lattice_const
     i = i + 1
     Read(10,*,err=20)	param%interaction
     i = i + 1
     Read(10,*,err=20)	param%fresnel
     i = i + 1
     Read(10,*,err=20)	param%out
     i = i + 1
     Read(10,*,err=20)	param%strenght
     i = i + 1
     Read(10,*,err=20)	param%polarizability
     i = i + 1
     Read(10,*,err=20)	param%Nenergy
     i = i + 1
     Read(10,*,err=20)	param%Mpole_order
     i = i + 1
     Read(10,*,err=20)	param%bound
     i = i + 1
     Read(10,*,err=20)	param%BC_energy
     i = i + 1
     Read(10,*,err=20)	param%outfilename
     i = i + 1
     Read(10,*,err=20)   param%int_method
     i = i + 1
     Read(10,*,err=20)	param%compa
     i = i + 1
     Read(10,*,err=20)	param%expfilename
     
     Close(10,status='keep')
     Return
     
20   Write(6,*) 'Error during reading of Spheroid.inc at line # ',i
     Pause
     Stop 
   End Subroutine Read_Param
   
   !---------------------------------------------------------------

   Subroutine prompt_param(param)
     Implicit None
     Type(parameters)   :: param
     
     Write(6,'(A\)') ' Type of calculation     : '
     Read(5,*) param%out
     Write(6,'(A\)') ' Island material         : '
     Read(5,*) param%island
     Write(6,'(A\)') ' Substrate material      : '
     Read(5,*) param%substrate 
     Write(6,'(A\)') ' Temperature(K)          : '
     Read(5,*) param%temperature
     Write(6,'(A\)') ' Truncation ratio        : '
     Read(5,*) param%tr  
     Write(6,'(A\)') ' Radius perpendicular to the surface : '
     Read(5,*) param%R_per   
     Write(6,'(A\)') ' Radius parallel to the surface : '
     Read(5,*) param%R_par 
     Write(6,'(A\)') ' Coverage                : '
     Read(5,*) param%coverage     
     Write(6,'(A\)') ' Lattice constant        : '
     Read(5,*) param%lattice_const
     Write(6,'(A\)') ' # multipoles            : '
     Read(5,*) param%Mpole_order 
     Write(6,'(A\)') ' Interaction             : '
     Read(5,*) param%interaction
     Write(6,'(A\)') ' Experimental file       : '
     Read(5,*) param%expfilename
     Write(6,*)
     
     Return
   End Subroutine prompt_param
   
   !---------------------------------------------------------------

   Subroutine initialize(A,b,param,Fresnel_coeff,mpoled,mpoleq,Int,XZ, &
        surf_const_coef,zeta)
     
     Use FMZM,  only:	ZM_SET
     Implicit None
     Type(parameters)                    :: param
     Type(integrals),Pointer             :: Int(:,:,:)
     Type(XZ_type), Pointer              :: XZ(:,:)     
     Type(surface_constitutive), Pointer :: surf_const_coef(:)   
     Real(wp),Pointer                    :: zeta(:,:,:), Fresnel_coeff(:)
     Complex(wpc), Pointer               :: A(:,:,:), b(:,:)
     Complex(wpc),Pointer                :: mpoled(:,:,:),mpoleq(:,:)
     Integer                             :: mpo
     
     Write(6,*) '*******************************************'
     Write(6,*) '               SPHEROID'
     Write(6,*) ' Optics calculations for supported islands'
     Write(6,*) '             January 1999'
     Write(6,*) '*******************************************'
     
     
     ! Initialize the parameters of the problem
     Call get_parameters(param)
          
     Select Case(param%geometry)
        ! Truncated spheroids case
     Case('island')
        mpo = param%Mpole_order
        !------------------------------------------------
        ! Allocate space for vectors and matrices
        !------------------------------------------------        
        ! Allocate space for the linear (matrix) system
        Allocate( A(2*mpo,2*mpo,0:1) )
        Allocate( b(2*mpo,0:1) ) 
        ! Allocate space for the various integrals
        Allocate( Int(0:mpo,0:mpo,0:1) )
        Allocate( XZ(0:mpo,0:1) )
        Allocate( zeta(0:mpo,0:mpo,0:1) )
        ! Allocate space for the optical output
        Allocate( mpoled(2,0:1,param%Nenergy) )
        Allocate( mpoleq(3,param%Nenergy) ) ! Just for compatibility with optics_mod
        Allocate( surf_const_coef(param%Nenergy))   
        Allocate( Fresnel_coeff(param%Nenergy) )    
        ! Calculate the zeta-coeffisients
        Call get_zeta(zeta,param)    
        ! Initize the multiple Precision module
        call ZM_SET(param%MP_precision)
        
	! Yamaguchi model spheroids
     Case('Yamaguchi','coated')
        ! Allocate space for the polarizability
        Allocate( mpoled(2,2,param%Nenergy) )
        ! Allocate space for the optical output
        Allocate( surf_const_coef(param%Nenergy))   
        Allocate( Fresnel_coeff(param%Nenergy) ) 
        
     Case Default
        Write(*,*) ' Error in initialize for param%geometry !! '
     End Select
     
     ! Initialize the dielectrics constants
     Call initialize_dielectric_constants(param)
     Return
   End Subroutine initialize
   
   !---------------------------------------------------------------
   
   Subroutine initialize_dielectric_constants(param)
 
     Implicit None
     Type(parameters)  ::  param
     Real(wp)          ::  denergy,de
     Integer           ::  ienergy,di
     
     ! Allocate space for the dielectric constants
     Allocate(param%energy(param%Nenergy))
     Allocate(param%eps_substrate(param%Nenergy))   
     Allocate(param%eps_island(param%Nenergy))

     ! Fill the energy vector
     denergy =  (param%energy_max-param%energy_min)/(param%Nenergy-1)
     param%energy(1) = param%energy_min
     Do ienergy=2,param%Nenergy
        param%energy(ienergy) = param%energy(ienergy-1)+denergy
     Enddo
     
     ! Fills the dielectric constant vectors
     Call dielectric_constants(param%energy,param%eps_island,param%island,param%path)
     Call dielectric_constants(param%energy,param%eps_substrate,param%substrate,param%path)
     
     ! To take intoaccount the thermoreflectance
     If(param%thermoreflectance) Then
        Write(6,*)
        Write(6,'(a\)') ' Enter the energy shift(eV) for the substrate : '
        Read(5,*) de
        di = Int(de/denergy)
        Do ienergy=param%Nenergy,di+1,-1
           param%eps_substrate(ienergy) = param%eps_substrate(ienergy-di)
        Enddo
     Endif
     
     ! Finite size scaling of the dielectric constant of the island 
     if (param%mean_free_path/='none')  &
          Call dielectric_func_corrections(param%eps_island,param%island,param%path,param)
     Return
     
   End Subroutine initialize_dielectric_constants
   
   !---------------------------------------------------------------

   
   Subroutine dielectric_constants(energy,epsilon,material,path)
     
     Implicit None
     Real(wp)			:: energy(:)
     Complex(wpc)               :: Epsilon(:)
     Character(len=*)		:: material,path
     Character(len=200)         :: filename
     Logical			:: exi
     Integer			:: lines,start,i
     Real(wp),    Allocatable	:: x(:)
     Complex(wpc), Allocatable  :: y(:)
     Real(wp)                   :: tmp(2),x1,x2,dx
     Integer			:: unit
     Complex(wpc)               :: slope
     Complex(wpc),parameter	:: im=(0._wp,1._wp)
     
     ! Opens the relevant data file for the given material
     ! Notice that the data-files contains the values for the 
     ! index of refracction, n,k extacted from the data base of
     ! SOPRA.
     ! Therefore "epsilon = epsilon**2" at the bottom of this routine    
     filename = Trim(Adjustl(path))//directory_separator//Trim(Adjustl(material))//'.nk'
     filename = Trim(Adjustl(path))//'/'//Trim(Adjustl(material))//'.nk'
     Inquire(file = filename, exist = exi)
     If(.not. exi) then
        Write(*,*) 'ERROR (get_epsilon) : File = ',trim(adjustl(filename)),' non existing'
        Pause
        Stop
     Endif
     Open(10,file=filename,status='old')
     
     !	Open(unit=10,file=filename)
     !	Read(10,*) lines
     !	Allocate( x(lines), y(lines) )
     !	Do i=1,lines
     !	 Read(10,*) x(i),x1,x2
     !	 y(i)=x1+im*x2
     !	Enddo
   
     Open(unit=10,file=filename)
     Read(10,*) unit,x1,x2,lines
     dx = (x2 - x1)/(lines-1)

     Allocate( x(lines), y(lines) )
     Select Case (unit)
     Case(1)
        ! Unit = Electron Volt (eV)
        Do i=1,lines 
           Read(10,*) tmp(1), tmp(2)
           x(i) = x1 + (i-1)*dx
           y(i) = tmp(1)+imu*tmp(2)
        Enddo
     Case(2)
        ! Unit = WaveLenght (*m)
        Do i=lines,1,-1 
           Read(10,*) tmp(1), tmp(2)
           x(i) = x1 + (lines-i)*dx
           y(i) = tmp(1)+imu*tmp(2)
        Enddo
        x(:) = 1.243/x(:) ! Conversiob *m-->eV
     End Select
     
     Close(10)

     ! Do the interpolation
     Do i=1,Size(energy,1)
        start=locate(Real(x(:),sp),Real(energy(i),sp))
        If ((start==0).Or.(start==lines)) then
           Write(*,*) 'ERROR (get_epsilon) : Energy not in range : ',energy(i)
           Pause
           stop
        End If
        ! Linear interpolation
        slope = (y(start+1)-y(start))/(x(start+1)-x(start))
        Epsilon(i) = y(start) + slope*(energy(i)-x(start))
        !		write(67,*) energy(i),real(epsilon(i)),aimag(epsilon(i))
     Enddo
     
     ! Calculates the dielectric constant (from the refraction index)
     epsilon = epsilon**2
     Deallocate(x,y)
     
     Return
   End Subroutine dielectric_constants
   

   !---------------------------------------------------------------

   Subroutine dielectric_func_corrections(epsilon,material,path,param,d)
     ! Calculates the correction to the dielectric functetion due to the 
     ! finit size of the islands
     ! See Physica A143, 164 (1987), for details 
	   
     Implicit None
     Type(Parameters)          :: param
     Complex(wpc)              :: Epsilon(:),tmp_surf_eff
     Character(len=*)          :: material,path
     Character(len=200)        :: filename
     Logical                   :: exi
     Integer                   :: ienergy 
     Real(wp)                  :: omega_p,inv_relaxation_time,fermi_velocity
     Real(wp)                  :: R_eff,inv_tau_bulk,inv_tau,E,surf_eff,disp_T
     Real(wp)                  :: percent,A
     Real(wp),Optional         :: d
     
     
     ! Opens the relevant data file for the given material
     filename = Trim(Adjustl(path))//directory_separator//'Finite_Size'//directory_separator//Trim(Adjustl(material))//'.dat'
     Inquire(file = filename, exist = exi)
     If(.not. exi) then
        Write(*,*) 'ERROR (dielectric_func_corrections) : File = ', &
             filename,' non existing'
        Pause
        Return
     Endif 
     Open(10,file=filename,status='old')
     
     ! Reads the data (all quanteties are multiplied by hbar)
     Read(10,*) omega_p             ! Plasma frequency  (eV)
     Read(10,*) inv_relaxation_time ! Inverse bulk relaxation time (eV) 
     Read(10,*) fermi_velocity      ! Fermi velicity (eV nm)
     Read(10,*) surf_eff            ! Surface effects (eV^2) 
     Read(10,*) disp_T		    ! Dispersion of plasma frequency with T
     Close(10)

     inv_tau_bulk = inv_relaxation_time
     A	= 0.8_wp
     Select case (trim(adjustl(param%island_type)))
     Case('oblate')
        R_eff        = param%R_per+param%div_surface
     Case('prolate')
        R_eff        = param%R_per+param%div_surface
        If(R_eff>param%R_par) R_eff = param%R_par      
     End select
     
     ! Modify Reff if optional d present
     If(Present(d)) R_eff = d

     ! Temperature effects on the dielectric constant
     ! See Rocca Moresco Valbusa PRB 45,3 1399 (1992)
     omega_p = omega_p + disp_T * (param%temperature-300)

     ! Nature of mean free path limitation
     Select case(param%mean_free_path)
     Case('finite_size')
        inv_tau      = inv_tau_bulk + A*fermi_velocity/R_eff  ! Physica A, Eq. (3.6)
     Case('s-only')
        Write(6,'(a,f6.2,a\)') ' Enter the inverse of relaxation time (eV-1) (bulk= ',inv_tau_bulk,')  : '
        Read(5,*) inv_tau
     Case('tau')
        Write(6,'(a,f6.2,a\)') ' Enter the inverse of relaxation time (eV-1) (bulk= ',inv_tau_bulk,')  : '
        Read(5,*) inv_tau
     Case('manual')
        Write(6,'(a\)') ' Enter the percentage of bulk absorption : '
        Read(5,*) percent
        Write(6,*)
        inv_tau      = inv_tau_bulk + A*fermi_velocity/R_eff
     Case('A-parameter')
        Write(6,'(a\)') ' Enter the A-parameter : '
        Read(5,*) A
        inv_tau      =   inv_tau_bulk + A*fermi_velocity/R_eff
     Case default
        Write(6,*) 'Error in dielectric_func_corrections for param%mean_free_path'
     End select
     
     
     ! Update the dielectric constant of the island-material 
     ! See Physica A143, 164 (1987)   
     Do ienergy = 1,param%Nenergy
        E                 =    param%energy(ienergy) 
        ! Surface effects 
        ! See Liebsch, PRB B48, 11 317 (1993).
        ! Eq. (5.1) and (5.2) 
        if (param%surface_effects) then
           tmp_surf_eff = omega_p**2/(E**2 + surf_eff/R_eff + imu*E*inv_tau)
        else
           tmp_surf_eff = omega_p**2/(E**2 +imu*E*inv_tau)
        end if
        ! Dielectric constant of d-electrons for silver
        Epsilon(ienergy)  =  Epsilon(ienergy)                         &
             + omega_p**2/(E**2 + imu*E*inv_tau_bulk)
        ! Type of mean free path 
        Select case(param%mean_free_path)
        Case('s-only')
           Epsilon(ienergy)  =  Real(Epsilon(ienergy)) - tmp_surf_eff
        Case('tau')
           Epsilon(ienergy)  =  Epsilon(ienergy) - tmp_surf_eff
        Case('manual')
           Epsilon(ienergy)  =	 Real(Epsilon(ienergy)-tmp_surf_eff) + &
                imu*percent*Aimag(Epsilon(ienergy)-tmp_surf_eff)
        Case('finite_size','A-parameter')
           Epsilon(ienergy)  =  Epsilon(ienergy) - tmp_surf_eff
        Case default
        End select
     Enddo
     
   End Subroutine dielectric_func_corrections


   !------------------------------------------------------------------

   Subroutine get_zeta(zeta,param)

     Implicit None
     Type(parameters)       :: param
     Real(wp)               :: zeta(0:,0:,0:)
     Integer                :: l1,l2,m
     Real(wp)               :: factorial
     
     zeta(:,:,:) = 0._wp
     
     Do m=0,1
        Selectcase(m)
        Case(0)
           Do l1=0,param%Mpole_order
              Do l2=0,param%Mpole_order
                 zeta(l1,l2,m) = 0.5_wp *  Sqrt( (2*l1+1._wp)*(2*l2+1._wp) )
              Enddo
           Enddo
        Case(1)
           Do l1=1,param%Mpole_order
              Do l2=1,param%Mpole_order
                 factorial = 1._wp/( l1*(l1+1._wp)*l2*(l2+1._wp) )
                 zeta(l1,l2,m) = 0.5_wp *  Sqrt( (2*l1+1._wp)*(2*l2+1._wp)*factorial ) 
              Enddo
           Enddo
        End Select
     Enddo
     
     Return
     
   End Subroutine get_zeta

   !----------------------------------------------------------------------
	
   Function locate(xx,x)
     Implicit None     
     Real, Dimension(:), Intent(IN)	:: xx
     Real, Intent(IN)                   :: x
     Integer                            :: locate
     Integer                            :: n,jl,jm,ju
     Logical                            :: ascnd
     
     n=size(xx)
     ascnd = (xx(n) >= xx(1))
     jl=0
     ju=n+1
     do
        if (ju-jl <= 1) exit
        jm=(ju+jl)/2
        if (ascnd .eqv. (x >= xx(jm))) then
           jl=jm
        else
           ju=jm
        end if
     end do
     if (x == xx(1)) then
        locate=1
     else if (x == xx(n)) then
        locate=n-1
     else
        locate=jl
     end if
     
   End Function locate
   


 End Module Initialize_mod
