
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: integral_mod.f90,v 1.1 2001/07/29 12:39:50 ingves Exp ingves $
!---------------------------------------------------------------------------------


Module Integral_mod

  Use Global_definitions
  Use Oblate_Int_Gauleg,    only : Get_integrals_oblate_Gauleg,XZ_oblate
  Use Prolate_Int_Gauleg,   only : Get_integrals_prolate_Gauleg,XZ_prolate
  Use Oblate_Int_Quadpack,  only : Get_integrals_oblate_Quadpack
  Use Prolate_Int_Quadpack, only : Get_integrals_prolate_Quadpack



!----------------------------------------------------------------
!	The main routines for getting the integrals and XZ functions
!----------------------------------------------------------------

Contains

!---------------------------------------------------------------
  Subroutine Get_Int_XZ(Int,XZ,param)

    Implicit None
    ! Input
    Type(parameters)           ::   param    
    Type(integrals)            ::   Int(0:,0:,0:)
    Type(XZ_type)              ::   XZ(0:,0:)
    ! Local
    Real(wp)                   ::   xi0(1)
    Type(XZ_type),Allocatable  ::   XZ1(:,:,:)

    !	integer		:: l,m
    xi0(1)	=	param%xi0
    Allocate(XZ1(1,0:size(XZ,1)-1,0:size(XZ,2)-1))
    
    !-------------------------------
    !		GEOMETRY
    !-------------------------------
    Select case(trim(adjustl(param%island_type))) 
       
       !-------------------------------
       !		Oblate
       !-------------------------------	     
    Case('oblate')
       !-------------------------------
       ! Main routine to get integrals
       !-------------------------------
       Select Case(param%int_method)
       Case('gauleg')
          Call Get_integrals_oblate_gauleg(Int,param)
       Case('quadpack')
          Call Get_integrals_oblate_quadpack(Int,param)
       Case Default
          Write(6,*) 'Error in the integration method !'
       End Select
       !-----------------------------------------
       ! Main routine to get the X,Z,dX,dZ in xi0
       !-----------------------------------------
       Call XZ_oblate(XZ1,xi0)
       !	do l=0,size(XZ,1)-1
       !	 do m=0,1
       !	 write(70,*) l,m,XZ(l,m)%X
       !	 write(71,*) l,m,XZ(l,m)%Z
       !	 write(72,*) l,m,XZ(l,m)%dXdxi
       !	 write(73,*) l,m,XZ(l,m)%dZdxi
       !	 enddo
       !	enddo
       
       
       
       !-------------------------------
       !		Prolate
       !-------------------------------
    Case('prolate')
       !-------------------------------
       ! Main routine to get integrals
       !-------------------------------
       Select Case(param%int_method)
       Case('gauleg')
          Call Get_integrals_prolate_gauleg(Int,param)
       Case('quadpack')
          Call Get_integrals_prolate_quadpack(Int,param)
       Case Default
          Write(6,*) 'Error in the integration method !'
       End Select
       !-----------------------------------------
       ! Main routine to get the X,Z,dX,dZ in xi0
       !-----------------------------------------
       Call XZ_prolate(XZ1,xi0)
       !	do l=0,size(XZ,1)-1
       !	 do m=0,1
       !	 write(70,*) l,m,XZ(l,m)%X
       !	 write(71,*) l,m,XZ(l,m)%Z
       !	 write(72,*) l,m,XZ(l,m)%dXdxi
       !	 write(73,*) l,m,XZ(l,m)%dZdxi
       !	 enddo
       !	enddo	
       
    Case default
       Write(6,*) 'Error in Get_Int_XZ for param%island_type'
       Pause
       stop
    End select
    
    XZ(0:,0:)	=	XZ1(1,0:,0:)
    Deallocate(XZ1)
    
  End Subroutine Get_int_XZ
  
  
End Module Integral_mod






