
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: linsolver_mod.f90,v 1.1 2001/07/29 12:41:28 ingves Exp ingves $
!---------------------------------------------------------------------------------





Module linsolver_mod
  
  Use Global_definitions, only	:	wp,wpc,sp,dp
  ! Use Graphics
  
Contains
  
  !---------------------------------------------------------------

  Subroutine linsolver(A,b,eps)

    Implicit None
    Complex(wpc)                ::  A(:,:), b(:)
    Real(wp)                    ::  eps
    Complex(wpc),Allocatable    ::  AA(:,:)
    Complex(wpc),Allocatable	::  bb(:)
    Integer                     ::  m, n
    Integer,Allocatable	        ::  pivot(:)
    Integer                     ::  info(4)
    Real(wp)                    ::  rcond
    Complex(wpc),Allocatable	::  work(:)
    Real(wp),Allocatable        ::  rwork(:)
    Real(wp)                    ::  ferr(1),berr(1)
    Real(wp)                    ::  error
    
    
    !    ! Plot the matrix A
    !    call contour(real(abs(A),sp),flag=0)
    rcond = 0._wp
  
    ! Some initial abbreviations
    m      =  Size(A,1)
    n      =  Size(A,2)
    
    ! Allocation of arrays
    Allocate(AA(m,n),bb(n),pivot(n),work(2*n),rwork(2*n)) 
    
    ! Working matrix and coulumns    
    AA=A
    bb=b
    
    Select Case(wp)
       !------------------------   
       ! ---Single precision
       !------------------------   
    Case(sp)                    
       ! LAPACK routines
       ! LU-factorization
       call cgetrf(m,n,AA,m,pivot,info(1))
       ! LU back susbtitution
       call cgetrs('n',n,1,AA,m,pivot,bb,n,info(2))
       ! Evaluation of the condition number
       call cgecon('1',n,AA,m,1._wp,rcond,work,rwork,info(3))
       ! Improvement of the calculated solution
       call cgerfs('n',n,1,A,m,AA,m,pivot,b,n,bb,n,ferr,berr,work,rwork,info(4))
       
       !------------------------   
       ! --- Double precision       
       !------------------------     
    Case(dp)                         
       ! LAPACK routines
       ! LU-factorization 	   
       call zgetrf(m,n,AA,m,pivot,info(1))
       ! LU back susbtitution
       call zgetrs('n',n,1,AA,m,pivot,bb,n,info(2))
       ! Evaluation of the condition number
       call zgecon('1',n,AA,m,1._wp,rcond,work,rwork,info(3))
       ! Improvement of the calculated solution
       call zgerfs('n',n,1,A,m,AA,m,pivot,b,n,bb,n,ferr,berr,work,rwork,info(4))
       
    End Select
    
    ! Error codes in the Lapack routines
    if (info(1)/=0) then
       write(*,*) 'Error code in LU decomposition : ',info(1)
    endif
    if (info(2)/=0) then
       write(*,*) 'Error code in LU backsubstitution : ',info(2)
    endif
    if (info(3)/=0) then
       write(*,*) 'Error code in condition number evaluation : ',info(3)
    endif
    if (info(4)/=0) then
       write(*,*) 'Error code in iterative improvment of solution : ',info(4)
    endif
    
    ! An estimation of the error
    error = Sqrt(Sum(Abs(Matmul(A(:,:),bb(:)) - b(:))**2)) 
    error = error/Sqrt(Sum(Abs(b(:))**2))
    
    open(unit=15,file='error_lin.dat',status='unknown')
    ! 1/rcond is the condition number of the matrix (ratio of max and min singular values)
    ! ferr(1) is a upper bound of the relative error in the solution of the system
    write(15,*) 1._wp/rcond,ferr(1),error
    
    ! Deallocation of arrays
    Deallocate(AA,pivot,work,rwork) 
    
    ! Check for the precision of the solution and if necessary
    ! resolve the linear system in multiple precision
    If(error.lt.eps) Then
       ! Output of the solution
       b = bb
       Deallocate(bb)
       Return
    Else
       Deallocate(bb)
       write(6,*) 'ZMlinsolver under progress !!'
       Call ZMlinsolver(A,b,eps)
    Endif
    
    Return
    
  End Subroutine linsolver

  !-------------------------------------------------------------------
	
  Subroutine ZMlinsolver(A,b,eps)
    
    ! Linear Solver in multiple precision
    
    Use FMZM	
    Implicit None
    Complex(wpc)              ::  A(:,:),b(:)
    Real(wp)                  ::  eps
    Type(ZM),Allocatable      ::  ZMA(:,:),ZMb(:)
    Type(ZM),Allocatable      ::  ZMbb(:)
    Integer,Allocatable	      ::  indx(:)
    Integer                   ::  n,i,j
    Integer                   ::  digit
    Type(ZM)                  ::  ZMbbb
    Type(FM)                  ::  error,bbb
    Character(len=60)         ::  string
    

    ! Allocation of arrays for multiple precision
    n		=	Size(A,1)
    Allocate(ZMA(n,n),ZMb(n),ZMbb(n),indx(n))
    
    ! Convert for wp-type to ZM type to full machine accuracy
10  Do i=1,n
       write(string,*) b(i)
       string = Trim(Adjustl(string))
       ZMb(i)		=	TO_ZM(string)
       ZMbb(i)	=	TO_ZM(string)
       Do j=1,n
	  write(string,*) A(i,j)		
	  string = Trim(Adjustl(string))
	  ZMA(i,j)	=	TO_ZM(string)
       Enddo
    Enddo
    
    ! Solve the linear system 
    Call ZMLUDCMP(ZMA,indx)
    Call ZMLUBKSB(ZMA,indx,ZMb)
    
    ! Check the backward error in multiple precision 
    ! Convert for wp-type to ZM type because ZMA contains 
    ! the LU-decomposition
    Do i=1,n
       Do j=1,n
	  write(string,*) A(i,j)
	  string = Trim(Adjustl(string))		
	  ZMA(i,j)	=	TO_ZM(string)
       Enddo
    Enddo
    error	=	TO_FM(0)
    bbb		=	TO_FM(0)
    Do i=1,n
       ZMbbb	=	TO_ZM(0)
       bbb	=	bbb + Abs(ZMbb(i))**2
       Do j=1,n
	  ZMbbb		=	ZMbbb + ZMA(i,j)*ZMb(j)
       Enddo
       ZMbbb		=	ZMbbb - ZMbb(i) 
       error		=	error + Abs(ZMbbb)**2
    Enddo
    error		=	Sqrt(error/bbb)
    
    
    If(error.gt.eps) Then
       ! Increase the number of significant digits
       digit	=	digit + 5
       Call ZM_SET(digit)
       Goto 10
    Endif
    
    ! Revert conversion for the solution of the linear system
    Select Case(wp)
    Case(sp)
       Do i=1,n
	  b(i)	=	TO_SPZ(ZMb(i))
       Enddo
    Case(dp)
       Do i=1,n
	  b(i)	=	TO_DPZ(ZMb(i))
       Enddo
    End Select
    
    ! Write the necessary precision for the inversion
    Call FM_FORM('f10.4',error,string)
    open(unit=15,file='error_lin.dat',status='unknown')
    write(15,'(i5,5x,a10)') digit,string(1:10)
    
    Deallocate(ZMA,ZMb,ZMbb,indx)
    
  End Subroutine ZMlinsolver
  
  !-------------------------------------------------------------------

  Subroutine ZMLUDCMP(ZMA,indx)
    
    ! Numerical Recipes Routine for LU-decomposition
    ! adapted for multiple complex precision
    
    Use FMZM
    Implicit None
    Type(ZM)                ::   ZMA(:,:)
    Integer                 ::   indx(:)
    Type(ZM)                ::   SMALL
    Integer                 ::   i,imax,j,k
    Type(ZM)                ::   sum,cdum
    Type(FM)                ::   aamax,rdum,ZERO
    Type(FM),Allocatable    ::   vv(:)
    Integer                 ::   n
    
    ZERO        = TO_FM('0')
    SMALL       = TO_ZM('1D-60+i1e-60') 
    n = Size(ZMA,1)
    Allocate(vv(n))
    
    do i=1,n
       aamax=ZERO
       do j=1,n
          if (abs(ZMA(i,j)).gt.aamax) aamax=abs(ZMA(i,j))
       enddo
       if (aamax.eq.ZERO) then
          pause 'singular matrix in ZMLUDCMP'
          stop
       endif
       vv(i)=1./aamax
    enddo
    
    do j=1,n
       do i=1,j-1
          sum=ZMA(i,j)
          do k=1,i-1
             sum=sum-ZMA(i,k)*ZMA(k,j)
          enddo
          ZMA(i,j)=sum
       enddo
       aamax=ZERO
       do i=j,n
          sum=ZMA(i,j)
          do k=1,j-1
             sum=sum-ZMA(i,k)*ZMA(k,j)
          enddo
          ZMA(i,j)=sum
          rdum=vv(i)*abs(sum)
          if (rdum.ge.aamax) then
             imax=i
             aamax=rdum
          endif
       enddo
       if (j.ne.imax)then
          do k=1,n
             cdum=ZMA(imax,k)
             ZMA(imax,k)=ZMA(j,k)
             ZMA(j,k)=cdum
          enddo
          vv(imax)=vv(j)
       endif
       indx(j)=imax
       if(ZMA(j,j).eq.ZERO) ZMA(j,j) = SMALL
       if(j.ne.n)then
          cdum=1./ZMA(j,j)
          do i=j+1,n
             ZMA(i,j)=ZMA(i,j)*cdum
          enddo
       endif
    enddo
    
    Deallocate(vv)
    Return
    
  End Subroutine ZMLUDCMP
  
  !----------------------------------------------------------------------
  
  Subroutine ZMLUBKSB(ZMA,indx,ZMb)

    ! Numerical Recipes Routine for LU-backsubstitution
    ! adapted for multiple complex precision
    
    
    Use FMZM
    Implicit None
    Type(ZM)     ::  ZMA(:,:),ZMb(:)
    Integer      ::  indx(:)
    Integer      ::  i,ii,j,ll,n
    Type(ZM)     ::  sum
    Type(FM)     ::  ZERO
    
    ZERO = TO_FM(0)
    n = Size(ZMA,1)
    ii=0
    do i=1,n
       ll=indx(i)
       sum=ZMb(ll)
       ZMb(ll)=ZMb(i)
       if (ii.ne.0)then
          do j=ii,i-1
             sum=sum-ZMA(i,j)*ZMb(j)
          enddo
       else if (sum.ne.ZERO) then
          ii=i
       endif
       ZMb(i)=sum
    enddo
    do i=n,1,-1
       sum=ZMb(i)
       do j=i+1,n
          sum=sum-ZMA(i,j)*ZMb(j)
       enddo
       ZMb(i)=sum/ZMA(i,i)
    enddo
    
    Return
    
  End Subroutine ZMLUBKSB

  !----------------------------------------------------------------------
  
End Module linsolver_mod










