
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: potential_mod.f90,v 1.1 2001/07/29 12:44:52 ingves Exp ingves $
!---------------------------------------------------------------------------------



Module Potential_mod


  Use Global_definitions
  Use Graphics, only	: contour
  
Contains

  !-----------------------------------------------------------------------

  Subroutine Potential(MPC,Int,XZ,ienergy,param)

    Implicit None	
    Type(parameters)	:: param 
    Integer             :: ienergy	
    Complex(wpc)        :: MPC(:,0:)
    Type(Integrals)     :: Int(0:,0:,0:) 
    Type(XZ_type)       :: XZ(0:,0:)
    
    Select Case(param%above)
    Case(.true.)
       Select Case(param%island_type)
       Case('oblate')
          Call Potential_oblate_above(MPC,Int,XZ,ienergy,param)
       Case('prolate')
          Call Potential_prolate_above(MPC,Int,XZ,ienergy,param)	 
       End Select
    Case Default
       Write(6,*) ' The below case for the potential is not supported yet !'
    End Select
    
  End Subroutine Potential

  !------------------------------------------------------------------------

  Subroutine Potential_oblate_above(MPC,Int,XZ,ienergy,param)
  
    !-----------------------------------------------------------
    ! Multipole located above the surface for an oblate spheroid
    !-----------------------------------------------------------
    
    Use Legendre_mod, only           : LegendreP_R
    Use Oblate_Int_Quadpack, only    : xi_eta_trans_oblate,X_Oblate,Z_Oblate
    
    Implicit none
    ! Global
    Type(parameters)    :: param 
    Integer             :: ienergy
    Complex(wpc)        :: MPC(:,0:)
    Type(Integrals)     :: Int(0:,0:,0:) 
    Type(XZ_type)       :: XZ(0:,0:)
    ! Local
    ! The multipole coefficients
    Complex(wpc),Allocatable	:: Am(:,:),Bm(:,:),Arm(:,:),Brm(:,:),Atm(:,:),Btm(:,:)
    ! The spheroidal coordinates on the grid 
    Real(wp),Allocatable        :: xi_mp(:),eta_mp(:),phi(:)
    ! Spheroidal coordinates from the image point
    Real(wp),Allocatable        :: xi_imp(:),eta_imp(:)
    ! Some abreviations
    Real(wp)                    :: a,tr,xi1,xi0,AR
    ! The spherical harmonics
    Real(wp)                    :: Y_mp,Y_imp
    Real(wp)                    :: Z_mp,Z_imp
    Real(wp)                    :: X_mp,X_imp
    ! For the constant b0
    Complex(wpc)                :: b0
    ! Prefactor of the sph erical harmonics                             
    Real(wp),Allocatable        :: prefactor(:,:)
    Real(wp),Allocatable        :: zeta(:)
    ! Various parameters
    Integer                     :: mpo,l,m,ipoint
    Real(wp)                    :: tmp,pos(2)
    Real(wp)                    :: t0,p0
    Real(wp)                    :: E0x,E0y,E0z
    Complex(wpc)                :: e1,e2,e3,e4,v0,v
    Complex(wpc)                :: phase,ctmp
    ! Parameters for the grid
    Real(sp),Allocatable        :: Grid(:,:)
    Real(wp),Allocatable        :: x1(:),x2(:)
    Integer,Parameter           :: Nx1=50
    Integer,Parameter           :: Nx2=50
    Integer,Parameter           :: Npoints=Nx1*Nx2
    Character(len=1)            :: cut
    Real(wp)                    :: value
    Real(wp), parameter         :: SG=2._wp
    Real(wp)                    :: dx1,dx2
    Real(wp)                    :: x,y,z
    Integer                     :: i,j
    Integer,Parameter           :: Ns=500
    Logical                     :: realimag
    

    !-------------------------------------------------
    !        Initialization of some parameters
    !-------------------------------------------------
    ! Open the output file
    Open(77,file='potreal.dat',status='unknown')
    Write(77,'(3a25)') 'x1','x2','potential'
    Open(78,file='potaimag.dat',status='unknown')
    Write(78,'(3a25)') 'x1','x2','potential'
    
    ! Some abreviations for the geometric parameters
    t0  = param%theta0_calc  
    p0 	= param%phi0_calc
    a	= param%a/param%R
    xi0 = param%xi0
    xi1	= param%tr*param%xi0
    tr	= param%tr
    AR	= param%R_par/param%R_per
    E0z	= cos(t0)
    E0x	= sin(t0)*cos(p0)
    E0y	= sin(t0)*sin(p0)	
    ! Dielectric constants
    e1 = param%eps_vacuum
    e2 = param%eps_substrate(ienergy)
    e3 = param%eps_island(ienergy)
    e4 = e2
    
    ! Type of surface of cut 
10  Write(6,'(a\)') '  Enter the plane of cut (x-y-z) : '
    Read(5,*,err=10) cut
    If(cut/='x'.and.cut/='y'.and.cut/='z') Then
       Write(6,*) ' Error in the choice of plane-cut ! '
       Goto 10
    Endif
20  Write(6,'(a\)') '  Enter the value of cut : '
    Read(5,*,err=20) value
    Select Case(cut)
    Case('x','y')
       If(Abs(value)>SG*AR) Then
	  Write(6,*) ' Error in the choice of cut-value (out of the grid) !'
	  Goto 20
       Endif
    Case('z')
       If(Abs(value)>SG) Then
	  Write(6,*) ' Error in the choice of cut-value (out of the grid) !'
	  Goto 20
       Endif
    End Select
30  Write(6,'(a\)') '  Real(T) or Imaginary(F) part : '
    Read(5,*,err=30) realimag
    
    !-------------------------------------------------
    ! Sets up the multipole expansion coefficients
    !-------------------------------------------------
    mpo = param%Mpole_order    
    Allocate( Am(mpo,0:1),  Bm(mpo,0:1)  ) 
    Allocate( Arm(mpo,0:1), Brm(mpo,0:1) )
    Allocate( Atm(mpo,0:1), Btm(mpo,0:1) )
    ! m=0,1
    Am(:,0:1)  =	MPC(1:mpo,0:1)
    Bm(:,0:1)  =	MPC(mpo+1:2*mpo,0:1)
    ! m=-1
    ! Am(:,-1)   = - Am(:,1) * exp(2*imu*p0)
    ! Bm(:,-1)   = - Bm(:,1) * exp(2*imu*p0)   
    ! Reflected multipole foefficients
    Do l=1,mpo
       Do m=0,1
          Arm(l,m) = (-1._wp)**(l+m) * (e1 - e2)/(e1 + e2) * Am(l,m)
          Brm(l,m) = (-1._wp)**(l+m) * (e3 - e4)/(e3 + e4) * Bm(l,m)
       Enddo
    Enddo
    ! transmitted multipole coefficients
    Atm  =  2*e1/(e1+e2) * Am
    Btm  =  2*e3/(e3+e4) * Bm


    !------------------------------------------------------
    !---  Calculates the constant b0
    !-----------------------------------------------------  
    Allocate(zeta(0:mpo))
    Do l=1,mpo
       zeta(l) = Sqrt(2*l+1._wp)/2
    Enddo
    ctmp  = (e1/e2-1._wp)*cos(t0) 
    b0    =    ctmp/sqrt(3._wp)*zeta(1)*int(0,1,0)%Q          &
                +   tr*ctmp * (1._wp-zeta(0)*int(0,0,0)%Q)
    Do l=1,mpo
       ctmp  = Am(l,0)*zeta(l)*xi0**(l+1)/sqrt(4*pi)*(e1-e2)/(e1+e2)
       ctmp  = ctmp                                                       &
            *( XZ(l,0)%Z*Int(0,l,0)%Q - (-1._wp)**l*Int(0,l,0)%V )
       b0 = b0 - ctmp     
       ctmp  = Bm(l,0)*zeta(l)*xi0**(-l)/sqrt(4*pi)*(e3-e4)/(e3+e4)
       ctmp  = ctmp                                                       &
            *( XZ(l,0)%X*Int(0,l,0)%Q - (-1._wp)**l*Int(0,l,0)%W )
       b0 = b0 + ctmp
    Enddo
    Deallocate(zeta)
    
    
    !-------------------------------------------------
    ! Sets up the grid and the spherical coordinates
    !-------------------------------------------------
    ! Allocation of the necessary space for the grid
    Allocate(Grid(Nx2,Nx1))
    Allocate(x1(Nx1),x2(Nx2))
    Allocate(xi_mp(Npoints),xi_imp(Npoints))
    Allocate(eta_mp(Npoints),eta_imp(Npoints))
    Allocate(phi(Npoints))
    
    ! Fills up the grid
    dx1 = 2*SG/(Nx1-1)
    dx2 = 2*SG/(Nx2-1)
    x1  = arth(-SG,dx1,Nx1)
    x2  = arth(-SG,dx2,Nx2)
    
    
    Select Case(cut)
       
    Case('x')
       !-----------------------------------
       !	Cut along the plane x=value
       !-----------------------------------
       x1(:) = AR*x1(:)
       x  = value
       Do i=1,Nx1
	  y = x1(i)
	  Do j=1,Nx2
             ipoint = (i-1)*Nx2+j
             z = x2(j)
             Call Sph_Coord_oblate(a,x,y,z,xi_mp(ipoint),eta_mp(ipoint),phi(ipoint))
	  Enddo
       Enddo
       x1	= -x1
       x2 = -x2
       
    Case('y')
       !-----------------------------------
       !	Cut along the plane y=value
       !-----------------------------------
       x1(:) = AR*x1(:)
       y = value
       Do i=1,Nx1
	  x = x1(i)
	  Do j=1,Nx2
             ipoint = (i-1)*Nx2+j
             z = x2(j)
             Call Sph_Coord_oblate(a,x,y,z,xi_mp(ipoint),eta_mp(ipoint),phi(ipoint))
	  Enddo
       Enddo
       x2 = -x2
       
    Case('z')
       !-----------------------------------
       !	Cut along the plane z=value
       !-----------------------------------
       x1(:) = AR*x1(:)
       x2(:) = AR*x2(:)
       z = value
       Do i=1,Nx1
	  x = x1(i)
	  Do j=1,Nx2
             ipoint = (i-1)*Nx2+j
             y = x2(j)
             Call Sph_Coord_oblate(a,x,y,z,xi_mp(ipoint),eta_mp(ipoint),phi(ipoint))
	  Enddo
       Enddo
       x2 = -x2
       
    Case Default
       Write(6,*) 'Error in the choice of plane-cut'
    End Select
    
    
    !-------------------------------------------------
    ! Transformed coordinates 
    !-------------------------------------------------
    Do i=1,Npoints
       Call xi_eta_trans_oblate(xi_mp(i),eta_mp(i),xi1,xi_imp(i),eta_imp(i))
    Enddo
    
    
    !------------------------------------------------------
    ! Calculates the prefactors for the spherical harmonics
    !------------------------------------------------------
    Allocate( prefactor(mpo,0:1) )
    Do l=1,mpo
       Do m=0,1
          ! the prefactor
          Select case(m)
          Case(1)
             tmp              =  1._wp/(l*(l+1))
          Case(0)
             tmp              =  1._wp                              
          End select
          prefactor(l,m)   =  sqrt( (2*l+1)/(4*pi) * tmp )
       Enddo
    Enddo
    
    
    
    !************************************************************
    !********************   POTENTIAL  **************************
    !************************************************************
    
    !------------------------------------------------------
    !---  Loops over points in the grid
    !------------------------------------------------------
    grid_loop : Do ipoint=1,Npoints
       
       v = 0._wp
       ! The phase factor for spherical harmonics
       phase = Exp(imu*phi(ipoint)) + Exp(imu*(2*p0-phi(ipoint)))
       
       
       !***********************************************
       !   Upper space 1-3
       !***********************************************
       Upper_Lower : if (eta_mp(ipoint)*xi_mp(ipoint) <= xi1) then
          
          
          
          !-----------------------------------------------------
          !   Space 1
          !-----------------------------------------------------
          Inside_Outside1 : if (xi_mp(ipoint) >= xi0) then		 
             
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------		               
             Do l=1,mpo
                ! Case m=0
                ! spherical harmonics
                Y_mp  = prefactor(l,0) * LegendreP_R(eta_mp(ipoint),l,0)
                Y_imp = prefactor(l,0) * LegendreP_R(eta_imp(ipoint),l,0)
                ! The radial dependencies
                Z_mp	= Z_Oblate(xi_mp(ipoint),l,0)
                Z_imp = Z_Oblate(xi_imp(ipoint),l,0)    
                ! Potentials                
                v = v + a**(-l-1) * &
                     ( Am(l,0)*Z_mp*Y_mp + Arm(l,0)*Z_imp*Y_imp )
                ! Case m=-1,+1
                ! spherical harmonics
                Y_mp  = prefactor(l,1) * LegendreP_R(eta_mp(ipoint),l,1) 
                Y_imp = prefactor(l,1) * LegendreP_R(eta_imp(ipoint),l,1)
                ! The radial dependencies
                Z_mp	 = Z_Oblate(xi_mp(ipoint),l,1)
                Z_imp  = Z_Oblate(xi_imp(ipoint),l,1) 
                ! Potentials                
                v = v - phase * a**(-l-1) * &
                     ( Am(l,1)*Z_mp*Y_mp + Arm(l,1)*Z_imp*Y_imp )
             Enddo
             
             ! Calculates the constant potential v0 of the incident field
             v0 = E0z * eta_mp(ipoint)*xi_mp(ipoint) + &
                  Sqrt( (xi_mp(ipoint)**2+1)*(1-eta_mp(ipoint)**2) ) * &
                  ( E0x*Cos(phi(ipoint)) + E0y*Sin(phi(ipoint)) )
             v0 = -a*v0
             
             
             ! Add the constant potential
             v = v +  v0
             
             
          else
             
             !-----------------------------------------------------
             !   Space 3
             !-----------------------------------------------------
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------		 			 			              
             Do l=1,mpo
                ! Case m=0
                ! spherical harmonics
                Y_mp  = prefactor(l,0) * LegendreP_R(eta_mp(ipoint),l,0)
                Y_imp = prefactor(l,0) * LegendreP_R(eta_imp(ipoint),l,0)
                ! The radial dependencies
                X_mp	 = X_Oblate(xi_mp(ipoint),l,0)
                X_imp   = X_Oblate(xi_imp(ipoint),l,0) 
                ! Potentials           
                v = v + a**l * &
                     ( Bm(l,0)*X_mp*Y_mp + Brm(l,0)*X_imp*Y_imp )
                ! Case m=-1,+1
                Y_mp  = prefactor(l,1) * LegendreP_R(eta_mp(ipoint),l,1)
                Y_imp = prefactor(l,1) * LegendreP_R(eta_imp(ipoint),l,1)
                ! The radial dependencies
                X_mp  = X_Oblate(xi_mp(ipoint),l,1)
                X_imp = X_Oblate(xi_imp(ipoint),l,1) 
                ! Potentials              
                v = v - phase * a**l * &
                     ( Bm(l,1)*X_mp*Y_mp + Brm(l,1)*X_imp*Y_imp )	   
             Enddo
             
             ! Add the constant potential
             v = v +  b0
             
          endif Inside_Outside1
          
          
          
          !***************************************************
          !   Lower space 2-4
          !***************************************************
       else
          
          !-----------------------------------------------------
          ! Space 2
          !-----------------------------------------------------
          Inside_Outside2 : if (xi_mp(ipoint) >= xi0) then		 
             
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             Do l=1,mpo
                ! Case m=0
                ! spherical harmonics 
                Y_mp  = prefactor(l,0) * LegendreP_R(eta_mp(ipoint),l,0) 
                ! The radial dependencies
                Z_mp	 = Z_Oblate(xi_mp(ipoint),l,0)  
                ! Potentials                
                v = v + a**(-l-1) * Atm(l,0)*Z_mp*Y_mp
                ! Case m=-1,+1
                ! spherical harmonics 
                Y_mp  = prefactor(l,1) * LegendreP_R(eta_mp(ipoint),l,1)
                ! The radial dependencies
                Z_mp  = Z_Oblate(xi_mp(ipoint),l,1)  
                ! Potentials                
                v = v - phase * a**(-l-1) * Atm(l,1)*Z_mp*Y_mp        
             Enddo
             
             ! Calculates the constant potetials
             v0 = (e1/e2)*E0z * eta_mp(ipoint)*xi_mp(ipoint) + &
                  Sqrt( (xi_mp(ipoint)**2+1)*(1-eta_mp(ipoint)**2) ) * &
                  ( E0x*Cos(phi(ipoint)) + E0y*Sin(phi(ipoint)) )
             v0 = -a*v0
             v0 = v0 - (1-e1/e2)*tr*E0z
             
             ! Add constant potential          
             v = v + v0 
             
             !-----------------------------------------------------
             ! Space 4
             !-----------------------------------------------------
          else	  
             
             !------------------------------------------------------
             !---  Loops over quantum numbers
             !------------------------------------------------------
             Do l=1,mpo
                ! Case m=0
                ! spherical harmonic
                Y_mp  = prefactor(l,0) * LegendreP_R(eta_mp(ipoint),l,0)
                ! The radial dependencies
                X_mp	 = X_Oblate(xi_mp(ipoint),l,0) 
                ! Potentials                   
                v = v + a**l * Btm(l,0)*X_mp*Y_mp
                ! Case m=-1,+1
                ! spherical harmonics
                Y_mp  = prefactor(l,1) * LegendreP_R(eta_mp(ipoint),l,1)
                ! The radial dependencies
                X_mp	 = X_Oblate(xi_mp(ipoint),l,1) 
                ! Potentials                   
                v = v - phase * a**l * Btm(l,1)*X_mp*Y_mp    
             Enddo
             
             ! Add constant potential          
             v = v +  b0
             
          endif Inside_Outside2
          
          !------------------------------------------------------
          !---  End Condition Upper-Lower
          !------------------------------------------------------
       endif Upper_Lower
       
       
       !------------------------------------------------------
       !---  Storage of the results
       !------------------------------------------------------
       i = (ipoint-1)/Nx2+1
       j = ipoint-(i-1)*Nx2
       ! Graphical output"
       If(realimag) Then
          Grid(Nx2+1-j,i)=real(v,4)
       Else
          Grid(Nx2+1-j,i)=real(aimag(v),4)
       Endif
       Write(77,'(3f25.5)') x1(i),x2(j),real(v,4)
       Write(78,'(3f25.5)') x1(i),x2(j),real(aimag(v),4)
       !	Write(77,'(3f25.5,3i6)') x1(i),x2(j),real(v,4),i,j,ipoint
       
       
       !------------------------------------------------------
       !---  End loops over points in the grid
       !------------------------------------------------------
    Enddo grid_loop
    
    !------------------------------------------------
    !		Contour plot of isolines
    !------------------------------------------------
    Call contour(Grid,title='Potential',flag=2)
    
    !------------------------------------------------
    !	Deallocation of space
    !------------------------------------------------
    Deallocate(Grid)
    Deallocate(x1,x2,prefactor,xi_mp,xi_imp,eta_mp,eta_imp,phi)
    Deallocate(Am,Bm,Atm,Btm,Arm,Brm)
    
    !-------------------------------------------------
    ! Set up the contour line for the cluster and
    ! the substrate surface
    !-------------------------------------------------
    Allocate(x1(Ns))
    Write(77,'(2a12)') 'xc','yc'
    Write(78,'(2a12)') 'xc','yc'
    
    Select Case(cut)
       !--------------------------------
       ! Contour along the plane x=value
       !--------------------------------
    Case('x')
       dx1 = 2*SG/(Ns-1)
       x1  = arth(-SG,dx1,Ns)
       x1(:) = AR*x1(:)
       Do i=1,Ns
	  Write(77,'(2f12.5)') x1(i),-param%tr
	  Write(78,'(2f12.5)') x1(i),-param%tr
       Enddo
       If(Abs(value)<=a*Sqrt(1+xi0**2)) Then
	  tmp = Sqrt(1-value**2/a**2/(xi0**2+1))
	  dx1 = (Min(tr,tmp)+tmp)/(Ns-1)
	  x1  = arth(-tmp,dx1,Ns)
	  ! Take the absolute value to prevent 0-
	  Do i=Ns,1,-1
             pos(1) = Sqrt( Abs(a**2*(xi0**2+1)*(1-x1(i)**2) - value**2) )
             pos(2) = a*xi0*x1(i)
             Write(77,'(2f12.5)') pos(1),-pos(2)
             Write(78,'(2f12.5)') pos(1),-pos(2)
	  Enddo
	  Do i=1,Ns
             pos(1) = Sqrt( Abs(a**2*(xi0**2+1)*(1-x1(i)**2) - value**2) )
             pos(2) = a*xi0*x1(i)
             Write(77,'(2f12.5)') -pos(1),-pos(2)
             Write(78,'(2f12.5)') -pos(1),-pos(2)
	  Enddo
       Endif
       !--------------------------------
       ! Contour along the plane y=value
       !--------------------------------	  
    Case('y')
       dx1 = 2*SG/(Ns-1)
       x1  = arth(-SG,dx1,Ns)
       x1(:) = AR*x1(:)
       Do i=1,Ns
	  Write(77,'(2f12.5)') x1(i),-param%tr
	  Write(78,'(2f12.5)') x1(i),-param%tr
       Enddo
       If(Abs(value)<=a*Sqrt(1+xi0**2)) Then
	  tmp = Sqrt(1-value**2/a**2/(xi0**2+1))
	  dx1 = (Min(tr,tmp)+tmp)/(Ns-1)
	  x1  = arth(-tmp,dx1,Ns)
	  Do i=Ns,1,-1
             pos(1) = Sqrt( Abs(a**2*(xi0**2+1)*(1-x1(i)**2) - value**2) )
             pos(2) = a*xi0*x1(i)
             Write(77,'(2f12.5)') pos(1), -pos(2)
             Write(78,'(2f12.5)') pos(1), -pos(2)
	  Enddo
	  Do i=1,Ns
             pos(1) = Sqrt( Abs(a**2*(xi0**2+1)*(1-x1(i)**2) - value**2) )
             pos(2) = a*xi0*x1(i)
             Write(77,'(2f12.5)') -pos(1), -pos(2)
             Write(78,'(2f12.5)') -pos(1), -pos(2)
	  Enddo
       Endif
       !--------------------------------
       ! Contour along the plane z=value
       !--------------------------------	  
    Case('z')
       If(value<=param%tr.and.value>=-1._wp) Then
	  dx1 = 2*pi/(Ns-1)
	  x1  = arth(0._wp,dx1,Ns)
	  Do i=1,Ns
	   tmp = a*Sqrt( (xi0**2+1)*(1-(value/a/xi0)**2) )
	   Write(77,'(2f12.5)') tmp*Cos(x1(i)),tmp*Sin(x1(i))
	   Write(78,'(2f12.5)') tmp*Cos(x1(i)),tmp*Sin(x1(i))
        Enddo
     Endif
     !----------------------------
     ! Default Case of cut 
     !----------------------------	
  Case Default
     Write(6,*) 'Error in the choice of plane-cut'
  End Select
  Deallocate(x1)
  
  Close(77,status='keep')
  Close(78,status='keep')
  
End Subroutine Potential_oblate_above

!-------------------------------------------------------------------------

Subroutine Sph_Coord_oblate(a,x,y,z,xi,eta,phi)
  
  Implicit None
  Real(wp)     :: a,x,y,z
  Real(wp)     :: xi,eta,phi
  Real(wp)     :: ro1,ro2
  
  ! Formulas (6.1) - (6.2) of Bedeaux's book
  phi = atan(abs(y/x))
  if(y>=0) then
     if(x>=0) then
        phi = phi
     else
        phi = pi - phi
     endif
  else
     if(x>=0) then
        phi = 2*pi - phi
     else
        phi = pi + phi
     endif
  endif
  ro1 = Sqrt( z**2+(x+a*cos(phi))**2+(y+a*sin(phi))**2 )
  ro2 = Sqrt( z**2+(x-a*cos(phi))**2+(y-a*sin(phi))**2 )
  ! Take the absolute value to get rid of round off errors
  xi  = Sqrt( Abs ( ((ro1+ro2)/(2*a))**2 - 1._wp ) )
  eta = Sqrt( Abs ( 1._wp - ((ro1-ro2)/(2*a))**2 ) )
  If(z<0) Then
     eta = -eta
  Endif
  
End Subroutine Sph_Coord_oblate


!=========================================================================
!=========================================================================
!=========================================================================
!=========================================================================
!=========================================================================

Subroutine Potential_prolate_above(MPC,Int,XZ,ienergy,param)
  
  !-----------------------------------------------------------
  ! Multipole located above the surface for an prolate spheroid
  !-----------------------------------------------------------
  
  Use Legendre_mod, only          : LegendreP_R
  Use Prolate_Int_Quadpack, only  : xi_eta_trans_prolate,X_Prolate,Z_Prolate
  
  Implicit none
  ! Global
  Type(parameters)             :: param 
  Integer                      :: ienergy
  Complex(wpc)                 :: MPC(:,0:)
  Type(Integrals)              :: Int(0:,0:,0:) 
  Type(XZ_type)	               :: XZ(0:,0:)
  
  ! Local
  ! The multipole coefficients
  Complex(wpc),Allocatable     :: Am(:,:),Bm(:,:),Arm(:,:),Brm(:,:),Atm(:,:),Btm(:,:)
  ! The spheroidal coordinates on the grid 
  Real(wp),Allocatable         :: xi_mp(:),eta_mp(:),phi(:)
  ! Spheroidal coordinates from the image point
  Real(wp),Allocatable         :: xi_imp(:),eta_imp(:)
  ! Some abreviations
  Real(wp)                     :: a,tr,doa,xi0,AR
  ! The spherical harmonics
  Real(wp)                     :: Y_mp,Y_imp
  Real(wp)                     :: Z_mp,Z_imp
  Real(wp)                     :: X_mp,X_imp
  ! For the constant b0
  Complex(wpc)                 :: b0
  ! Prefactor of the sph erical harmonics                             
  Real(wp),Allocatable         :: prefactor(:,:)
  Real(wp),Allocatable         :: zeta(:)
  ! Various parameters
  Integer                      :: mpo,l,m,ipoint
  Real(wp)                     :: tmp,pos(2)
  Real(wp)                     :: t0,p0
  Real(wp)                     :: E0x,E0y,E0z
  Complex(wpc)                 :: e1,e2,e3,e4,v0,v
  Complex(wpc)                 :: phase,ctmp
  ! Parameters for the grid
  Real(sp),Allocatable         :: Grid(:,:)
  Real(wp),Allocatable         :: x1(:),x2(:)
  Integer,Parameter            :: Nx1=50
  Integer,Parameter            :: Nx2=50
  Integer,Parameter            :: Npoints=Nx1*Nx2
  Character(len=1)             :: cut
  Real(wp)                     :: value
  Real(wp), parameter          :: SG=2._wp
  Real(wp)                     :: dx1,dx2
  Real(wp)                     :: x,y,z
  Integer                      :: i,j
  Integer,Parameter            :: Ns=500
  Logical                      :: realimag

  !-------------------------------------------------
  !        Initialization of some parameters
  !-------------------------------------------------
  ! Open the output file
  Open(77,file='potreal.dat',status='unknown')
  Write(77,'(3a25)') 'x1','x2','potential'
  Open(78,file='potaimag.dat',status='unknown')
  Write(78,'(3a25)') 'x1','x2','potential'
  
  ! Some abreviations for the geometric parameters
  t0  = param%theta0_calc  
  p0 	= param%phi0_calc
  a	= param%a/param%R
  xi0   = param%xi0
  tr	= param%tr
  doa	= Abs(param%div_surface)/param%a
  AR	= param%R_per/param%R_par
  E0z	= cos(t0)
  E0x	= sin(t0)*cos(p0)
  E0y	= sin(t0)*sin(p0)	
  ! Dielectric constants
  e1 = param%eps_vacuum
  e2 = param%eps_substrate(ienergy)
  e3 = param%eps_island(ienergy)
  e4 = e2
  
  ! Type of surface of cut 
10 Write(6,'(a\)') '  Enter the plane of cut (x-y-z) : '
  Read(5,*,err=10) cut
  If(cut/='x'.and.cut/='y'.and.cut/='z') Then
     Write(6,*) ' Error in the choice of plane-cut ! '
     Goto 10
  Endif
20 Write(6,'(a\)') '  Enter the value of cut : '
  Read(5,*,err=20) value
  Select Case(cut)
  Case('x','y')
     If(Abs(value)>SG) Then
        Write(6,*) ' Error in the choice of cut-value (out of the grid) !'
        Goto 20
     Endif
  Case('z')
     If(Abs(value)>SG*AR) Then
        Write(6,*) ' Error in the choice of cut-value (out of the grid) !'
        Goto 20
     Endif
  End Select
30 Write(6,'(a\)') '  Real(T) or Imaginary(F) part : '
  Read(5,*,err=30) realimag
  
  !-------------------------------------------------
  ! Sets up the multipole expansion coefficients
  !-------------------------------------------------
  mpo = param%Mpole_order    
  Allocate( Am(mpo,0:1),  Bm(mpo,0:1)  ) 
  Allocate( Arm(mpo,0:1), Brm(mpo,0:1) )
  Allocate( Atm(mpo,0:1), Btm(mpo,0:1) )
  ! m=0,1
  Am(:,0:1)  =	MPC(1:mpo,0:1)
  Bm(:,0:1)  =	MPC(mpo+1:2*mpo,0:1)
  ! m=-1
  ! Am(:,-1)   = - Am(:,1) * exp(2*imu*p0)
  ! Bm(:,-1)   = - Bm(:,1) * exp(2*imu*p0)   
  ! Reflected multipole foefficients
  Do l=1,mpo
     Do m=0,1
        Arm(l,m) = (-1._wp)**(l+m) * (e1 - e2)/(e1 + e2) * Am(l,m)
        Brm(l,m) = (-1._wp)**(l+m) * (e3 - e4)/(e3 + e4) * Bm(l,m)
     Enddo
  Enddo
  ! transmitted multipole coefficients
  Atm  =  2*e1/(e1+e2) * Am
  Btm  =  2*e3/(e3+e4) * Bm
  
  
  !------------------------------------------------------
  !---  Calculates the constant b0
  !-----------------------------------------------------  
  Allocate(zeta(0:mpo))
  Do l=1,mpo
     zeta(l) = Sqrt(2*l+1._wp)/2
  Enddo
  ctmp  = (e1/e2-1._wp)*cos(t0) 
  b0  =    ctmp/sqrt(3._wp)*zeta(1)*int(0,1,0)%Q          &
       +   tr*ctmp * (1._wp-zeta(0)*int(0,0,0)%Q)
  Do l=1,mpo
     ctmp  = Am(l,0)*zeta(l)*xi0**(l+1)/sqrt(4*pi)*(e1-e2)/(e1+e2)
     ctmp  = ctmp                       &
          *( XZ(l,0)%Z*Int(0,l,0)%Q - (-1._wp)**l*Int(0,l,0)%V )
     b0 = b0 - ctmp     
     ctmp  = Bm(l,0)*zeta(l)*xi0**(-l)/sqrt(4*pi)*(e3-e4)/(e3+e4)
     ctmp  = ctmp                                                       &
          *( XZ(l,0)%X*Int(0,l,0)%Q - (-1._wp)**l*Int(0,l,0)%W )
     b0 = b0 + ctmp
  Enddo
  Deallocate(zeta)
  
  
  !-------------------------------------------------
  ! Sets up the grid and the spherical coordinates
  !-------------------------------------------------
  ! Allocation of the necessary space for the grid
  Allocate(Grid(Nx2,Nx1))
  Allocate(x1(Nx1),x2(Nx2))
  Allocate(xi_mp(Npoints),xi_imp(Npoints))
  Allocate(eta_mp(Npoints),eta_imp(Npoints))
  Allocate(phi(Npoints))
  
  ! Fills up the grid
  dx1 = 2*SG/(Nx1-1)
  dx2 = 2*SG/(Nx2-1)
  x1  = arth(-SG,dx1,Nx1)
  x2  = arth(-SG,dx2,Nx2)
  
  
  Select Case(cut)
     
  Case('x')
     !-----------------------------------
     !	Cut along the plane x=value
     !-----------------------------------
     !	 x2(:) = x2(:)*AR
     x1(:) = x1(:)/AR
     x  = value
     Do i=1,Nx1
        y = x1(i)
        Do j=1,Nx2
	   ipoint = (i-1)*Nx2+j
	   z = x2(j)
	   Call Sph_Coord_prolate(a,x,y,z,xi_mp(ipoint),eta_mp(ipoint),phi(ipoint))
        Enddo
     Enddo
     x2 = -x2
     
  Case('y')
     !-----------------------------------
     !	Cut along the plane y=value
     !-----------------------------------
     !	 x2(:) = x2(:)*AR
     x1(:) = x1(:)/AR
     y = value
     Do i=1,Nx1
        x = x1(i)
        Do j=1,Nx2
	   ipoint = (i-1)*Nx2+j
	   z = x2(j)
	   Call Sph_Coord_prolate(a,x,y,z,xi_mp(ipoint),eta_mp(ipoint),phi(ipoint))
        Enddo
     Enddo
     x2 = -x2
     
  Case('z')
     !-----------------------------------
     !	Cut along the plane z=value
     !-----------------------------------
     !	 x1(:) = x1(:)*AR
     !	 x2(:) = x2(:)*AR
     x1(:) = x1(:)/AR
     x2(:) = x2(:)/AR
     z = value
     Do i=1,Nx1
        x = x1(i)
        Do j=1,Nx2
	   ipoint = (i-1)*Nx2+j
	   y = x2(j)
	   Call Sph_Coord_prolate(a,x,y,z,xi_mp(ipoint),eta_mp(ipoint),phi(ipoint))
        Enddo
     Enddo
     
  Case Default
     Write(6,*) 'Error in the choice of plane-cut'
  End Select
  
  
  !-------------------------------------------------
  ! Transformed coordinates 
  !-------------------------------------------------
  Do i=1,Npoints
     Call xi_eta_trans_prolate(xi_mp(i),eta_mp(i),doa,xi_imp(i),eta_imp(i))
  Enddo
  
  
  !------------------------------------------------------
  ! Calculates the prefactors for the spherical harmonics
  !------------------------------------------------------
  Allocate( prefactor(mpo,0:1) )
  Do l=1,mpo
     Do m=0,1
        ! the prefactor
        Select case(m)
        Case(1)
           tmp              =  1._wp/(l*(l+1))
        Case(0)
           tmp              =  1._wp                              
        End select
        prefactor(l,m)   =  sqrt( (2*l+1)/(4*pi) * tmp )
     Enddo
  Enddo
  
  
  
  !************************************************************
  !********************   POTENTIAL  **************************
  !************************************************************
  
  !------------------------------------------------------
  !---  Loops over points in the grid
  !------------------------------------------------------
  grid_loop : Do ipoint=1,Npoints
     
     v = 0._wp
     ! The phase factor for spherical harmonics
     phase = Exp(imu*phi(ipoint)) + Exp(imu*(2*p0-phi(ipoint)))
     
     
     !***********************************************
     !	 Upper space 1-3
     !***********************************************
     Upper_Lower : if (eta_mp(ipoint)*xi_mp(ipoint) <= doa) then
        
        
        
        !-----------------------------------------------------
        !   Space 1
        !-----------------------------------------------------
        Inside_Outside1 : if (xi_mp(ipoint) >= xi0) then		 


           !------------------------------------------------------
           !---  Loops over quantum numbers
           !------------------------------------------------------     
           Do l=1,mpo
              ! Case m=0
              ! spherical harmonics
              Y_mp  = prefactor(l,0) * LegendreP_R(eta_mp(ipoint),l,0)
              Y_imp = prefactor(l,0) * LegendreP_R(eta_imp(ipoint),l,0)
              ! The radial dependencies
              Z_mp  = Z_Prolate(xi_mp(ipoint),l,0)
              Z_imp = Z_Prolate(xi_imp(ipoint),l,0)    
              ! Potentials                
              v = v + a**(-l-1) * &
                   ( Am(l,0)*Z_mp*Y_mp + Arm(l,0)*Z_imp*Y_imp )
              ! Case m=-1,+1
              ! spherical harmonics
              Y_mp  = prefactor(l,1) * LegendreP_R(eta_mp(ipoint),l,1) 
              Y_imp = prefactor(l,1) * LegendreP_R(eta_imp(ipoint),l,1)
              ! The radial dependencies
              Z_mp  = Z_Prolate(xi_mp(ipoint),l,1)
              Z_imp = Z_Prolate(xi_imp(ipoint),l,1) 
              ! Potentials                
              v = v - phase * a**(-l-1) * &
                   ( Am(l,1)*Z_mp*Y_mp + Arm(l,1)*Z_imp*Y_imp )
           Enddo
           
           ! Calculates the constant potential v0 of the incident field
           v0 = E0z * eta_mp(ipoint)*xi_mp(ipoint) + &
                Sqrt( (xi_mp(ipoint)**2-1)*(1-eta_mp(ipoint)**2) ) * &
                ( E0x*Cos(phi(ipoint)) + E0y*Sin(phi(ipoint)) )
           v0 = -a*v0
           
           
           ! Add the constant potential
           v = v +  v0
           
           
        else
           
           !-----------------------------------------------------
           !						Space 3
           !-----------------------------------------------------
           
           
           !------------------------------------------------------
           !---  Loops over quantum numbers
           !------------------------------------------------------			              
           Do l=1,mpo
              ! Case m=0
              ! spherical harmonics
              Y_mp  = prefactor(l,0) * LegendreP_R(eta_mp(ipoint),l,0)
              Y_imp = prefactor(l,0) * LegendreP_R(eta_imp(ipoint),l,0)
              ! The radial dependencies
              X_mp  = X_Prolate(xi_mp(ipoint),l,0)
              X_imp = X_Prolate(xi_imp(ipoint),l,0) 
              ! Potentials                
              v = v + a**l * &
                   ( Bm(l,0)*X_mp*Y_mp + Brm(l,0)*X_imp*Y_imp )
              ! Case m=-1,+1
              Y_mp  = prefactor(l,1) * LegendreP_R(eta_mp(ipoint),l,1)
              Y_imp = prefactor(l,1) * LegendreP_R(eta_imp(ipoint),l,1)
              ! The radial dependencies
              X_mp  = X_Prolate(xi_mp(ipoint),l,1)
              X_imp = X_Prolate(xi_imp(ipoint),l,1) 
              ! Potentials                
              v = v - phase * a**l * &
                   ( Bm(l,1)*X_mp*Y_mp + Brm(l,1)*X_imp*Y_imp )  
           Enddo
           
           ! Add the constant potential
           v = v +  b0
           
        endif Inside_Outside1
        
        
        
	!***************************************************
	!   Lower space 2-4
	!***************************************************
     else
        
        !-----------------------------------------------------
        ! Space 2
        !-----------------------------------------------------
        Inside_Outside2 : if (xi_mp(ipoint) >= xi0) then		 
           
           
           !------------------------------------------------------
           !---  Loops over quantum numbers
           !------------------------------------------------------
           Do l=1,mpo
              ! Case m=0
              ! spherical harmonics 
              Y_mp = prefactor(l,0) * LegendreP_R(eta_mp(ipoint),l,0) 
              ! The radial dependencies
              Z_mp = Z_Prolate(xi_mp(ipoint),l,0)  
              ! Potentials                
              v = v + a**(-l-1) * Atm(l,0)*Z_mp*Y_mp
              ! Case m=-1,+1
              ! spherical harmonics 
              Y_mp  = prefactor(l,1) * LegendreP_R(eta_mp(ipoint),l,1)
              ! The radial dependencies
              Z_mp = Z_Prolate(xi_mp(ipoint),l,1)  
              ! Potentials                
              v = v - phase * a**(-l-1) * Atm(l,1)*Z_mp*Y_mp        
           Enddo
           
           ! Calculates the constant potetials
           v0 = (e1/e2)*E0z * eta_mp(ipoint)*xi_mp(ipoint) + &
                Sqrt( (xi_mp(ipoint)**2-1)*(1-eta_mp(ipoint)**2) ) * &
                ( E0x*Cos(phi(ipoint)) + E0y*Sin(phi(ipoint)) )
           v0 = -a*v0
           v0 = v0 - (1-e1/e2)*tr*E0z
           
           ! Add constant potential          
           v = v + v0 
           
           !-----------------------------------------------------
           ! Space 4
           !-----------------------------------------------------
        else	 
           
           
           !------------------------------------------------------
           !---  Loops over quantum numbers
           !------------------------------------------------------
           Do l=1,mpo
              ! Case m=0
              ! spherical harmonic
              Y_mp  = prefactor(l,0) * LegendreP_R(eta_mp(ipoint),l,0)
              ! The radial dependencies
              X_mp  = X_Prolate(xi_mp(ipoint),l,0) 
              ! Potentials                   
              v = v + a**l * Btm(l,0)*X_mp*Y_mp
              ! Case m=-1,+1
              ! spherical harmonics
              Y_mp  = prefactor(l,1) * LegendreP_R(eta_mp(ipoint),l,1)
              ! The radial dependencies
              X_mp  = X_Prolate(xi_mp(ipoint),l,1) 
              ! Potentials                   
              v = v - phase * a**l * Btm(l,1)*X_mp*Y_mp    
           Enddo
           
           ! Add constant potential          
           v = v +  b0
           
        endif Inside_Outside2
        
        !------------------------------------------------------
        !---  End Condition Upper-Lower
        !------------------------------------------------------
     endif Upper_Lower

     
     !------------------------------------------------------
     !---  Storage of the results
     !------------------------------------------------------
     i = (ipoint-1)/Nx2+1
     j = ipoint-(i-1)*Nx2
     Write(77,'(3f25.5)') x1(i),x2(j),real(v,4)
     Write(78,'(3f25.5)') x1(i),x2(j),real(aimag(v),4)
     !	Write(77,'(3f25.5,3i6)') x1(i),x2(j),real(v,4),i,j,ipoint
     If(realimag) Then
        Grid(Nx2+1-j,i)=real(v,4)
     Else
        Grid(Nx2+1-j,i)=real(aimag(v),4)
     Endif
     
     !------------------------------------------------------
     !---  End loops over points in the grid
     !------------------------------------------------------
  Enddo grid_loop
  
  !------------------------------------------------
  !		Contour plot of isolines
  !------------------------------------------------
  Call contour(Grid,title='Potential',flag=2)
  
  !------------------------------------------------
  !	Deallocation of space
  !------------------------------------------------
  Deallocate(Grid)
  Deallocate(x1,x2,prefactor,xi_mp,xi_imp,eta_mp,eta_imp,phi)
  Deallocate(Am,Bm,Atm,Btm,Arm,Brm)
  
  !-------------------------------------------------
  ! Set up the contour line for the cluster and
  ! the substrate surface
  !-------------------------------------------------
  Allocate(x1(Ns))
  Write(77,'(2a12)') 'xc','yc'
  Write(78,'(2a12)') 'xc','yc'
  Select Case(cut)
     !--------------------------------
     ! Contour along the plane x=value
     !--------------------------------
  Case('x')
     dx1 = 2*SG/(Ns-1)/AR
     x1  = arth(-SG/AR,dx1,Ns)
     Do i=1,Ns
        Write(77,'(2f12.5)') x1(i),-param%tr
        Write(78,'(2f12.5)') x1(i),-param%tr
     Enddo
     If(Abs(value)<=a*Sqrt(xi0**2-1)) Then
        tmp = Sqrt(1-value**2/a**2/(xi0**2-1))
        dx1 = (Min(tr,tmp)+tmp)/(Ns-1)
        x1  = arth(-tmp,dx1,Ns)
        Do i=Ns,1,-1
	   pos(1) = Sqrt( Abs(a**2*(xi0**2-1)*(1-x1(i)**2) - value**2) )
	   pos(2) = a*xi0*x1(i)
	   Write(77,'(2f12.5)') pos(1),-pos(2)
	   Write(78,'(2f12.5)') pos(1),-pos(2)
        Enddo
        Do i=1,Ns
	   pos(1) = Sqrt( Abs(a**2*(xi0**2-1)*(1-x1(i)**2) - value**2) )
	   pos(2) = a*xi0*x1(i)
	   Write(77,'(2f12.5)') -pos(1),-pos(2)
	   Write(78,'(2f12.5)') -pos(1),-pos(2)
        Enddo
     Endif
     !--------------------------------
     ! Contour along the plane y=value
     !--------------------------------	  
  Case('y')
     dx1 = 2*SG/(Ns-1)/AR
     x1  = arth(-SG/AR,dx1,Ns)
     Do i=1,Ns
        Write(77,'(2f12.5)') x1(i),-param%tr
        Write(78,'(2f12.5)') x1(i),-param%tr
     Enddo
     If(Abs(value)<=a*Sqrt(xi0**2-1)) Then
        tmp = Sqrt(1-value**2/a**2/(xi0**2-1))
        dx1 = (Min(tr,tmp)+tmp)/(Ns-1)
        x1  = arth(-tmp,dx1,Ns)
        Do i=Ns,1,-1
	   pos(1) = Sqrt( Abs(a**2*(xi0**2-1)*(1-x1(i)**2) - value**2) )
	   pos(2) = a*xi0*x1(i)
	   Write(77,'(2f12.5)') pos(1), -pos(2)
	   Write(78,'(2f12.5)') pos(1), -pos(2)
        Enddo
        Do i=1,Ns
	   pos(1) = Sqrt( Abs(a**2*(xi0**2-1)*(1-x1(i)**2) - value**2) )
	   pos(2) = a*xi0*x1(i)
	   Write(77,'(2f12.5)') -pos(1), -pos(2)
	   Write(78,'(2f12.5)') -pos(1), -pos(2)
        Enddo
     Endif
	!--------------------------------
     ! Contour along the plane z=value
     !--------------------------------	  
  Case('z')
     If(value<=param%tr.and.value>=-1._wp) Then
        dx1 = 2*pi/(Ns-1)
        x1  = arth(0._wp,dx1,Ns)
        Do i=1,Ns
	   tmp = a*Sqrt( (xi0**2-1)*(1-(value/a/xi0)**2) )
	   Write(77,'(2f12.5)') tmp*Cos(x1(i)),tmp*Sin(x1(i))
	   Write(78,'(2f12.5)') tmp*Cos(x1(i)),tmp*Sin(x1(i))
        Enddo
     Endif
     !----------------------------
     ! Default Case of cut 
     !----------------------------	
  Case Default
     Write(6,*) 'Error in the choice of plane-cut'
  End Select
  Deallocate(x1)
  
  Close(77,status='keep')
  Close(78,status='keep')
  
End Subroutine Potential_prolate_above

!-------------------------------------------------------------------------
Subroutine Sph_Coord_prolate(a,x,y,z,xi,eta,phi)
  
  Implicit None
  Real(wp)   :: a,x,y,z
  Real(wp)   :: xi,eta,phi
  Real(wp)   :: ro1,ro2
  
  ! Formulas (6.1) - (6.2) of Bedeaux's book
  phi = atan(abs(y/x))
  if(y>=0) then
     if(x>=0) then
        phi = phi
     else
        phi = pi - phi
     endif
  else
     if(x>=0) then
        phi = 2*pi - phi
     else
        phi = pi + phi
     endif
  endif
  ro1 = Sqrt( (z+a)**2+x**2+y**2 )
  ro2 = Sqrt( (z-a)**2+x**2+y**2 )
  xi  = (ro1+ro2)/(2*a) 
  eta = (ro1-ro2)/(2*a)
  
End Subroutine Sph_Coord_prolate

!-------------------------------------------------------------------

Function arth(first,increment,n)
  
  Implicit None
  Real(wp)                  ::  first,increment
  Integer                   ::  n,i
  Real(wp),Dimension(n)     ::  arth
  
  arth(1) = first
  Do i=2,n
     arth(i) = arth(i-1)+increment
  Enddo
  Return
  
End Function arth


End Module Potential_mod




