
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: prolate_int_gauleg.f90,v 1.1 2001/07/29 12:45:37 ingves Exp ingves $
!---------------------------------------------------------------------------------



Module Prolate_Int_Gauleg

  Use Global_definitions
  Use Legendre_mod, only	: gausslegendre,LegendreP
  
Contains
  
  !-----------------------------------------------------------------------------
  
  Subroutine Get_integrals_prolate_Gauleg(Int,param)
    
    Implicit none
    ! Input
    Type(parameters)            :: param
    Type(Integrals)             :: Int(0:,0:,0:)   
    ! Local
    Integer                     ::  l1,l2,m,mpo,Nint
    Real(wp)                    ::  tr,doa,xi0
    Real(wp),Allocatable        ::  eta(:),w(:)
    Real(wp),Allocatable        ::  integrand(:)
    Real(wp),Allocatable        ::  P(:,:,:),dPdx(:,:,:)
    Real(wp),Allocatable        ::  P_trans(:,:,:),dPdx_trans(:,:,:)
    Real(wp),Allocatable        ::  xi_trans(:),eta_trans(:)
    Real(wp),Allocatable        ::  deriv_xi_trans(:),deriv_eta_trans(:)
    Type(XZ_type),Allocatable   ::  XZ_trans(:,:,:)
    

    ! Some abbreviations or common statements
    mpo	    =   param%Mpole_order
    Nint    =   param%Nint
    tr	    =   param%tr
    doa     =   Abs(param%div_surface)/param%a
    xi0     =   param%xi0
    
    
    ! Initialization
    Allocate(eta(Nint),w(Nint),integrand(Nint))
    Allocate(P(Nint,0:mpo,0:1),dPdx(Nint,0:mpo,0:1))
    Allocate(P_trans(Nint,0:mpo,0:1),dPdx_trans(Nint,0:mpo,0:1))
    Allocate(xi_trans(Nint),eta_trans(Nint))
    Allocate(deriv_xi_trans(Nint),deriv_eta_trans(Nint))
    Allocate(XZ_trans(Nint,0:mpo,0:1))
    
    
    !-----------------------------------------------------------------
    !---  Gets the weights and abscissa for Gauss-Legendre integration
    !-----------------------------------------------------------------   
    Call GaussLegendre(-1._wp,tr,eta,w)

    !-----------------------------------------------------------------
    !---  Gets the Transformed Coordinates
    !-----------------------------------------------------------------
    Call xi_eta_trans_prolate(xi0,eta,doa,xi_trans,eta_trans)
    Call deriv_xi_eta_trans_prolate(xi0,eta,doa,deriv_xi_trans,deriv_eta_trans)	
    
    !-----------------------------------------------------------------
    !---  Gets the XZ
    !-----------------------------------------------------------------
    Call XZ_prolate(XZ_trans,xi_trans)
    
    !-----------------------------------------------------------------
    !---  Gets the Legendre Functions and Derivates
    !-----------------------------------------------------------------
    Call  LegendreP(P,dPdx,eta,mpo)
    Call  LegendreP(P_trans,dPdx_trans,eta_trans,mpo)
    
    !----------------------------------------------
    !---  Calculates the Integrals
    !----------------------------------------------    
    ! Loops over all "quantum numbers"
    Do l1=0,mpo
       Do l2=0,mpo
          Do m=0,1	
             
             !------------------------------------------------
             ! --- Q-integrals 
             !------------------------------------------------
             integrand(:)       =   P(:,l1,m)*P(:,l2,m)
             Int(l1,l2,m)%Q	=   Sum(w*integrand)
             
             !------------------------------------------------
             ! --- V-integrals 
             !------------------------------------------------         
             integrand(:)       =   P(:,l1,m)*P_trans(:,l2,m)*XZ_trans(:,l2,m)%Z
             Int(l1,l2,m)%V     =   Sum(w*integrand)
             
             !------------------------------------------------
             ! --- W-integrals 
             !------------------------------------------------
             integrand(:)       =   P(:,l1,m)*P_trans(:,l2,m)*XZ_trans(:,l2,m)%X
             Int(l1,l2,m)%W     =   Sum(w*integrand)	
             
             !------------------------------------------------
             ! --- dV-integrals 
             !------------------------------------------------
             integrand(:)       =   P(:,l1,m)* &
                  ( deriv_eta_trans(:)*dPdx_trans(:,l2,m)*XZ_trans(:,l2,m)%Z + &
                  deriv_xi_trans(:)*P_trans(:,l2,m)*XZ_trans(:,l2,m)%dZdxi )
             Int(l1,l2,m)%dVdx  =   Sum(w*integrand)
             
             !------------------------------------------------
             ! --- DW-integrals 
             !------------------------------------------------
             integrand(:)        =  P(:,l1,m)* &
                  ( deriv_eta_trans(:)*dPdx_trans(:,l2,m)*XZ_trans(:,l2,m)%X + &
                  deriv_xi_trans(:)*P_trans(:,l2,m)*XZ_trans(:,l2,m)%dXdxi )
             Int(l1,l2,m)%dWdx  =   Sum(w*integrand)
             
          Enddo
       Enddo
    Enddo
    
    Deallocate(eta,w,integrand)
    Deallocate(P,dPdx)
    Deallocate(P_trans,dPdx_trans)
    Deallocate(xi_trans,eta_trans)
    Deallocate(deriv_xi_trans,deriv_eta_trans)
    Deallocate(XZ_trans)
    
  End Subroutine Get_integrals_prolate_Gauleg
  


  ! ----------------------------------------------------------
  ! --- Routines for the transformed coordinates
  ! ----------------------------------------------------------
  
  Subroutine xi_eta_trans_prolate(xi,eta,doa,xi_trans,eta_trans)
    
    ! Transformed coordinates of the image point for the oblate case
    Implicit None
    ! Input
    Real(wp)      ::  xi,eta(:)
    Real(wp)      ::   doa
    ! Output
    Real(wp)      ::   xi_trans(size(eta)),eta_trans(size(eta))
    ! Local
    Real(wp)      ::   R,tmp
    Integer       ::   i
    
    R   =  doa/xi
    Do i=1,Size(eta,1)
       tmp          =  1._wp+4*R**2-4*R*eta(i)+(eta(i)/xi)**2
       tmp          =  tmp + sqrt( tmp**2 - (2*(2*R-eta(i))/xi)**2 )
       tmp          =  Sqrt(tmp)
       xi_trans(i)  =  xi*tmp/Sqrt(2._wp)
       eta_trans(i) =  Sqrt(2._wp)*(eta(i)-2*R)/tmp
    Enddo
    
  End Subroutine xi_eta_trans_prolate
  
  
  !-----------------------------------------------------------------
  Subroutine deriv_xi_eta_trans_prolate(xi,eta,doa,deriv_xi_trans,deriv_eta_trans)
    
    Implicit None
    ! Input
    Real(wp)     ::  xi,eta(:)
    Real(wp)     ::  doa
    ! Output
    Real(wp)     ::  deriv_xi_trans(size(eta)),deriv_eta_trans(size(eta))
    ! Local
    Real(wp)     ::  tmp(10),R
    Integer      ::  i
    
    R		=	doa/xi
    Do i=1,Size(eta,1)
       tmp(1)   =  1._wp+4*R**2-4*R*eta(i)+(eta(i)/xi)**2
       tmp(2)   =  2*(2*R-eta(i))/xi
       tmp(3)   =  -8*R**3/doa+4*R**2*eta(i)/doa-2*eta(i)**2/(xi**3)	
       tmp(4)   =  -8*doa/(xi**3)+2*eta(i)/(xi**2)
       tmp(5)   =  Sqrt(tmp(1)**2-tmp(2)**2)
       tmp(6)   =  tmp(1)*tmp(3)-tmp(2)*tmp(4)
       tmp(7)   =  Sqrt(tmp(1)+tmp(5))
       tmp(8)   =  (tmp(3)+tmp(6)/tmp(5))/(2*tmp(7))
       tmp(9)   =  tmp(7)**3
       tmp(10)  =  (tmp(3)+tmp(6)/tmp(5))/(2*tmp(9))
       
       deriv_xi_trans(i)   =  tmp(7)+xi*tmp(8)
       deriv_xi_trans(i)   =  deriv_xi_trans(i)/Sqrt(2._wp)
       
       deriv_eta_trans(i)  =  2*doa/(xi**2)/tmp(7)-(eta(i)-2*R)*tmp(10)
       deriv_eta_trans(i)  =  deriv_eta_trans(i)*Sqrt(2._wp)
    Enddo
    
  End Subroutine deriv_xi_eta_trans_prolate
  
  
  ! ----------------------------------------------------------
  ! --- Routines for the XZ functions
  ! ----------------------------------------------------------
  
  Subroutine XZ_prolate(XZ,xi)
    
    ! Calculates the functions X, Z and their derivatives.
    
    Implicit none
    ! Input variables
    Real(wp)                     ::  xi(:)
    Type(XZ_type)                ::  XZ(:,0:,0:)
    ! Local variables
    Integer                      ::  npts,mpo,l,m
    Complex(wpc),Allocatable     ::  P(:,:,:)
    Complex(wpc),Allocatable     ::  dPdx(:,:,:)
    Complex(wpc),Allocatable     ::  Q(:,:,:)
    Complex(wpc),Allocatable     ::  dQdx(:,:,:)
    Complex(wpc)                 ::  xfactor,zfactor
    
    ! Some abbreviations
    npts  =  size(xi,1)
    mpo	  =  size(XZ,2)-1
    
    ! Allocation of space
    Allocate(P(npts,0:mpo,0:1),dPdx(npts,0:mpo,0:1))
    Allocate(Q(npts,0:mpo,0:1),dQdx(npts,0:mpo,0:1))
    
    ! Calculates the Legendre Functions
    call LegendreP_real(P,dPdx,xi,mpo)
    call LegendreQ_real(Q,dQdx,xi,mpo)
    
    ! Loop over quantum numbers
    Do m=0,1      
       Do l=0,mpo
          xfactor = (imu)**m*Xfactor_func(l,m) 
          zfactor = Zfactor_func(l,m)   
          ! The X function
          XZ(:,l,m)%X      = Real(xfactor*P(:,l,m))
          XZ(:,l,m)%dXdxi  = Real(xfactor*dPdx(:,l,m))          
          ! The Z function
          XZ(:,l,m)%Z      = Real(zfactor*Q(:,l,m))
          XZ(:,l,m)%dZdxi  = Real(zfactor*dQdx(:,l,m))               
       Enddo
    Enddo
    
    ! Free space
    Deallocate(P,dPdx,Q,dQdx)
    
  End Subroutine XZ_prolate
  
  !-------------------------------------------------------------------------

  Function XFactor_Func(l,m)	Result(factor)
    ! Factorial prefactor of X function
    
    Implicit None
    Integer     ::  l,m
    Integer     ::  i
    Real(wp)    ::  factor
    
    factor   = 1._wp
    If(m==1.and.l/=0) Then
       factor  =  factor/l
    Endif
    Do i=1,l
       factor =	factor*i/(2*i-1._wp)
    Enddo
    
  End Function XFactor_Func
  
  !-------------------------------------------------------------------------

  Function ZFactor_Func(l,m)	Result(factor)
    ! Factorial prefactor of Z function
    
    Implicit None
    Integer       ::  l,m
    Integer       ::  i
    Real(wp)      ::  factor
    
    factor	= 1._wp
    If(m==1) Then
       factor=factor/(l+1)
    Endif
    Do i=1,l
       factor =	factor*(2*i+1._wp)/i
    Enddo
    
  End Function ZFactor_Func
  
  
  ! ----------------------------------------------------------
  ! Legendre Functions of First Kind For Real Argument 1<x<inf
  ! ----------------------------------------------------------
  
  Subroutine LegendreP_Real(P,dPdx,x,lmax)
    ! Calculates the Associated Legendre function of the 1st kind
    ! INPUT  :    x         : the argument 1<x<inf
    !             lmax      : the maximum degree
    ! OUTPUT :    P         : the associated legendre function of the 1st kind
    !                         for degrees 0...lmax and order m=0,1
    !             dPdx      : the derivative of the associated legendre function
    ! 
    
    
    Implicit none
    ! INPUT
    Integer        ::  lmax
    Real(wp)       ::  x(:)
    Complex(wpc)   ::  P(size(x),0:lmax,0:1)
    Complex(wpc)   ::  dPdx(size(x),0:lmax,0:1)  
    ! Local
    Integer        ::  l,ix,m
    Real(wp)       ::  y,z
    

    !-----------------------
    ! Start loop over x
    !-----------------------
    Do ix = 1,size(x)
       y=x(ix)
       z=y**2-1
       
       !-----------------------
       ! Start loop over m
       !-----------------------
       Do m=0,1
          ! Formula (9.37) Bedeaux's book          
          ! Initial values for degree l=m and m+1
          ! Notice that we only need these functions for m=0,1
          Select case(m)
          Case(0) 
             P(ix,0,0)    = 1._wp   
             P(ix,1,0)    = y
             dPdx(ix,0,0) = 0._wp
          Case(1)
             P(ix,0,1)	= 0._wp      ! Strictly speaking P(l,m) is not defined for l<m, 
             dPdx(ix,0,1) = 0._wp		 ! but we define for convenience p(l,m)=0 
             ! in this case. 
             P(ix,1,1)	= -imu*sqrt(z)          
             If(lmax>1) P(ix,2,1)	= -imu*3*y*sqrt(z)          
             dPdx(ix,1,1) = -imu*y/sqrt(z)         
          Case default
             ! Error 
             write(*,*) 'ERROR : Illegal value for m in LegendreP !'
             stop
          End select
          
          ! Perfoms forward l-wise recurrence to obtain
          ! P(m,0,x),...,P(m,lmax,x)
          Do l=1+m,lmax-1
             P(ix,l+1,m)  = ( (2*l+1)*y*P(ix,l,m) - (l+m)*P(ix,l-1,m) )/(l-m+1)  
             dPdx(ix,l,m) = ( l*y*P(ix,l,m) - (l+m)*P(ix,l-1,m) )/z
          Enddo
          ! Calculates the top level for dPdx
          l	= lmax
          If(l>=m+1) & 
               dPdx(ix,l,m) = ( l*y*P(ix,l,m) - (l+m)*P(ix,l-1,m) )/z
          
       Enddo
       !-----------------------
       ! End loop over m
       !-----------------------
       
    Enddo
    !-----------------------
    ! End loop over x
    !-----------------------
    
  End Subroutine LegendreP_Real
  
  
  ! -----------------------------------------------------------
  ! Legendre Functions of Second Kind For Real Argument 1<x<inf
  ! -----------------------------------------------------------
  
  Subroutine LegendreQ_Real(Q,dQdx,x,lmax)
    ! Calculates the Associated Legendre function of the 2nd kind
    ! INPUT  :    x         : the argument
    !             lmax      : the maximum degree
    ! OUTPUT :    Q         : the associated legendre function of the 1st kind
    !                         for degrees 0...lmax and order m=0,1
    !             dQdx      : the derivative of the associated legendre function
    ! 
    
    Implicit none
    ! INPUT
    Integer        ::  lmax
    Real(wp)       ::  x(:)
    Complex(wpc)   ::  Q(size(x),0:lmax,0:1)
    Complex(wpc)   ::  dQdx(size(x),0:lmax,0:1)
    ! Local
    Integer        ::  l,ix,m
    Real(wp)       ::  y,z,w
    
    !-----------------------
    ! Start loop over x
    !-----------------------
    Do ix = 1,size(x)
       y=x(ix)
       z=y**2-1
       w=0.5*log((y+1)/(y-1))

       !-----------------------
       ! Start loop over m
       !-----------------------
       Do m=0,1
          
          ! Formula (9.38) Bedeaux's book
          ! Initial values for degree l=m and m+1
          ! Notice that we only need these functions for m=0,1
          Select case(m)
          Case(0) 
             Q(ix,0,0)	= w 
             Q(ix,1,0)	= y*w - 1
             dQdx(ix,0,0) = -1/z
          Case(1)
             Q(ix,0,1)	= 0._wp     ! Strictly speaking Q(l,m) is not defined for l<m, 
             dQdx(ix,0,1) = 0._wp		! but we define for convenience p(l,m)=0 
             ! in this case.  
             Q(ix,1,1)	= -w*sqrt(z)+y/sqrt(z)
             If(lmax>1) Q(ix,2,1)	= -3*y*sqrt(z)*w + (3*y**2-2)/sqrt(z)
             dQdx(ix,1,1) = -(w*y+(2-y**2)/z)/sqrt(z)
          Case default
             ! Error 
             write(*,*) 'ERROR : Illegal value for m in LegendreQ'
             stop
          End select
          
          ! Perfoms forward l-wise recurrence to obtain
          ! Q(m,0,x),...,Q(m,lmax,x)
          ! This forward recurrence is stable only for m=0,1m
          ! REFERENCES  OLVER AND SMITH,J.COMPUT.PHYSICS,51(1983),N0.3,502-518
          Do l=1+m,lmax-1
             Q(ix,l+1,m) = ((2*l+1)*y*Q(ix,l,m) - (l+m)*Q(ix,l-1,m))/(l-m+1)  
             dQdx(ix,l,m) = ( l*y*Q(ix,l,m) - (l+m)*Q(ix,l-1,m) )/z 
          Enddo
          ! Calculates the top level for dQdx 
          l	= lmax 
          If(l>=m+1) & 
               dQdx(ix,l,m) = ( l*y*Q(ix,l,m) - (l+m)*Q(ix,l-1,m) )/z
          
       Enddo
       !-----------------------
       ! End loop over m
       !-----------------------
       
    Enddo
    !-----------------------
    ! End loop over x
    !-----------------------
    
  End Subroutine LegendreQ_Real
  
End Module Prolate_Int_Gauleg




