
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: prolate_int_quadpack.f90,v 1.1 2001/07/29 12:46:08 ingves Exp ingves $
!---------------------------------------------------------------------------------




Module Prolate_Int_Quadpack

  Use Global_definitions
  Use Legendre_mod, only	: Quadpack,LegendreP_R,LegendredP_R

Contains


  !-----------------------------------------------------------------------------

  Subroutine Get_integrals_prolate_Quadpack(Int,param)
   
    Implicit none
    ! Input
    Type(parameters)    ::  param
    Type(Integrals)     ::  Int(0:,0:,0:)   
    ! Local
	Integer         ::  l1,l2,m,mpo
	Real(wp)        ::  tr,doa,xi0

	Common /index/ l1,l2,m
	Common /shape/ xi0,doa

    ! Some abbreviations
    mpo	   =  param%Mpole_order
    tr	   =  param%tr
    doa	   =  Abs(param%div_surface)/param%a
    xi0	   =  param%xi0
    
    !----------------------------------------------
    !---  Calculates the Integrals
    !----------------------------------------------    
    ! Loops over all "quantum numbers"
     Do l1=0,mpo
        Do l2=0,mpo
           Do m=0,1	

              !------------------------------------------------
              ! --- Q-integrals 
              !------------------------------------------------
              Call Quadpack(Int_Q,tr,Int(l1,l2,m)%Q,param)
			  			
              !------------------------------------------------
              ! --- V-integrals 
              !------------------------------------------------         
              Call Quadpack(Int_V,tr,Int(l1,l2,m)%V,param)

              !------------------------------------------------
              ! --- W-integrals 
              !------------------------------------------------
              Call Quadpack(Int_W,tr,Int(l1,l2,m)%W,param)	

              !------------------------------------------------
              ! --- dV-integrals 
              !------------------------------------------------
              Call Quadpack(Int_dV,tr,Int(l1,l2,m)%dVdx,param)
			  	
              !------------------------------------------------
              ! --- DW-integrals 
              !------------------------------------------------
              Call Quadpack(Int_dW,tr,Int(l1,l2,m)%dWdx,param)

           Enddo
         Enddo
      Enddo


  End Subroutine Get_integrals_prolate_Quadpack

!--------------------------------------------------------------------

  Function Int_Q(eta)	Result(Q_integrand)
    
    Implicit None
    Real(wp)             ::  Q_integrand
    Real(wp),Intent(in)	 ::  eta
    Integer              ::  l1,l2,m
    
    Common /index/ l1,l2,m
    
    Q_integrand = LegendreP_R(eta,l1,m)*LegendreP_R(eta,l2,m)
    Return
    
  End Function Int_Q

  !--------------------------------------------------------------------

  Function Int_V(eta)      Result(V_integrand)
    
    Implicit None
    Real(wp)             ::  V_integrand
    Real(wp),Intent(in)  ::  eta
    Integer              ::  l1,l2,m
    Real(wp)             ::  xi0,doa
    Real(wp)             ::  xi_trans,eta_trans
    
    Common /index/ l1,l2,m
    Common /shape/ xi0,doa
    
    Call xi_eta_trans_prolate(xi0,eta,doa,xi_trans,eta_trans)
    V_integrand  =  LegendreP_R(eta,l1,m)*LegendreP_R(eta_trans,l2,m)*  &
         Z_prolate(xi_trans,l2,m)
    Return
    
  End Function Int_V

  !--------------------------------------------------------------------
	
  Function Int_W(eta)     Result(W_integrand)
    
    Implicit None
    Real(wp)              ::  W_integrand
    Real(wp),Intent(in)   ::  eta
    Integer               ::  l1,l2,m
    Real(wp)              ::  xi0,doa
    Real(wp)              ::  xi_trans,eta_trans
    
    Common /index/ l1,l2,m
    Common /shape/ xi0,doa
    
    Call xi_eta_trans_prolate(xi0,eta,doa,xi_trans,eta_trans)
    W_integrand  =  LegendreP_R(eta,l1,m)*LegendreP_R(eta_trans,l2,m)*  &
         X_prolate(xi_trans,l2,m)
    Return
    
  End Function Int_W
  
  !--------------------------------------------------------------------
	
  Function Int_dV(eta)    Result(dV_integrand)

    Implicit None
    Real(wp)              ::  dV_integrand
    Real(wp),Intent(in)	  ::   eta
    Integer               ::  l1,l2,m
    Real(wp)              ::   xi0,doa
    Real(wp)              ::   xi_trans,eta_trans
    Real(wp)              ::   deriv_xi_trans,deriv_eta_trans
    
    Common /index/ l1,l2,m
    Common /shape/ xi0,doa
    
    Call xi_eta_trans_prolate(xi0,eta,doa,xi_trans,eta_trans)
    Call deriv_xi_eta_trans_prolate(xi0,eta,doa,deriv_xi_trans,deriv_eta_trans)
    
    dV_integrand   =  LegendreP_R(eta,l1,m)* &
         ( deriv_eta_trans*LegendredP_R(eta_trans,l2,m)*Z_prolate(xi_trans,l2,m) + &
         deriv_xi_trans*LegendreP_R(eta_trans,l2,m)*dZ_prolate(xi_trans,l2,m) )
    Return
    
  End Function Int_dV
  
  !--------------------------------------------------------------------
	
  Function Int_dW(eta)    Result(dW_integrand)

    Implicit None
    Real(wp)               ::  dW_integrand
    Real(wp),Intent(in)    ::  eta
    Integer                ::  l1,l2,m
    Real(wp)               ::  xi0,doa
    Real(wp)               ::  xi_trans,eta_trans
    Real(wp)               ::  deriv_xi_trans,deriv_eta_trans
    
    Common /index/ l1,l2,m
    Common /shape/ xi0,doa
    
    Call xi_eta_trans_prolate(xi0,eta,doa,xi_trans,eta_trans)
    Call deriv_xi_eta_trans_prolate(xi0,eta,doa,deriv_xi_trans,deriv_eta_trans)
    
    dW_integrand   =  LegendreP_R(eta,l1,m)* &
         ( deriv_eta_trans*LegendredP_R(eta_trans,l2,m)*X_prolate(xi_trans,l2,m) + &
         deriv_xi_trans*LegendreP_R(eta_trans,l2,m)*dX_prolate(xi_trans,l2,m) )
    Return
    
  End Function Int_dW




  ! ----------------------------------------------------------
  ! --- Routines for the transformed coordinates
  ! ----------------------------------------------------------

  Subroutine xi_eta_trans_prolate(xi,eta,doa,xi_trans,eta_trans)

    ! Transformed coordinates of the image point for the prolate case
    Implicit None
    ! Input
    Real(wp)       ::   xi,eta
    Real(wp)       ::   doa
    ! Output
    Real(wp)       ::   xi_trans,eta_trans
    ! Local
    Real(wp)       ::   R,tmp
    
    R            =  doa/xi
    tmp          =  1._wp+4*R**2-4*R*eta+(eta/xi)**2
    tmp          =  tmp + sqrt( tmp**2 - (2*(2*R-eta)/xi)**2 )
    tmp          =  Sqrt(tmp)
    xi_trans     =  xi*tmp/Sqrt(2._wp)
    eta_trans    =  Sqrt(2._wp)*(eta-2*R)/tmp
    
    
  End Subroutine xi_eta_trans_prolate
  
  !-----------------------------------------------------------------

  Subroutine deriv_xi_eta_trans_prolate(xi,eta,doa,deriv_xi_trans,deriv_eta_trans)

    Implicit None
    ! Input
    Real(wp)        ::  xi,eta
    Real(wp)        ::  doa
    ! Output
    Real(wp)        ::  deriv_xi_trans,deriv_eta_trans
    ! Local
    Real(wp)        ::  tmp(10),R
    
    
    R          =   doa/xi
    tmp(1)     =   1._wp+4*R**2-4*R*eta+(eta/xi)**2
    tmp(2)     =   2*(2*R-eta)/xi
    tmp(3)     =   -8*R**3/doa+4*R**2*eta/doa-2*eta**2/(xi**3)	
    tmp(4)     =   -8*doa/(xi**3)+2*eta/(xi**2)
    tmp(5)     =   Sqrt(tmp(1)**2-tmp(2)**2)
    tmp(6)     =   tmp(1)*tmp(3)-tmp(2)*tmp(4)
    tmp(7)     =   Sqrt(tmp(1)+tmp(5))
    tmp(8)     =   (tmp(3)+tmp(6)/tmp(5))/(2*tmp(7))
    tmp(9)     =   tmp(7)**3
    tmp(10)    =   (tmp(3)+tmp(6)/tmp(5))/(2*tmp(9))
    
    deriv_xi_trans    =	tmp(7)+xi*tmp(8)
    deriv_xi_trans    =	deriv_xi_trans/Sqrt(2._wp)
    
    deriv_eta_trans   =	2*doa/(xi**2)/tmp(7)-(eta-2*R)*tmp(10)
    deriv_eta_trans   =	deriv_eta_trans*Sqrt(2._wp)
    
    
  End Subroutine deriv_xi_eta_trans_prolate
  
  
  ! ----------------------------------------------------------
  ! --- Routines for the XZ functions
  ! ----------------------------------------------------------
  
  Function X_Prolate(xi,l,m)    Result(X)
    
    
    Implicit None
    Real(wp)              ::  X
    Real(wp),Intent(in)   ::  xi
    Integer,Intent(in)    ::  l,m
        
    X = Real( imu**m*Xfactor_func(l,m)*LegendreP_Real(xi,l,m) )
    Return
    
  End Function X_Prolate
  
  !-------------------------------------------------------------------------
  Function Z_Prolate(xi,l,m)   Result(Z)
    
    
    Implicit None
    Real(wp)             ::  Z
    Real(wp),Intent(in)  ::  xi
    Integer,Intent(in)   ::  l,m
    
    Z = Real( Zfactor_func(l,m)*LegendreQ_Real(xi,l,m) )
    Return
    
  End Function Z_Prolate
  
  !-------------------------------------------------------------------------

  Function dX_Prolate(xi,l,m)   Result(dX)    
    
    Implicit None
    Real(wp)              ::  dX
    Real(wp),Intent(in)   ::  xi
    Integer,Intent(in)    ::  l,m
    
    dX = Real( imu**m*Xfactor_func(l,m)*LegendredP_Real(xi,l,m) )
    Return
    
  End Function dX_Prolate
  
  !-------------------------------------------------------------------------
  Function dZ_Prolate(xi,l,m)   Result(dZ)
    
    Implicit None
    Real(wp)            ::  dZ
    Real(wp),Intent(in) ::  xi
    Integer             ::  l,m
    
    dZ = Real( Zfactor_func(l,m)*LegendredQ_Real(xi,l,m) )
    Return
    
  End Function dZ_Prolate
  
  !-------------------------------------------------------------------------

  Function XFactor_Func(l,m)   Result(factor)
    ! Factorial prefactor of X function
    
    Implicit None
    Integer     ::  l,m
    Integer     ::  i
    Real(wp)    ::  factor
    
    factor  = 1._wp
    If(m==1.and.l/=0) Then
       factor  =  factor/l
    Endif
    Do i=1,l
       factor =	factor*i/(2*i-1._wp)
    Enddo
    
  End Function XFactor_Func
  
  !-------------------------------------------------------------------------

  Function ZFactor_Func(l,m)    Result(factor)
    ! Factorial prefactor of Z function
    
    Implicit None
    Integer     ::  l,m
    Integer     ::  i
    Real(wp)    ::  factor
    
    factor	= 1._wp
    If(m==1) Then
       factor=factor/(l+1)
    Endif
    Do i=1,l
       factor =	factor*(2*i+1._wp)/i
    Enddo
    
  End Function ZFactor_Func
  
  
  
  
  


  ! ---------------------------------------------------------------
  ! Legendre Functions of First Kind For Purely Real Argument
  ! ---------------------------------------------------------------
    
  Function LegendreP_Real(x,l,m)   Result(P)
    ! Calculates the Associated Legendre function of the 1st kind 
    ! for purely imaginary input (Eq 9.37 Bedeaux's book)
    ! INPUT  :    x         : the argument
    !             l,m       : the degree
    ! OUTPUT :    P         : the associated legendre function of the 1st kind
    
    Implicit none
    ! INPUT
    Integer,Intent(in)    ::  l,m
    Real(wp),Intent(in)   ::  x
    Complex(wpc)          ::  P
    ! Local
    Real(wp)              ::  z
    Complex(wpc)          ::  P1,P2,P3
    Integer               ::  i
    
    ! First terms
    z=x**2-1
    Select case(m)
    Case(0) 
       P1    = 1._wp   
       P2    = x
    Case(1)
       P1    = -imu*sqrt(z)          
       P2    = -3*imu*x*sqrt(z)                   
    End select
    ! Get out for the first values
    If(l-1<1+m) Then
       Select Case(m)
       Case(0)
	  If(l==0) P = P1
	  If(l==1) P = P2
       Case(1)
	  If(l==0) P = 0._wp
	  If(l==1) P = P1
	  If(l==2) P = P2
       End Select
       Return
    Endif
    ! Perfoms forward l-wise recurrence to obtain
    ! P(l,m)(x)
    Do i=1+m,l-1
       P3  = ( (2*i+1)*x*P2 - (i+m)*P1 )/(i-m+1)
       P1  = P2
       P2  = P3    
    Enddo
    P = P3
    Return
    
  End Function LegendreP_Real
  
  !-------------------------------------------------------------------------
  
  Function LegendredP_Real(x,l,m)    Result(dPdx)
    
    ! Calculates the derivate of the associated Legendre function of the 1st kind 
    ! for purely real input (Eq 9.37)
    ! INPUT  :    x         : the argument
    !             l,m       : the degree
    ! OUTPUT :    dPdx      : the associated legendre function of 
    !                         the 1st kind
    
    Implicit None
    ! Global
    Real(wp),Intent(in)   ::  x
    Integer, Intent(in)	  ::  l,m
    Complex(wpc)          ::  dPdx
    ! Local
    Complex(wpc)          ::  P2,P1
    
    ! First terms	
    If(l-1<m) Then
       SelectCase(m)
       Case(0)
	  If(l==0) dPdx = 0._wp
       Case(1)
	  If(l==0) dPdx = 0._wp
	  If(l==1) dPdx = -imu*x/sqrt(x**2-1)
       End Select
       Return
    Endif
    ! The formula
    P2 = LegendreP_Real(x,l,m)
    P1 = LegendreP_Real(x,l-1,m)
    dPdx = ( l*x*P2 - (l+m)*P1 )/(x**2-1)
    Return 
    
  End Function LegendredP_Real
  
  
  ! ---------------------------------------------------------------
  ! Legendre Functions of Second Kind For Purely Real Argument
  ! ---------------------------------------------------------------
  
  Function LegendreQ_Real(x,l,m)   Result(Q)
    ! Calculates the Associated Legendre function of the 2nd kind 
    ! for purely real input (Eqs 9.38 Bedeaux's book)
    ! INPUT  :    x         : the argument
    !             l,m       : the degree
    ! OUTPUT :    Q         : the associated legendre function of the 2nd kind
    
    Implicit none
    ! INPUT
    Integer          ::  l,m
    Real(wp)         ::  x
    Complex(wpc)     ::  Q
    ! Local
    Real(wp)         ::  z,w
    Complex(wpc)     ::  Q1,Q2,Q3
    Integer          ::  i
    
    ! First terms
    z=x**2-1
    w=0.5*log((x+1)/(x-1))
    Select case(m)
    Case(0) 
       Q1    = w    
       Q2    = x*w - 1
    Case(1)
       Q1    = -w*sqrt(z)+x/sqrt(z)         
       Q2    = -3*x*sqrt(z)*w + (3*x**2-2)/sqrt(z)                 
    End select
    ! Get out for the first values
    If(l-1<1+m) Then
       Select Case(m)
       Case(0)
	  If(l==0) Q = Q1
	  If(l==1) Q = Q2
       Case(1)
	  If(l==0) Q = 0._wp
	  If(l==1) Q = Q1
	  If(l==2) Q = Q2
       End Select
       Return
    Endif
    ! Perfoms forward l-wise recurrence to obtain
    ! P(l,m)(x)
    Do i=1+m,l-1
       Q3  = ( (2*i+1)*x*Q2 - (i+m)*Q1 )/(i-m+1)
       Q1  = Q2
       Q2  = Q3    
    Enddo
    Q = Q3
    Return
    
  End Function LegendreQ_Real
  
  !-------------------------------------------------------------------------
	
  Function LegendredQ_Real(x,l,m)   Result(dQdx)
    
    ! Calculates the derivate of the associated Legendre function of 
    ! the 2nd kind 
    ! for purely real input (Eqs 9.38 Bedeaux's book)
    ! INPUT  :    x         : the argument
    !             l,m       : the degree
    ! OUTPUT :    Q         : the associated legendre function of the 2nd kind
    
    Implicit None
    ! Global
    Real(wp),Intent(in)       ::  x
    Integer, Intent(in)	      ::  l,m
    Complex(wpc)              ::  dQdx
    ! Local
    Complex(wpc)              ::  Q2,Q1
    Real(wp)                  ::  z,w
    
    
    ! First terms
    z=x**2-1
    w=0.5*log((x+1)/(x-1))
    If(l-1<m) Then
       SelectCase(m)
       Case(0)
	  If(l==0) dQdx = -1/z
       Case(1)
	  If(l==0) dQdx = 0._wp
	  If(l==1) dQdx = -(w*x+(2-x**2)/z)/sqrt(z) 
       End Select
       Return
    Endif
    ! The formula
    Q2 = LegendreQ_Real(x,l,m)
    Q1 = LegendreQ_Real(x,l-1,m)
    dQdx = ( l*x*Q2 - (l+m)*Q1 )/z
    Return 
    
  End Function LegendredQ_Real
	  


End Module Prolate_Int_Quadpack
 





