
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: prolate_mod.f90,v 1.1 2001/07/29 12:46:39 ingves Exp ingves $
!---------------------------------------------------------------------------------



Module Prolate_mod


  Use Global_definitions

  !---------------------------------------------------------------------------------------
  !
  ! Module containing the matrix elements used for the prolate spheroid
  !
  !---------------------------------------------------------------------------------------

Contains

!---------------------------------------------------------------------

  Subroutine get_linear_system_prolate(A,b,Ienergy,param,int,XZ,zeta)
    ! NOTICE :
    ! This routine sets up the reduced dimensionless linear system 
    ! for the determination of the multipole coeficients.
    Implicit None
    Type(parameters)       :: param       
    Type(integrals)        :: Int(0:,0:,0:)
    Type(XZ_Type)          :: XZ(0:,0:)    
    Integer                :: Ienergy
    Complex(wpc)           :: A(:,:,0:), b(:,0:)
    Real(wp)               :: zeta(0:,0:,0:)

    If (param%above) Then
       ! The center of the spheroid located ABOVE substrate
       Call get_linear_system_prolate_above( A,b,Ienergy,param,int,XZ,zeta )
    Else
       ! The center of the spheroid located BELOW substrate 
       Call get_linear_system_prolate_below( A,b,Ienergy,param,int,XZ,zeta )
    End If

  End Subroutine get_linear_system_prolate


  !--------------------------------------------------------------------------

  Subroutine get_linear_system_prolate_above(A,b,Ienergy,param,int,XZ,zeta)
    ! NOTICE :
    ! This routine sets up the reduced dimensionless linear system 
    ! for the determination of the multipole coeficients, when 
    ! the multipoles are located ABOVE the substrate. 
    Implicit None
    Type(parameters)       :: param       
    Type(integrals)        :: Int(0:,0:,0:)
    Type(XZ_Type)          :: XZ(0:,0:)    
    Integer                :: Ienergy
    Complex(wpc)           :: A(:,:,0:), b(:,0:)
    Integer                :: mpo,m,l,l1
    Real(wp)               :: zeta(0:,0:,0:), xi0, inv_xi0
    Real(wp)               :: tr,sin0,cos0,exp0,kronl
    Complex(wpc)           :: e1,e2,e3,e4,eps(4)

 

    ! Some initial abrevations
    tr      = param%tr
    sin0    = Sin(param%theta0_calc)
    cos0    = Cos(param%theta0_calc)
    exp0    = Exp(-imu*param%phi0_calc)
    mpo     = param%Mpole_order 
    xi0     = param%xi0
    inv_xi0 = 1._wp/xi0
    
    ! The dieletctric constants
    e1  = param%eps_vacuum 
    e2  = param%eps_substrate(ienergy)
    e3  = param%eps_island(ienergy)
    e4  = e2
    
    Do m=0,1
       Do l=1,mpo
          
          ! Calculated the kronecker delta
          If (l==1) Then
             kronl=1._wp
          Else
             kronl=0._wp
          Endif
          
          !------------------------------------------------------------
          !---------  Generates the right hand side; Eq. (9.64)
          !------------------------------------------------------------
          Select Case(m)
           Case(0)
              !--- H
              b(l,0) = Sqrt(4*pi/3)*cos0                                           &
                       *( e1/e2*inv_xi0*XZ(1,0)%X*kronl                            &
                          +( (e1-e2)/e2)                                           &
                             *(Sqrt(3._wp)*tr*zeta(l,0,0)*Int(l,0,0)%Q             &
                                    -inv_xi0*XZ(1,0)%X*zeta(l,1,0)*Int(l,1,0)%Q )  &
                        )
              !--- J 
              b(mpo+l,0) = Sqrt(4*pi/3)*cos0*e1*XZ(1,0)%dXdxi*kronl   

           Case(1)
              !--- H 
              b(l,1) = -Sqrt(2*pi/3)*sin0*exp0*inv_xi0*XZ(1,1)%X*kronl       
              !--- J
              b(mpo+l,1) = -Sqrt(2*pi/3)*sin0*exp0*XZ(1,1)%dXdxi*   &
                              ( e2*kronl                              &
                                 +(e1-e2)*zeta(l,1,1)*Int(l,1,1)%Q  )  
           End Select
           !------------------------------------------------------------
           !---------  Generates the left hand side; Eq (9.55)
           !------------------------------------------------------------
           Do l1=1,mpo

              ! Calculated the kronecker delta
              If (l==l1) Then
                 kronl=1._wp
              Else
                 kronl=0._wp
              Endif         

              eps  = (/  (e1-e2),2*e1,2*e1*e2,e1*(e1-e2) /) /(e1+e2) 
              ! --- C
              A(l,l1,m)     =   eps(2)*xi0**(l1+1)*XZ(l1,m)%Z*kronl      &
                  - eps(1)*zeta(l,l1,m)*xi0**(l1+1)*                     &
                      ( XZ(l1,m)%Z*Int(l,l1,m)%Q-(-1._wp)**(l1+m)*Int(l,l1,m)%V )

              ! --- F
              A(mpo+l,l1,m) = eps(3)*xi0**(l1+2)*XZ(l1,m)%dZdxi*kronl    &
                 + eps(4)*zeta(l,l1,m)*xi0**(l1+2)*                      &
                    ( XZ(l1,m)%dZdxi*Int(l,l1,m)%Q+(-1._wp)**(l1+m)*Int(l,l1,m)%dVdx )

              eps = (/ (e3 -e4),2*e3,2*e3*e4,e3*(e3-e4) /) /(e3+e4)
              ! --- D
              A(l,mpo+l1,m) =  -eps(2)*inv_xi0**l1*XZ(l1,m)%X*kronl       &
                 +eps(1)*zeta(l,l1,m)*inv_xi0**l1*                        &
                   ( XZ(l1,m)%X*Int(l,l1,m)%Q - (-1._wp)**(l1+m)*Int(l,l1,m)%W )

              ! --- G
              A(mpo+l,mpo+l1,m) = -eps(3)*inv_xi0**(l1-1)*XZ(l1,m)%dXdxi*kronl  &
                  - eps(4)*zeta(l,l1,m)*inv_xi0**(l1-1)*                        &
                      ( XZ(l1,m)%dXdxi*Int(l,l1,m)%Q+(-1._wp)**(l1+m)*Int(l,l1,m)%dWdx )  

           Enddo   ! End loop over l1
        Enddo      ! End loop over l
     Enddo         ! End loop over m

     Return
   End Subroutine get_linear_system_prolate_above

  !--------------------------------------------------------------------------

  Subroutine get_linear_system_prolate_below(A,b,Ienergy,param,int,XZ,zeta)
    ! NOTICE :
    ! This routine sets up the reduced dimensionless linear system 
    ! for the determination of the multipole coeficients, when 
    ! the multipoles are located BELOW the substrate. 
    Implicit None
    Type(parameters)       :: param       
    Type(integrals)        :: Int(0:,0:,0:)
    Type(XZ_Type)          :: XZ(0:,0:)    
    Integer                :: Ienergy
    Complex(wpc)           :: A(:,:,0:), b(:,0:)
    Integer                :: mpo,m,l,l1
    Real(wp)               :: zeta(0:,0:,0:), xi0, inv_xi0, factor
    Real(wp)               :: tr,sin0,cos0,exp0,kronl
    Complex(wpc)           :: e1,e2,e3,e4,eps(4)



    ! Some initial abrevations
    tr      = param%tr
    sin0    = Sin(param%theta0_calc)
    cos0    = Cos(param%theta0_calc)
    exp0    = Exp(-imu*param%phi0_calc)
    mpo     = param%Mpole_order 
    xi0     = param%xi0
    inv_xi0 = 1._wp/xi0
    
    ! The dieletctric constants
    e1  = param%eps_vacuum 
    e2  = param%eps_substrate(ienergy)
    e3  = param%eps_island(ienergy)
    e4  = e2
    
    Do m=0,1
       Do l=1,mpo
          
          ! Calculated the kronecker delta
          If (l==1) Then
             kronl=1._wp
          Else
             kronl=0._wp
          Endif
          
          !------------------------------------------------------------
          !---------  Generates the right hand side; Eq. (9.78)
          !------------------------------------------------------------

          factor = (-1._wp)**(l+1)

          Select Case(m)
           Case(0)
              !--- H
              b(l,0) = Sqrt(4*pi/3)*cos0                                           &
                       *(inv_xi0*XZ(1,0)%X*kronl                                   &
                          +( (e2-e1)/e2)*factor                                    &
                             *(Sqrt(3._wp)*tr*zeta(l,0,0)*Int(l,0,0)%Q             &
                                    -inv_xi0*XZ(1,0)%X*zeta(l,1,0)*Int(l,1,0)%Q )  &
                        )
              !--- J 
              b(mpo+l,0) = Sqrt(4*pi/3)*cos0*e1*XZ(1,0)%dXdxi*kronl   

           Case(1)
              !--- H 
              b(l,1) = -Sqrt(2*pi/3)*sin0*exp0*inv_xi0*XZ(1,1)%X*kronl       
              !--- J
              b(mpo+l,1) = -Sqrt(2*pi/3)*sin0*exp0*XZ(1,1)%dXdxi*                  &
                              ( e1*kronl+(e2-e1)*factor*zeta(l,1,1)*Int(l,1,1)%Q  )  
           End Select
           !------------------------------------------------------------
           !---------  Generates the left hand side; Eq (9.74)
           !------------------------------------------------------------
           Do l1=1,mpo

              ! Calculated the kronecker delta
              If (l==l1) Then
                 kronl=1._wp
              Else
                 kronl=0._wp
              Endif         

              factor = (-1._wp)**(l+l1)

              eps  = (/  (e2-e1),2*e2,2*e2*e1,e2*(e2-e1) /) /(e2+e1) 
              ! --- C
              A(l,l1,m)     =   eps(2)*xi0**(l1+1)*XZ(l1,m)%Z*kronl      &
                  - eps(1)*zeta(l,l1,m)*xi0**(l1+1)*factor*              &
                      ( XZ(l1,m)%Z*Int(l,l1,m)%Q-(-1._wp)**(l1+m)*Int(l,l1,m)%V )

              ! --- F
              A(mpo+l,l1,m) = eps(3)*xi0**(l1+2)*XZ(l1,m)%dZdxi*kronl    &
                 + eps(4)*zeta(l,l1,m)*xi0**(l1+2)*factor*               &
                    ( XZ(l1,m)%dZdxi*Int(l,l1,m)%Q+(-1._wp)**(l1+m)*Int(l,l1,m)%dVdx )


              eps = (/ (e4-e3),2*e4,2*e4*e3,e4*(e4-e3) /) /(e4+e3)
              ! --- D
              A(l,mpo+l1,m) =  -eps(2)*inv_xi0**l1*XZ(l1,m)%X*kronl       &
                 +eps(1)*zeta(l,l1,m)*inv_xi0**l1*factor*                 &
                   ( XZ(l1,m)%X*Int(l,l1,m)%Q - (-1._wp)**(l1+m)*Int(l,l1,m)%W )

              ! --- G
              A(mpo+l,mpo+l1,m) = -eps(3)*inv_xi0**(l1-1)*XZ(l1,m)%dXdxi*kronl  &
                  - eps(4)*zeta(l,l1,m)*inv_xi0**(l1-1)*factor*                 &
                      ( XZ(l1,m)%dXdxi*Int(l,l1,m)%Q+(-1._wp)**(l1+m)*Int(l,l1,m)%dWdx )  

           Enddo   ! End loop over l1
        Enddo      ! End loop over l
     Enddo         ! End loop over m

     Return
   End Subroutine get_linear_system_prolate_below



End Module Prolate_mod















