
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: spheroid.f90,v 1.1 2001/07/29 12:47:08 ingves Exp ingves $
!---------------------------------------------------------------------------------




!=================================================================
!		MAIN PROGRAM : SPHEROIDS OPTICAL RESPONSE
!================================================================


Program Spheroids


! Module used by the main program
  Use Global_definitions
  Use Initialize_mod,       only : initialize
  Use Oblate_mod,           only : get_linear_system_oblate 
  Use Prolate_mod,          only : get_linear_system_prolate
  Use Integral_mod,         only : Get_Int_XZ
  Use linsolver_mod,        only : linsolver
  Use Optics_mod,           only : Surf_Const_Coef_Island,Fresnel_calc, &
				       Surf_const_coef_island_Yamaguchi
  Use Yamaguchi_mod,        only : Yamaguchi
  Use Write_mod,            only : Write2screen2file
  Use Potential_mod,        only : potential
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! THIS LINES CAN BE SUPPRESSED SAFELY
  Use  Windows_Menu_Mod, only : Windows_Menu 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Global variables
  Implicit None
  Type(parameters)                     :: param    
  Type(integrals),Pointer              :: Int(:,:,:)
  Type(XZ_type), Pointer               :: XZ(:,:)
  Type(surface_constitutive), Pointer  :: surf_const_coef(:)
  Integer                              :: ienergy,m,i,itmp(1) 
  Real(wp), Pointer                    :: zeta(:,:,:), Fresnel_coeff(:)
  Complex(wpc), Pointer                :: A(:,:,:)
  Complex(wpc),Pointer                 :: bd(:,:)
  Complex(wpc),Pointer		       :: mpoled(:,:,:),mpoleq(:,:)
  Character(len=1)		       :: rep
  Integer			       :: dealloc


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! THIS LINES CAN BE SUPPRESSED SAFELY
  	Call Windows_Menu 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  ! ---------------------------------------------------------------------
  !	Initialize all the parameters and the pointers
  ! ---------------------------------------------------------------------
  ! Initialize
10 Call initialize( A,bd,param,Fresnel_coeff,mpoled,mpoleq, &
        Int,XZ,surf_const_coef,zeta )   
  itmp = minloc(abs(param%energy-param%BC_energy))
  param%BC_energy = param%energy(itmp(1))
  
  
  
  Select Case(param%geometry)
  Case('island')
     !***************************************************************************
     !*******          		TRUNCATED ISLAND SPHEROID MODEL 
     !***************************************************************************
     Write(6,*)
     Write(6,*) ' Setting up all the integrals ! Wait...'
     ! ---------------------------------------------------------------------
     !		Calculate the integrals and the special functions
     ! ---------------------------------------------------------------------
     ! Calculate the all integrals needed and the fuctions X_lm and Z_lm (and their derivatives)
     Call Get_Int_XZ(Int,XZ,param)
     
     
     !----------------------------------------------------------------------
     !		Energy Loop
     !----------------------------------------------------------------------
     Write(6,*) ' Evaluating the polarizabilites ! Wait...'
     
     
     Energy_loop :	 Do ienergy=1,param%Nenergy
        Write(6,'(a,f8.3)') '   Energy (eV) : ',param%energy(ienergy)
        !----------------------------------------------------------------------
        !	Get the multipoles coefficients
        !----------------------------------------------------------------------   
        ! Gets the left/right hand side (A/b) of the linear system         
        Select case(trim(adjustl(param%island_type)))
        Case('oblate')
           Call get_linear_system_oblate( A,bd,Ienergy,param,Int,XZ,zeta )        
        Case('prolate')
           Call get_linear_system_prolate( A,bd,Ienergy,param,Int,XZ,zeta )        
        Case default
           Write(6,*) 'Error in spheroids :param%island_type : ', &
                param%island_type,' not supported !'
           Pause
           stop
        End select
        ! Solve the linear system Ax=b
        Do m=0,1
           Call linsolver( A(:,:,m),bd(:,m),param%epslin )
        Enddo
        ! Store the dipole and quadropole coefficients
        mpoled(:,:,ienergy) = bd(1:2,:)
        
        !----------------------------------------------------------------------
        !	Output for the oscillators strenghts
        !----------------------------------------------------------------------
        ! Output of the oscillators strenght if necessary
        If(param%strenght) then
           Open(48,file='Multipole_Strenght.dat',status='unknown')
           Write(48,'(f10.5,8x,71(2f10.5,5x))')  &
                param%energy(ienergy), ( ( Real(bd(i,m)),Aimag(bd(i,m)),m=0,1 ) ,i=1,param%Mpole_order)
        Endif
        
        !----------------------------------------------------------------------
        !	Check the boundary conditions or calculate the potential 
        !       for an uniform incident field
        !----------------------------------------------------------------------
        ! For testing the boundary conditions or the potential
	If (ienergy==itmp(1)) then
           Select Case (param%bound)
           Case('none')
              Continue
           Case('Pot')
              Write(6,*) ' Calculating the potential ! Wait...'
              Write(6,'(a,f5.3,a)') &
                   '  Potential at : ',param%energy(ienergy),' eV'
              Call potential(bd(:,:),Int,XZ,ienergy,param)
           Case Default
              Write(6,*) 'Error in the choice of param%bound ! (',param%bound,')'
           End Select
	Endif
        
        
     Enddo Energy_loop
     
     !----------------------------------------------------------------------
     !	End Energy Loop
     !----------------------------------------------------------------------
     ! Free the unecessary allocated space
     Deallocate(A)
     Deallocate(bd)
     Deallocate(Int)
     Deallocate(XZ)
     Deallocate(zeta)
     
     !-----------------------------------------------------------------------
     !	Calculates the surface constitutive coefficients	
     !-----------------------------------------------------------------------
     Call surf_const_coef_island(surf_const_coef,mpoled,mpoleq,param)

     ! Free the unecessary allocated space 
     Deallocate(mpoled,mpoleq)
     
     
     
  Case('Yamaguchi','coated')
     !**************************************************************************
     !*******          		YAMAGUCHI SPHEROID MODEL 
     !**************************************************************************
     !----------------------------------------------------------------------
     !    Calculates the modified polarizabilities
     !----------------------------------------------------------------------
     Call Yamaguchi(param,mpoled)
     !----------------------------------------------------------------------
     !	 Calculates the optical properties of the layer
     !----------------------------------------------------------------------
     Call surf_const_coef_island_Yamaguchi(surf_const_coef,mpoled,param)
     ! Free the unecessary allocated space 
     Deallocate(mpoled)
     
     
  Case Default
     Write(*,*) ' Error in spheroid for param%geometry !'
     Pause
     stop
  End Select
  
  ! ---------------------------------------------------------------------
  !	Calculates the optical properties of the layer
  ! ---------------------------------------------------------------------
  Write(6,*) ' Calculating the Fresnel coefficient ! Wait...'
  Call Fresnel_calc(Fresnel_coeff,surf_const_coef,param)
  
  ! Free the unecessary allocated space 
  Deallocate(surf_const_coef)
  
  ! ---------------------------------------------------------------------
  !	Plots and writes the results
  ! ---------------------------------------------------------------------
  Write(6,*)
  Call Write2screen2file(Fresnel_coeff,param)
  
  ! ---------------------------------------------------------------------
  !     Frees the allocated space
  ! ---------------------------------------------------------------------
  Deallocate(param%energy)
  Deallocate(param%eps_substrate)
  Deallocate(param%eps_island)
  Deallocate(param%eps_coating,STAT=dealloc) 
  Deallocate(Fresnel_coeff)
  
  ! ---------------------------------------------------------------------
  !	Other calculation ?
  ! ---------------------------------------------------------------------
  
20 Write(6,*)
  Write(6,'(a\)') '  Other simulation (y-n) ?  '
  Read(5,*) rep
  Write(6,*)
  Select Case(rep)
  Case('y')
     Goto 10
  Case('n')
  Case Default
     Write(6,*) 'Band answer !'
     Goto 20
  End Select
  

End Program Spheroids






