
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: yamaguchi.f90,v 1.1 2001/07/29 12:50:05 ingves Exp ingves $
!---------------------------------------------------------------------------------



Module Yamaguchi_mod

  Use Global_definitions


Contains

  !-------------------------------------------------------------------
	
  Subroutine Yamaguchi(param,alpha)

    Implicit None
    Type(parameters)     ::  param
    Complex(wpc)         ::  alpha(2,2,param%Nenergy)
    Integer              ::  ienergy
    Real(wp)             ::  V


    ! Normailzed volume
    V	=	param%volume/(param%R**3)
    Select Case(param%geometry)
    Case('Yamaguchi')
       ! Simple Yamaguchi model with interaction with the substrate
       Call Spheroid_Yamaguchi(param,alpha)
    Case('coated')
       ! Coating of the spheroids
       Call Coating_Yamaguchi(param,alpha)
    Case Default
       Write(*,*) ' Error in Yamaguchi for param%geometry !! '
    End Select
    
    ! Put interaction between island to dipolar order to be
    ! compatible with optics_mod
    If(param%interaction=='quadrupole') param%interaction='dipole'
    
    ! Output for the polarizability
    If(param%polarizability) Then
       Open(31,file='Polarizability.dat',status='unknown')
       Do ienergy=1,param%Nenergy
          Write(31,'(f10.5,5x,2(2f13.5,8x))') &
               param%energy(ienergy),alpha(1,1,ienergy)/V,alpha(1,2,ienergy)/V
       Enddo
       Close(31)
    EndIf
    
  End Subroutine Yamaguchi
  
  !-------------------------------------------------------------------
	
  Subroutine Spheroid_Yamaguchi(param,alpha)
	
    !------------------------------------------------------
    ! Computes the polarizabilities of a spheroid in vacuum
    ! and correct it in the framework of Yamaguchi model
    ! up to dipolar or quadrupolar order
    ! Formulas of Chapter 6.2 of Bedeaux's book
    !------------------------------------------------------
    
    Implicit None
    Type(parameters)   ::  param
    Complex(wpc)       ::  alpha(2,2,param%Nenergy)
    Real(wp)           ::  Laz,Lap,Laz1,Lap1,La11
    Real(wp)           ::  e1,a,xi0,arctan,loga,volume,d
    Integer            ::  ienergy
    !,itmp(1)
    Complex(wpc)       ::  e2,e3,tmp(5),Dz,Dp,imgs


    ! Usefull abreviations
    xi0	  =  param%xi0
    a     =  param%a/param%R
    e1    =  param%eps_vacuum
    d     =  Abs(param%tr)

    ! Nature of the island type
    Select Case(param%island_type)
    Case('oblate')
       volume =	4*pi*a**3*xi0*(1+xi0**2)/3
       arctan = atan(1._wp/xi0)
       ! Depolarization factor (formulas 6.31)
       Laz	 = (1+xi0**2)*(1-xi0*arctan)
       Lap	 = (1+xi0**2)*(xi0*arctan-xi0**2/(1+xi0**2))/2 
       Laz1	 = 3*(1+xi0**2)*(xi0*(1+3*xi0**2)*arctan-3*xi0**2)/2
       Lap1	 = (1+2*xi0**2)*(2+3*xi0**2-3*(1+xi0**2)*xi0*arctan)/2 
       La11	 = (3*(1+xi0**2)**2*xi0*arctan-xi0**2*(5+3*xi0**2))/4
    Case('prolate')
       volume =	4*pi*a**3*xi0*(xi0**2-1)/3
       loga	 = log((xi0+1)/(xi0-1))/2
       ! Depolarization factor (formulas 6.35)
       Laz	 = (1-xi0**2)*(1-xi0*loga)
       Lap	 = (1-xi0**2)*(xi0**2/(1-xi0**2)+xi0*loga)/2 
       Laz1	 = 3*(1-xi0**2)*((1-3*xi0**2)*xi0*loga+3*xi0**2)/2
       Lap1	 = (1-2*xi0**2)*(-3*(1-xi0**2)*xi0*loga+2-3*xi0**2)/2 
       La11	 = (3*(1-xi0**2)**2*xi0*loga+xi0**2*(5-3*xi0**2))/4
    Case Default
       Write(*,*) 'Error in Spheroid_Yamaguchi for param%island_type'
    End Select
    
    ! Get the polarizabilities
    Do ienergy=1,param%Nenergy
       
       !Write(6,'(a,f8.3)') '   Energy (eV) : ',param%energy(ienergy)
       e3		=	param%eps_island(ienergy)
       e2		=	param%eps_substrate(ienergy)
       tmp(1)	=	e1*(e3-e1)*volume/(e1+Lap*(e3-e1)) ! Parallel
       tmp(2)	=	e1*(e3-e1)*volume/(e1+Laz*(e3-e1)) ! Perpendicular
       Select Case(param%island_type)
       Case('oblate')
          ! Formulas 6.30
          tmp(3)	=	e1*(e3-e1)*(1+3*xi0**2)*a**2*volume/5/(e1+Laz1*(e3-e1)) 
          tmp(4)	=	e1*(e3-e1)*(1+2*xi0**2)*a**2*volume/5/(e1+Lap1*(e3-e1)) 
          tmp(5)	=	e1*(e3-e1)*(1+xi0**2)*a**2*volume/3/(e1+La11*(e3-e1))
       Case('prolate')
          ! Formulas 6.77
          tmp(3)	=	e1*(e3-e1)*(3*xi0**2-1)*a**2*volume/5/(e1+Laz1*(e3-e1)) 
          tmp(4)	=	e1*(e3-e1)*(2*xi0**2-1)*a**2*volume/5/(e1+Lap1*(e3-e1)) 
          tmp(5)	=	e1*(e3-e1)*(xi0**2-1)*a**2*volume/3/(e1+La11*(e3-e1))
       End Select
       ! Strenght of image
       imgs		=	(e1-e2)/(e1+e2)/(32*pi*e1*d**3)
       ! Polarizability of the interacting island
       ! Calculates the (dimensionless) polarizabilities 
       ! 1st index : 1 : dipole order
       !             2 : quadropole order
       ! 2nd index : 1 : parallel
       !             2 : perpendicular
       Select Case(param%Mpole_order)
          ! No interaction with the substrate
       Case(0) ! formula 5.36 - 5.37 - 5.38
          alpha(1,1,ienergy)		=	tmp(1)
          alpha(1,2,ienergy)		=	tmp(2)
          alpha(2,1,ienergy)		=	0._wp
          alpha(2,2,ienergy)		=	0._wp
	   ! Dipolar interaction
       Case(1)  ! formula 5.61  -  5.62
          alpha(1,1,ienergy)		=	tmp(1)/(1+imgs*tmp(1))
          alpha(1,2,ienergy)		=	tmp(2)/(1+2*imgs*tmp(2))
          alpha(2,1,ienergy)		=	-d*tmp(1)/(1+imgs*tmp(1))
          alpha(2,2,ienergy)		=	-d*tmp(2)/(1+2*imgs*tmp(2))
          ! Quadrupolar interaction
       Case(2) ! formual 6.37  -   6.39
          Dz		=	1+2*imgs*tmp(2)+(3*imgs/(2*d**2))*(2+imgs*tmp(2))*tmp(3)
          Dp		=	1+imgs*tmp(1)+(3*imgs/(4*d**2))*(4+imgs*tmp(1))*tmp(4)
          ! Polarizabilities
          alpha(1,1,ienergy)		=	tmp(1)*(1+3*imgs/(d**2)*tmp(4))/Dp
          alpha(1,2,ienergy)		=	-3*imgs/d*tmp(1)*tmp(4)/2/Dp
          alpha(2,1,ienergy)		=	tmp(2)*(1+3*imgs/(d**2)*tmp(3))/Dz
          alpha(2,2,ienergy)		=	-3*imgs/d*tmp(2)*tmp(3)/2/Dz
       Case Default
          Write(*,*) ' Error in Spheroid_Yamaguchi for param%Mpole_order : ',param%Mpole_order
          Pause
          stop
       End Select
       
    Enddo
    !	  itmp(:) = maxloc(Aimag(alpha(1,1,:)))
    !	  write(6,*) param%energy(itmp(1))
    !	  itmp(:) = maxloc(Aimag(alpha(1,2,:)))
    !	  write(6,*) param%energy(itmp(1))
    
    
  End Subroutine Spheroid_Yamaguchi

!--------------------------------------------------------------------
  
  Subroutine Coating_Yamaguchi(param,alpha)

    !----------------------------------------------------------------------------
    ! All the problem of coated ellipsoid is treated at dipolar
    ! order
    ! Formulas of the book 'Absorption and Scattering of light by Small Particles'
    ! Bohren - Huffman     P  149
    !----------------------------------------------------------------------------
    
    Use Initialize_mod, only   :  dielectric_constants,dielectric_func_corrections
    Use Legendre_mod, only     :  Quadpack_Infinite
    Implicit None
    Type(parameters)   ::  param
    Complex(wpc)       ::  alpha(2,2,param%Nenergy)	
    Complex(wpc)       ::  tmp(2)
    Complex(wpc)       ::  e1,e2,e3,ec,imgs
    Real(wp)           ::  d,volume,R(2,2),L(2,2),t,f
    Real(wp)           ::  radius(3),x
    Integer            ::  i,j,ienergy
    
    ! Common for the integration scheme
    Common /axes/  radius,x
    
    ! Abreviations
    t			=	param%thickness/param%R
    e1			=	param%eps_vacuum
    d			=	Abs(param%tr)	
    
    ! Dimension less axes lenght
    ! 1st index : 1 : parallel
    !             2 : perpendicular
    ! 2nd index : 1 : inner shell
    !             2 : outer shell
    R(1,2)		=	param%R_par/param%R  
    R(1,1)		=	R(1,2) - t 
    R(2,2)		=	param%R_per/param%R 
    R(2,1)		=	R(2,2) - t 
    
    ! Get the dielectric constant for the coating
    Allocate(param%eps_coating(param%Nenergy))
    Call dielectric_constants(param%energy,param%eps_coating,param%coating,param%path)
    Call dielectric_func_corrections(param%eps_coating,param%coating,param%path,param)
    
    ! Volume and geometric factor
    Select Case(param%island_type)
    Case('oblate')
       volume		=	4*pi*R(1,2)**2*R(2,2)/3
       f		=	R(1,1)**2*R(2,1)/(R(1,2)**2*R(2,2))
       Do j=1,2
          ! inner - outer
          radius(:)	= (/R(1,j),R(1,j),R(2,j)/)
          Do i=1,2
             ! parallel - perpendicular
             x		= R(i,j)
             Call Quadpack_Infinite(Fk,0._wp,L(i,j),param)
          Enddo
       Enddo
       L(1,:)		=	L(1,:)*R(1,:)**2*R(2,:)/2
       L(2,:)		=	L(2,:)*R(1,:)**2*R(2,:)/2
    Case('prolate')
       volume		=	4*pi*R(2,2)**2*R(1,2)/3
       f			=	R(2,1)**2*R(1,1)/(R(2,2)**2*R(1,2))
       Do j=1,2
          ! inner - outer
          radius(:)	= (/R(1,j),R(2,j),R(2,j)/)
          Do i=1,2
             ! parallel - perpendicular
             x		= R(i,j)
             Call Quadpack_Infinite(Fk,0._wp,L(i,j),param)
          Enddo
       Enddo
       L(1,:)		=	L(1,:)*R(2,:)**2*R(1,:)/2
       L(2,:)		=	L(2,:)*R(2,:)**2*R(1,:)/2
    End Select
    
    ! Get the polarizabilities
    Do ienergy=1,param%Nenergy
       Write(6,'(a,f8.3)') '   Energy (eV) : ',param%energy(ienergy)
       ! Abreviations
       e2	=	param%eps_substrate(ienergy)
       e3	=	param%eps_island(ienergy)
       ec	=	param%eps_coating(ienergy)
       ! Strenght of image
       imgs		=	(e1-e2)/(e1+e2)/(32*pi*e1*d**3)
       ! Polarizabilty of an isolated coated island
       ! Parallel
       tmp(1)		=	(ec-e1)*(ec+(e3-ec)*(L(1,1)-f*L(1,2))) + f*ec*(e3-ec)
       tmp(1)		=	volume*tmp(1)/ &
            ( (ec + (e3-ec)*(L(1,1)-f*L(1,2)))*(e1+(ec-e1)*L(1,2)) + f*L(1,2)*ec*(e3-ec) )
       ! Perpendicular
       tmp(2)		=	(ec-e1)*(ec+(e3-ec)*(L(2,1)-f*L(2,2))) + f*ec*(e3-ec)
       tmp(2)		=	volume*tmp(2)/ &
            ( (ec + (e3-ec)*(L(2,1)-f*L(2,2)))*(e1+(ec-e1)*L(2,2)) + f*L(2,2)*ec*(e3-ec) )   	
       ! Polarizability of the interacting island
       ! Calculates the (dimensionless) polarizabilities 
       ! 1st index : 1 : dipole order
       !             2 : quadropole order
       ! 2nd index : 1 : parallel
       !             2 : perpendicular
       Select Case(param%Mpole_order)
       Case(0)
          alpha(1,1,ienergy)		=	tmp(1)
          alpha(1,2,ienergy)		=	tmp(2)
          alpha(2,1,ienergy)		=	0._wp
          alpha(2,2,ienergy)		=	0._wp
       Case(1)
          alpha(1,1,ienergy)		=	tmp(1)/(1+imgs*tmp(1))
          alpha(1,2,ienergy)		=	tmp(2)/(1+2*imgs*tmp(2))
          alpha(2,1,ienergy)		=	-d*tmp(1)/(1+imgs*tmp(1))
          alpha(2,2,ienergy)		=	-d*tmp(2)/(1+2*imgs*tmp(2))
       Case Default
          Write(*,*) ' Error in Coating_Yamaguchi for param%Mpole_order :',param%Mpole_order
          Pause
          stop
       End Select
       
    Enddo
    
    
    
  End Subroutine Coating_Yamaguchi
  
  !----------------------------------------------------------------------------------------
	
  Function Fk(q)	Result(F)

    ! Function to get the depolarization factor for a coated ellipsoid
    Implicit None
    Real(wp)       ::  a,b,c,x,q,F
    
    Common /axes/  a,b,c,x
    
    F = 1._wp/( (q+x**2)*sqrt((a**2+q)*(b**2+q)*(c**2+q)) )
    Return
    
  End Function Fk
  
  
End Module Yamaguchi_mod
